//----------------------------------------------------------------------------
//  Project Test_At3dx_Viewer
//  MicroMouse Productions
//  Copyright  2010-2020. All Rights Reserved.
//  Author: Wayne Hogue
//
//---------------------------------------------------------------------------

#ifndef Test_At3dx_LocationsAppH
#define Test_At3dx_LocationsAppH
//---------------------------------------------------------------------------
#include <System.Classes.hpp>
#include <Vcl.Controls.hpp>
#include <Vcl.StdCtrls.hpp>
#include <Vcl.Forms.hpp>
#include <Vcl.ExtCtrls.hpp>
#include <Vcl.Buttons.hpp>
#include <Vcl.ComCtrls.hpp>
//---------------------------------------------------------------------------

#if defined(_WIN64)
#else
  #define USING_WIN32
#endif

#define LEN_LAYER_NAME       80
#define LEN_FILE_PATH        MAXPATH

// used for measuring with rubberband line
//
#define MESH_LOCATION_NO_DATA   0    // no useful data in struct
#define MESH_LOCATION_NO_XYZ    1    // screen coordinates only
#define MESH_LOCATION_HAS_XYZ   2    // has xyz location on mesh

// used for storing measurements
// and used with functions
// At3d_MeasureList_Add_Item(...) and
// At3d_MeasureList_Get_Item(...)
//
typedef struct tag_MEASURE_DATA {
  char id[LEN_LAYER_NAME];
  //
  Int32 visible_Flag;
  Int32 observer_Flag;
  Int32 obstruction_Flag;
  Int32 spareFlag;               // padding for align
  //
  float lineWidth;
  float rgb_Line_Array[3];
  //
  double vertex1[3];
  double vertex2[3];
  double observer_DeltaXYZ[3];
  double obstruction[3];
  //
  // calculated variables
  //
  double observer[3];
  double distance;
  double obstruction_Distance;
  } MEASURE_DATA;


// used for billboards used with measurement lines
// and used with functions
// At3d_Measure_Billboard_Get_Item(...)
// At3d_Measure_Billboard_Replace_Item(...)
//
typedef struct tag_MEASURE_BILLBOARD {
  char id[LEN_LAYER_NAME];
  //
  UInt8* imagePtr;
  Int32 imageSize;
  Int32 imageWidth;
  Int32 imageHeight;
  Int32 alphaFlag;
  //
  Int32 pivotTypeFlag;
  Int32 staticFlag;
  Int32 visibleFlag;
  Int32 oglReplaceFlag;
  Int32 typeFlag;    // user defined value that the DLL only stores but does not use
  Int32 spareFlag;   // padding for align
  //
  float rgb_Line_Array[3];
  float lineWidth;
  //
#ifdef USING_WIN32
  Int32 w32_pad1;        // padding for align
#endif
  double height;
  double position[3];
  double pivot[3];
  double rotationStatic[3];
  } MEASURE_BILLBOARD;

//---------------------------------------------------------------------------

class TTest_At3dx_LocationsForm : public TForm
{
__published:  // IDE-managed Components
  TPanel *ViewPanel;
  TGroupBox *GroupBox23;
  TSpeedButton *ViewTop;
  TSpeedButton *ViewBack;
  TSpeedButton *ViewLeft;
  TSpeedButton *ViewFront;
  TSpeedButton *ViewRight;
  TSpeedButton *ViewBottom;
  TGroupBox *GroupBox19;
  TSpeedButton *GetCameraViewpoint;
  TSpeedButton *SetCameraViewpoint;
  TEdit *CameraXViewpoint;
  TEdit *CameraYViewpoint;
  TEdit *CameraZViewpoint;
  TGroupBox *GroupBox24;
  TSpeedButton *GetCameraFieldOfView;
  TSpeedButton *SetCameraFieldOfView;
  TSpeedButton *SetCameraFOV1;
  TSpeedButton *SetCameraFOV2;
  TEdit *CameraFieldOfView;
  TEdit *CameraStepFOV;
  TGroupBox *GroupBox20;
  TSpeedButton *GetCameraAngles;
  TSpeedButton *SetCameraAngles;
  TLabel *Label15;
  TSpeedButton *SetCameraRotateSpeed1;
  TSpeedButton *SetCameraRotateSpeed2;
  TSpeedButton *SetCameraXAngle1;
  TSpeedButton *SetCameraYAngle1;
  TSpeedButton *SetCameraZAngle1;
  TSpeedButton *SetCameraXAngle2;
  TSpeedButton *SetCameraYAngle2;
  TSpeedButton *SetCameraZAngle2;
  TEdit *CameraXAngle;
  TEdit *CameraYAngle;
  TEdit *CameraZAngle;
  TEdit *CameraRotateSpeed;
  TEdit *CameraStepAngle;
  TGroupBox *GroupBox21;
  TSpeedButton *SetCameraPan;
  TSpeedButton *SetCameraXPan1;
  TSpeedButton *SetCameraZPan1;
  TSpeedButton *SetCameraXPan2;
  TSpeedButton *SetCameraZPan2;
  TSpeedButton *ViewCenterX;
  TSpeedButton *ViewCenterZ;
  TEdit *CameraXPan;
  TEdit *CameraZPan;
  TEdit *CameraStepPan;
  TCheckBox *ReversePanDirection;
  TGroupBox *GroupBox22;
  TLabel *Label16;
  TLabel *Label17;
  TLabel *Label18;
  TSpeedButton *GetCameraInitialDistance;
  TSpeedButton *GetCameraDistance;
  TSpeedButton *GetCameraZoomSpeed;
  TSpeedButton *SetCameraInitialDistance;
  TSpeedButton *SetCameraDistance;
  TSpeedButton *SetCameraZoomSpeed1;
  TSpeedButton *SetCameraZoomSpeed2;
  TSpeedButton *SetCameraZoomSpeed3;
  TSpeedButton *SetCameraZoomSpeed4;
  TSpeedButton *SetCameraZoomSpeed5;
  TSpeedButton *SetCameraZoomSpeed6;
  TSpeedButton *ZoomIn;
  TSpeedButton *ZoomOut;
  TSpeedButton *ZoomReset;
  TEdit *CameraInitialDistance;
  TEdit *CameraDistance;
  TEdit *CameraZoomSpeed;
  TCheckBox *ReverseZoomDirection;
  TGroupBox *GroupBox14;
  TCheckBox *AlwaysShowCameraCursor;
  TCheckBox *TouchScreenInput;
  TCheckBox *UseTextures;
  TGroupBox *GroupBox30;
  TGroupBox *GroupBox31;
  TRadioButton *TypeWrl;
  TRadioButton *TypeObj;
  TSpeedButton *Read3dFile;
  TGroupBox *GroupBox32;
  TRadioButton *SetViewpoint_Middle;
  TRadioButton *SetViewpoint_Left;
  TGroupBox *GroupBox33;
  TRadioButton *LeftToggle_Rotate;
  TRadioButton *LeftToggle_Pan;
  TButton *MultiRead;
  TGroupBox *GroupBox42;
  TButton *SetViewpoint1;
  TButton *SetViewpoint2;
  TSpeedButton *ZoomAutomatic;
  TGroupBox *GroupBox45;
  TLabel *Label130;
  TSpeedButton *RotateX1Object;
  TSpeedButton *RotateX2Object;
  TSpeedButton *RotateX3Object;
  TLabel *Label131;
  TSpeedButton *RotateY1Object;
  TSpeedButton *RotateY2Object;
  TSpeedButton *RotateY3Object;
  TSpeedButton *RotateZ1Object;
  TSpeedButton *RotateZ2Object;
  TSpeedButton *RotateZ3Object;
  TLabel *Label132;
  TPageControl *PageControl1;
  TTabSheet *LocationsSheet;
  TTabSheet *CamerasSheet;
  TGroupBox *GroupBox103;
  TLabel *Label36;
  TLabel *ObstructionMessage;
  TGroupBox *GroupBox113;
  TGroupBox *GroupBox115;
  TLabel *Label170;
  TLabel *Label171;
  TLabel *Label172;
  TLabel *Location1_MeshX;
  TLabel *Location1_MeshY;
  TLabel *Location1_MeshZ;
  TGroupBox *GroupBox1;
  TLabel *Label1;
  TLabel *Label2;
  TLabel *Location1_ScreenX;
  TLabel *Location1_ScreenY;
  TGroupBox *GroupBox3;
  TGroupBox *GroupBox4;
  TLabel *Label3;
  TLabel *Label4;
  TLabel *Label5;
  TLabel *Location2_MeshX;
  TLabel *Location2_MeshY;
  TLabel *Location2_MeshZ;
  TGroupBox *GroupBox5;
  TLabel *Label9;
  TLabel *Label10;
  TLabel *Location2_ScreenX;
  TLabel *Location2_ScreenY;
  TButton *SwapEnds;
  TGroupBox *GroupBox8;
  TRadioButton *Locations_Off;
  TRadioButton *Locations_Measure;
  TRadioButton *Locations_Pause;
  TRadioButton *Locations_Continue;
  TGroupBox *GroupBox15;
  TLabel *Label21;
  TLabel *Label24;
  TLabel *Label25;
  TLabel *Label28;
  TLabel *Label29;
  TLabel *Label30;
  TPanel *Vertex2_Panel;
  TPanel *Vertex1_Panel;
  TPanel *Line_Panel;
  TEdit *LineWidth;
  TEdit *PointSize;
  TCheckBox *UseDepthTest;
  TCheckBox *DisplayVertices;
  TButton *SetRubberbandParameters;
  TGroupBox *GroupBox16;
  TLabel *Label26;
  TLabel *Label31;
  TLabel *Label32;
  TLabel *Label37;
  TPanel *Obstruction_Panel;
  TEdit *ObstructionPointSize;
  TCheckBox *DisplayObstructionVertex;
  TButton *SetObstructionParameters;
  TEdit *ObstructionSkipDistance;
  TGroupBox *GroupBox17;
  TGroupBox *GroupBox18;
  TLabel *Label33;
  TLabel *Label34;
  TLabel *Label35;
  TLabel *Obstruction_MeshX;
  TLabel *Obstruction_MeshY;
  TLabel *Obstruction_MeshZ;
  TGroupBox *GroupBox25;
  TLabel *Label39;
  TLabel *Label40;
  TLabel *Obstruction_ScreenX;
  TLabel *Obstruction_ScreenY;
  TGroupBox *GroupBox27;
  TButton *ClipAtObstructionPoint;
  TButton *CheckForObstruction;
  TButton *UpdateScreenXY;
  TButton *UpdateObstruction;
  TGroupBox *GroupBox36;
  TGroupBox *GroupBox37;
  TLabel *Label42;
  TLabel *Label43;
  TLabel *Label45;
  TLabel *Label49;
  TLabel *Label50;
  TPanel *ObserverVertex_Panel;
  TPanel *ObserverLine_Panel;
  TEdit *ObserverLineWidth;
  TEdit *ObserverPointSize;
  TCheckBox *DisplayObserverLine;
  TCheckBox *DisplayObserverVertex;
  TButton *SetObserverParameters;
  TGroupBox *GroupBox38;
  TLabel *Label51;
  TLabel *Label52;
  TLabel *Label53;
  TEdit *Observer_DeltaX;
  TEdit *Observer_DeltaY;
  TEdit *Observer_DeltaZ;
  TCheckBox *UseObserver;
  TGroupBox *GroupBox47;
  TGroupBox *GroupBox43;
  TRadioButton *Zoom_Lock_None;
  TRadioButton *Zoom_Lock_In;
  TRadioButton *Zoom_Lock_Out;
  TRadioButton *Zoom_Lock_Both;
  TGroupBox *GroupBox46;
  TButton *Move_Camera_Back;
  TButton *Move_Camera_Ahead;
  TButton *Move_Camera_Left;
  TButton *Move_Camera_Turn_Left;
  TButton *Move_Camera_Turn_Right;
  TButton *Move_Camera_Up;
  TButton *Move_Camera_Down;
  TButton *Move_Camera_Right;
  TGroupBox *GroupBox48;
  TGroupBox *GroupBox35;
  TButton *ViewFromV1ToV2;
  TButton *ViewFromV2ToV1;
  TGroupBox *GroupBox49;
  TLabel *Label57;
  TLabel *Label58;
  TLabel *Label59;
  TEdit *Roaming_Eye_X;
  TEdit *Roaming_Eye_Y;
  TEdit *Roaming_Eye_Z;
  TEdit *Roaming_Center_X;
  TEdit *Roaming_Center_Y;
  TEdit *Roaming_Center_Z;
  TEdit *Roaming_Up_X;
  TEdit *Roaming_Up_Y;
  TEdit *Roaming_Up_Z;
  TButton *Roaming_Get;
  TButton *Roaming_Set;
  TEdit *Move_AheadBack_Step;
  TEdit *Move_RightLeft_Step;
  TEdit *Move_Turn_Step;
  TEdit *Move_UpDown_Step;
  TGroupBox *GroupBox34;
  TLabel *Label38;
  TLabel *Label41;
  TCheckBox *Axis_Lock_X;
  TCheckBox *Axis_Lock_Y;
  TCheckBox *Axis_Lock_Z;
  TCheckBox *Axis_Constrain_X;
  TCheckBox *Axis_Constrain_Y;
  TCheckBox *Axis_Constrain_Z;
  TButton *Get_Axis_X;
  TButton *Get_Axis_Y;
  TButton *Get_Axis_Z;
  TButton *Set_Axis_X;
  TButton *Set_Axis_Y;
  TButton *Set_Axis_Z;
  TEdit *Constrain_From_X;
  TEdit *Constrain_From_Y;
  TEdit *Constrain_From_Z;
  TEdit *Constrain_To_X;
  TEdit *Constrain_To_Y;
  TEdit *Constrain_To_Z;
  TLabel *Label62;
  TEdit *Total_Cameras;
  TButton *Add_Camera;
  TButton *Free_Roaming_Cameras;
  TCheckBox *Roaming_As_Displayed;
  TButton *Roaming_Get_From;
  TEdit *Get_From_CameraID;
  TGroupBox *GroupBox44;
  TListBox *CameraID_List;
  TLabel *Label63;
  TEdit *CameraID;
  TGroupBox *GroupBox50;
  TLabel *Label64;
  TListBox *Active_CameraID_List;
  TEdit *Active_CameraID;
  TLabel *Label19;
  TButton *ZeroCameraAngleX;
  TButton *ZeroCameraAngleY;
  TButton *ZeroCameraAngleZ;
  TButton *ZeroCameraAngleXYZ;
  TButton *Get_Axis_XYZ;
  TButton *Set_Axis_XYZ;
  TButton *Get_Roaming_Axis;
  TButton *Set_Roaming_Axis;
  TButton *Roaming_Up_Default;
  TTabSheet *ListSheet;
  TGroupBox *GroupBox52;
  TRadioButton *Locations_Drag;
  TRadioButton *Locations_Click;
  TGroupBox *GroupBox53;
  TGroupBox *GroupBox54;
  TLabel *Label66;
  TLabel *Label67;
  TLabel *Label68;
  TLabel *Label69;
  TLabel *Label70;
  TLabel *Label71;
  TPanel *Vertex2_Panel_ML;
  TPanel *Vertex1_Panel_ML;
  TPanel *Line_Panel_ML;
  TEdit *LineWidth_ML;
  TEdit *PointSize_ML;
  TCheckBox *UseDepthTest_ML;
  TCheckBox *DisplayVertices_ML;
  TButton *SetRubberbandParameters_ML;
  TGroupBox *GroupBox55;
  TLabel *Label72;
  TLabel *Label73;
  TLabel *Label74;
  TEdit *ObstructionPointSize_ML;
  TCheckBox *DisplayObstructionVertex_ML;
  TButton *SetObstructionParameters_ML;
  TGroupBox *GroupBox56;
  TLabel *Label76;
  TLabel *Label77;
  TLabel *Label78;
  TLabel *Label79;
  TLabel *Label80;
  TPanel *ObserverVertex_Panel_ML;
  TPanel *ObserverLine_Panel_ML;
  TEdit *ObserverLineWidth_ML;
  TEdit *ObserverPointSize_ML;
  TCheckBox *DisplayObserverLine_ML;
  TCheckBox *DisplayObserverVertex_ML;
  TButton *SetObserverParameters_ML;
  TPanel *Obstruction_Panel_ML;
  TGroupBox *GroupBox57;
  TCheckBox *DisplayMeasureList;
  TEdit *MeasureID;
  TButton *AddCurrentMeasure;
  TEdit *MeasureCount;
  TButton *GetMeasureCount;
  TButton *ClearMeasureList;
  TGroupBox *GroupBox60;
  TGroupBox *GroupBox61;
  TLabel *Label88;
  TLabel *Label89;
  TLabel *Label90;
  TLabel *Measure_Vertex1_X;
  TLabel *Measure_Vertex1_Y;
  TLabel *Measure_Vertex1_Z;
  TGroupBox *GroupBox62;
  TLabel *Label91;
  TLabel *Label92;
  TLabel *Label93;
  TLabel *Measure_Vertex2_X;
  TLabel *Measure_Vertex2_Y;
  TLabel *Measure_Vertex2_Z;
  TGroupBox *GroupBox66;
  TGroupBox *GroupBox69;
  TGroupBox *GroupBox70;
  TLabel *Label100;
  TLabel *Label101;
  TLabel *Label102;
  TLabel *Measure_Obstruction_X;
  TLabel *Measure_Obstruction_Y;
  TLabel *Measure_Obstruction_Z;
  TLabel *Label106;
  TLabel *Measure_Obstruction_Distance;
  TGroupBox *GroupBox72;
  TCheckBox *Measure_Visible_Flag;
  TCheckBox *Measure_Observer_Flag;
  TCheckBox *Measure_Obstruction_Flag;
  TTabSheet *OtherSheet;
  TGroupBox *GroupBox51;
  TLabel *Label65;
  TButton *GetGPUMaxTextureSize;
  TButton *GetUserMaxTextureSize;
  TEdit *GPUMaxTextureSize;
  TEdit *UserMaxTextureSize;
  TEdit *PowerOf2_MaxTextureSize;
  TButton *SetUserMaxTextureSize;
  TGroupBox *GroupBox58;
  TLabel *Label75;
  TLabel *Label81;
  TEdit *ImageWidth;
  TEdit *ImageHeight;
  TButton *SaveImage;
  TGroupBox *GroupBox59;
  TLabel *Label82;
  TLabel *Label83;
  TLabel *Label84;
  TLabel *Label85;
  TButton *StatusBar_Show;
  TButton *StatusBar_Hide;
  TEdit *StatusBar_Top;
  TEdit *StatusBar_Left;
  TEdit *StatusBar_Height;
  TEdit *StatusBar_Width;
  TPanel *FontColorPanel;
  TButton *SetFontColor;
  TEdit *FontName;
  TButton *SetFontName;
  TEdit *FontSize;
  TButton *SetFontSize;
  TCheckBox *FontBold;
  TCheckBox *FontItalic;
  TCheckBox *FontUnderline;
  TCheckBox *FontStrikeOut;
  TButton *SetFontStyle;
  TEdit *StatusBar_Text;
  TButton *SetStatusBarText;
  TGroupBox *GroupBox73;
  TButton *Transfer_All_From;
  TButton *Transfer_All_To;
  TButton *Transfer_Random5;
  TLabel *Label107;
  TEdit *Transfer_Count;
  TButton *Transfer_Clear;
  TGroupBox *GroupBox74;
  TLabel *Label86;
  TLabel *Label87;
  TEdit *Get_ID;
  TButton *GetMeasurement;
  TEdit *Get_Index;
  TLabel *Label108;
  TEdit *Measure_Index;
  TLabel *Label112;
  TEdit *Measure_ID;
  TCheckBox *No_Ctrl_Key_Pan;
  TGroupBox *GroupBox75;
  TGroupBox *GroupBox76;
  TListBox *Measure_ID_List;
  TButton *Visible_All;
  TButton *Visible_None;
  TButton *Visible_Set_One;
  TButton *Only_One_Visible;
  TCheckBox *Measure_ID_Visible;
  TButton *DeleteTest1;
  TButton *DeleteTest2;
  TButton *ReplaceTest1;
  TButton *ReplaceTest2;
  TButton *SelectTest1;
  TButton *SelectTest2;
  TButton *SelectCancel;
  TCheckBox *SelectMeasureLine;
  TButton *CenterAtOrigin;
  TGroupBox *GroupBox64;
  TLabel *Measure_Distance;
  TLabel *Label104;
  TLabel *Label103;
  TLabel *Label105;
  TLabel *Measure_Observer_X;
  TLabel *Measure_Observer_Y;
  TLabel *Measure_Observer_Z;
  TGroupBox *GroupBox12;
  TLabel *Locations_Distance;
  TLabel *Obstruction_Distance;
  TGroupBox *GroupBox39;
  TGroupBox *GroupBox40;
  TLabel *Label54;
  TLabel *Label55;
  TLabel *Label56;
  TLabel *Observer_MeshX;
  TLabel *Observer_MeshY;
  TLabel *Observer_MeshZ;
  TGroupBox *GroupBox41;
  TLabel *Label60;
  TLabel *Label61;
  TLabel *Observer_ScreenX;
  TLabel *Observer_ScreenY;
  TGroupBox *GroupBox2;
  TRadioButton *MeasureColor_Default;
  TRadioButton *MeasureColor_Sniper1;
  TRadioButton *MeasureColor_Sniper2;
  TLabel *Label6;
  TEdit *Measure_Line_Width;
  TPanel *Measure_Line_Color;
  TLabel *Label7;
  TButton *GetStatusbarText;
  TEdit *Statusbar_ReadText;
  TButton *Export_Measurements;
  TButton *Import_Measurements;
  TCheckBox *No_Reduce_Vert_Edge_Poly;
  TCheckBox *InvertSelectedColor;
  TCheckBox *No_Ctrl_Key_Zoom;
  TLabel *Label11;
  TEdit *CameraRotateDegrees;
  TButton *SetCameraRotateDegrees;
  TGroupBox *GroupBox10;
  TCheckBox *CullBackFaces;
  TCheckBox *TwoSidedLighting;
  TCheckBox *NoMouseButtonMoveActivity;
  TCheckBox *NoMouseWheelActivity;
  TCheckBox *No_Ctrl_Key_FOV;
  TSpeedButton *GetNoCtrlStepFOV;
  TEdit *NoCtrlStepFOV;
  TSpeedButton *SetNoCtrlStepFOV;
  TTabSheet *BillboardsSheet;
  TGroupBox *GroupBox6;
  TLabel *Label12;
  TLabel *Label22;
  TCheckBox *DisplayBillboards;
  TButton *MakeBillboard;
  TCheckBox *BillboardVisible;
  TEdit *BillboardHeight;
  TGroupBox *GroupBox7;
  TLabel *Label13;
  TLabel *Label14;
  TLabel *Label20;
  TEdit *BillboardPositionX;
  TEdit *BillboardPositionY;
  TEdit *BillboardPositionZ;
  TEdit *BillboardID;
  TGroupBox *GroupBox13;
  TGroupBox *GroupBox11;
  TListBox *Billboard_ID_List;
  TButton *BillboardPosition_V1;
  TButton *BillboardPosition_V2;
  TGroupBox *GroupBox71;
  TLabel *Label8;
  TEdit *BillboardText;
  TCheckBox *BillboardAlpha;
  TGroupBox *GroupBox9;
  TRadioButton *BillboardFontColor1;
  TRadioButton *BillboardFontColor2;
  TRadioButton *BillboardFontColor3;
  TGroupBox *GroupBox77;
  TListBox *BillboardPivotTypeList;
  TCheckBox *BillboardStatic;
  TCheckBox *BillboardOglReplace;
  TGroupBox *GroupBox78;
  TLabel *Label46;
  TLabel *Label47;
  TLabel *Label48;
  TEdit *BillboardRotateZ;
  TEdit *BillboardRotateY;
  TEdit *BillboardRotateX;
  TGroupBox *GroupBox82;
  TButton *Billboard_Copy_Delete;
  TButton *Billboard_Replace;
  TCheckBox *Billboard_Copy_Image;
  TCheckBox *Billboard_Copy_Index;
  TGroupBox *GroupBox83;
  TLabel *Label97;
  TLabel *Label98;
  TLabel *Label99;
  TEdit *BillboardPivotX;
  TEdit *BillboardPivotY;
  TEdit *BillboardPivotZ;
  TButton *BillboardPivot_V1;
  TButton *BillboardPivot_V2;
  TEdit *BillboardLineWidth;
  TLabel *Label109;
  TGroupBox *GroupBox84;
  TEdit *Billboard_Selected_ID;
  TPageControl *PageControl2;
  TTabSheet *Billboard_ID_Sheet;
  TGroupBox *GroupBox65;
  TButton *Billboard_GetID;
  TEdit *Billboard_ID;
  TButton *Billboard_SetID;
  TTabSheet *Billboard_Image_Sheet;
  TGroupBox *GroupBox67;
  TButton *Billboard_GetImage1;
  TButton *Billboard_GetImage2;
  TButton *Billboard_SetImage1;
  TButton *Billboard_SetImage2;
  TTabSheet *Billboard_Height_Sheet;
  TGroupBox *GroupBox63;
  TEdit *Billboard_Height;
  TButton *Billboard_GetHeight1;
  TButton *Billboard_GetHeight2;
  TButton *Billboard_SetHeight1;
  TButton *Billboard_SetHeight2;
  TTabSheet *Billboard_Delete_Sheet;
  TGroupBox *GroupBox26;
  TButton *Billboard_DeleteTest1;
  TButton *Billboard_DeleteTest2;
  TButton *DeleteAllBillboards;
  TTabSheet *Billboard_Visibility_Sheet;
  TGroupBox *GroupBox29;
  TButton *Billboard_Visible_All;
  TButton *Billboard_Visible_One;
  TCheckBox *Billboard_ID_Visible;
  TButton *Billboard_Visible_Set_One;
  TButton *Billboard_Only_One_Visible;
  TTabSheet *Billboard_Pivot_Sheet;
  TGroupBox *GroupBox28;
  TLabel *Label23;
  TLabel *Label27;
  TLabel *Label44;
  TEdit *Billboard_PivotX;
  TEdit *Billboard_PivotY;
  TEdit *Billboard_PivotZ;
  TButton *Billboard_SetPivot1;
  TButton *Billboard_SetPivot2;
  TButton *Billboard_GetPivot1;
  TButton *Billboard_GetPivot2;
  TTabSheet *Billboard_PivotType_Sheet;
  TGroupBox *GroupBox68;
  TListBox *Billboard_PivotTypeList;
  TButton *Billboard_PivotTypeGet1;
  TButton *Billboard_PivotTypeGet2;
  TButton *Billboard_PivotTypeSet1;
  TButton *Billboard_PivotTypeSet2;
  TTabSheet *Billboard_OGL_Replace_Sheet;
  TGroupBox *GroupBox79;
  TButton *Billboard_GetOglReplace1;
  TButton *Billboard_GetOglReplace2;
  TButton *Billboard_SetOglReplace1;
  TButton *Billboard_SetOglReplace2;
  TCheckBox *Billboard_OglReplace;
  TTabSheet *Billboard_Static_Sheet;
  TGroupBox *GroupBox80;
  TButton *Billboard_GetStatic1;
  TButton *Billboard_GetStatic2;
  TButton *Billboard_SetStatic1;
  TButton *Billboard_SetStatic2;
  TCheckBox *Billboard_Static;
  TTabSheet *Billboard_RotationStatic_Sheet;
  TGroupBox *GroupBox81;
  TLabel *Label94;
  TLabel *Label95;
  TLabel *Label96;
  TEdit *Billboard_RotateX;
  TEdit *Billboard_RotateY;
  TEdit *Billboard_RotateZ;
  TButton *Billboard_SetRotation1;
  TButton *Billboard_SetRotation2;
  TButton *Billboard_GetRotation1;
  TButton *Billboard_GetRotation2;
  TButton *Billboard_Cancel_Selected;
  TCheckBox *Billboard_InvertSelectedColor;
  TLabel *Label110;
  TEdit *Billboard_Selected_Red;
  TEdit *Billboard_Selected_Green;
  TEdit *Billboard_Selected_Blue;
  TLabel *Label111;
  TEdit *Billboard_Selected_Width;
  TButton *Billboard_SetColor;
  TButton *Billboard_SetWidth;
  TEdit *Billboard_PositionX;
  TEdit *Billboard_PositionY;
  TEdit *Billboard_PositionZ;
  TLabel *Label113;
  TLabel *Label114;
  TTabSheet *Billboard_Line_Sheet;
  TGroupBox *GroupBox85;
  TLabel *Label115;
  TEdit *Billboard_Line_Red;
  TEdit *Billboard_Line_Green;
  TEdit *Billboard_Line_Blue;
  TLabel *Label116;
  TEdit *Billboard_Line_Width;
  TButton *Billboard_Line_Get1;
  TButton *Billboard_Line_Get2;
  TButton *Billboard_Line_Set1;
  TButton *Billboard_Line_Set2;
  TGroupBox *GroupBox86;
  TEdit *Billboard_Count;
  TButton *Billboard_Get_Count;
  TEdit *BillboardType;
  TLabel *Label117;
  TTabSheet *Billboard_Type_Sheet;
  TGroupBox *GroupBox87;
  TEdit *Billboard_Type;
  TButton *Billboard_GetType1;
  TButton *Billboard_GetType2;
  TButton *Billboard_SetType1;
  TButton *Billboard_SetType2;
  TGroupBox *GroupBox88;
  TGroupBox *GroupBox89;
  TLabel *Label118;
  TLabel *Label119;
  TEdit *Get_ScreenX;
  TEdit *Get_ScreenY;
  TGroupBox *GroupBox90;
  TLabel *Label120;
  TLabel *Label121;
  TLabel *Label122;
  TLabel *Get_MeshX;
  TLabel *Get_MeshY;
  TLabel *Get_MeshZ;
  TButton *Get_Location;
  TCheckBox *Integrated_GPU;
  TGroupBox *GroupBox91;
  TLabel *Label123;
  TLabel *Label124;
  TLabel *Label125;
  TEdit *LineMesh_Intersection_X;
  TEdit *LineMesh_V2_X;
  TEdit *LineMesh_V1_X;
  TEdit *LineMesh_V1_Y;
  TEdit *LineMesh_V2_Y;
  TEdit *LineMesh_Intersection_Y;
  TEdit *LineMesh_Intersection_Z;
  TEdit *LineMesh_V2_Z;
  TEdit *LineMesh_V1_Z;
  TButton *Get_Intersection_Point;
  TLabel *Label126;
  TLabel *Label127;
  TLabel *Label128;
  TLabel *Label129;
  TEdit *LineMesh_SkipDistance;
  TCheckBox *Locations_Depth_Test;
  TTabSheet *MeshesSheet;
  TGroupBox *GroupBox95;
  TGroupBox *GroupBox92;
  TLabel *Label136;
  TLabel *Label137;
  TGroupBox *GroupBox93;
  TLabel *Label133;
  TLabel *Label134;
  TLabel *Label135;
  TEdit *ReadAddPositionX;
  TEdit *ReadAddPositionY;
  TEdit *ReadAddPositionZ;
  TButton *ReadAddPosition_V1;
  TButton *ReadAddPosition_V2;
  TEdit *ReadAddScaleFactor;
  TEdit *ReadAddPrefix;
  TGroupBox *GroupBox94;
  TRadioButton *ReadAddRotateX_No;
  TRadioButton *ReadAddRotateX_Minus90;
  TRadioButton *ReadAddRotateX_Plus90;
  TRadioButton *ReadAddRotateX_180;
  TButton *Read_Add_Static_Mesh;
  TGroupBox *GroupBox96;
  TGroupBox *GroupBox97;
  TListBox *AdjustMesh_List;
  TGroupBox *GroupBox98;
  TLabel *Label138;
  TLabel *Label139;
  TLabel *Label140;
  TEdit *AdjustMesh_PivotX;
  TEdit *AdjustMesh_PivotY;
  TEdit *AdjustMesh_PivotZ;
  TButton *AdjustMesh_Pivot_Get;
  TButton *AdjustMesh_Pivot_Set;
  TButton *AdjustMesh_Pivot_Calculate;
  TGroupBox *GroupBox99;
  TLabel *Label141;
  TLabel *Label142;
  TLabel *Label143;
  TEdit *AdjustMesh_RotateX;
  TEdit *AdjustMesh_RotateY;
  TEdit *AdjustMesh_RotateZ;
  TButton *AdjustMesh_Rotate;
  TGroupBox *GroupBox100;
  TLabel *Label144;
  TLabel *Label145;
  TLabel *Label146;
  TEdit *AdjustMesh_ScaleX;
  TEdit *AdjustMesh_ScaleY;
  TEdit *AdjustMesh_ScaleZ;
  TButton *AdjustMesh_Scale;
  TGroupBox *GroupBox101;
  TLabel *Label147;
  TLabel *Label148;
  TLabel *Label149;
  TEdit *AdjustMesh_TranslateX;
  TEdit *AdjustMesh_TranslateY;
  TEdit *AdjustMesh_TranslateZ;
  TButton *AdjustMesh_Translate;
  TButton *AdjustMesh_Translate_Relative;
  TGroupBox *GroupBox102;
  TButton *AdjustMesh_Delete;
  TGroupBox *GroupBox104;
  TButton *AdjustMesh_On;
  TButton *AdjustMesh_Off;
  TGroupBox *GroupBox105;
  TLabel *Label150;
  TLabel *Label151;
  TLabel *Label152;
  TEdit *AdjustMesh_SizeX;
  TEdit *AdjustMesh_SizeY;
  TEdit *AdjustMesh_SizeZ;
  TButton *AdjustMesh_Size;
  TButton *AdjustMesh_Pivot_V1;
  TButton *AdjustMesh_Pivot_V2;
  TButton *AdjustMesh_Translate_V1;
  TButton *AdjustMesh_Translate_V2;
  TGroupBox *GroupBox106;
  TLabel *Get_LayerName;
  TTabSheet *ClonesSheet;
  TGroupBox *GroupBox107;
  TGroupBox *GroupBox108;
  TEdit *Add_CloneMesh_ScaleFactor;
  TLabel *Label153;
  TLabel *Label154;
  TEdit *Add_CloneMesh_Name;
  TGroupBox *GroupBox109;
  TRadioButton *Add_CloneMesh_RotateX_No;
  TRadioButton *Add_CloneMesh_RotateX_Minus90;
  TRadioButton *Add_CloneMesh_RotateX_Plus90;
  TRadioButton *Add_CloneMesh_RotateX_180;
  TButton *CloneMesh_Add;
  TListBox *CloneMesh_List;
  TButton *CloneMesh_DeleteOne;
  TButton *CloneMesh_DeleteAll;
  TGroupBox *Data;
  TButton *CloneMesh_Get;
  TLabel *Label155;
  TEdit *Data_CloneMesh_Filename;
  TLabel *Label156;
  TEdit *Data_CloneMesh_ScaleFactor;
  TEdit *Data_CloneMesh_RotationX;
  TLabel *Label157;
  TGroupBox *GroupBox110;
  TListBox *CloneItem_List;
  TButton *CloneItem_DeleteOne;
  TButton *CloneItem_DeleteAll;
  TGroupBox *GroupBox111;
  TButton *CloneItem_Visible_On;
  TButton *CloneItem_Visible_Off;
  TGroupBox *GroupBox112;
  TLabel *Label158;
  TEdit *Add_CloneItem_LocationX;
  TEdit *Add_CloneItem_LocationY;
  TLabel *Label159;
  TLabel *Label160;
  TEdit *Add_CloneItem_LocationZ;
  TButton *Add_CloneItem_Location_V1;
  TButton *Add_CloneItem_Location_V2;
  TEdit *Add_CloneItem_RotationX;
  TEdit *Add_CloneItem_RotationY;
  TEdit *Add_CloneItem_RotationZ;
  TLabel *Label161;
  TLabel *Label162;
  TLabel *Label163;
  TLabel *Label164;
  TEdit *Add_CloneItem_ItemName;
  TEdit *Add_CloneItem_MeshName;
  TButton *Add_CloneItem_GetMeshName;
  TCheckBox *Add_CloneItem_Visible;
  TButton *Add_CloneItem_Add;
  TGroupBox *GroupBox114;
  TLabel *Label165;
  TEdit *CloneItem_MeshName;
  TButton *CloneItem_GetMeshName;
  TButton *CloneItem_Change_MeshName;
  TGroupBox *GroupBox116;
  TLabel *Label166;
  TEdit *Data_CloneItem_MeshName;
  TLabel *Label167;
  TLabel *Label168;
  TLabel *Label169;
  TLabel *Label173;
  TEdit *Data_CloneItem_LocationX;
  TEdit *Data_CloneItem_LocationY;
  TEdit *Data_CloneItem_LocationZ;
  TLabel *Label174;
  TEdit *Data_CloneItem_RotationX;
  TEdit *Data_CloneItem_RotationY;
  TEdit *Data_CloneItem_RotationZ;
  TCheckBox *Data_CloneItem_Visible;
  TButton *CloneItem_GetData;
  TGroupBox *GroupBox117;
  TLabel *Label175;
  TLabel *Label176;
  TLabel *Label177;
  TEdit *CloneItem_RotateX;
  TEdit *CloneItem_RotateY;
  TEdit *CloneItem_RotateZ;
  TButton *CloneItem_Rotate;
  TButton *CloneItem_Rotate_Relative;
  TGroupBox *GroupBox118;
  TLabel *Label178;
  TLabel *Label179;
  TLabel *Label180;
  TEdit *CloneItem_TranslateX;
  TEdit *CloneItem_TranslateY;
  TEdit *CloneItem_TranslateZ;
  TButton *CloneItem_Translate;
  TButton *CloneItem_Translate_Relative;
  TButton *CloneItem_Translate_V1;
  TButton *CloneItem_Translate_V2;
  TGroupBox *GroupBox119;
  TEdit *CloneMesh_SizeY;
  TEdit *CloneMesh_SizeX;
  TEdit *CloneMesh_SizeZ;
  TButton *CloneMesh_Size_Get;
  TLabel *Label181;
  TLabel *Label182;
  TLabel *Label183;
  TLabel *Label184;
  TEdit *CloneMesh_Old_ScaleFactor;
  TEdit *CloneMesh_New_ScaleFactor;
  TLabel *Label185;
  TButton *CloneMesh_Size_Set;
  TGroupBox *GroupBox120;
  TRadioButton *CloneMesh_RotateX_Minus90;
  TRadioButton *CloneMesh_RotateX_Plus90;
  TRadioButton *CloneMesh_RotateX_180;
  TButton *CloneMesh_RotateX_Rotate;
  TButton *CloneMesh_Change;
  TGroupBox *GroupBox121;
  TGroupBox *GroupBox122;
  TLabel *Label186;
  TLabel *Label187;
  TLabel *Label188;
  TLabel *Label189;
  TLabel *Label190;
  TEdit *Size_CloneItem_OriginalX;
  TEdit *Size_CloneItem_CurrentX;
  TEdit *Size_CloneItem_CurrentY;
  TEdit *Size_CloneItem_OriginalY;
  TEdit *Size_CloneItem_OriginalZ;
  TEdit *Size_CloneItem_CurrentZ;
  TLabel *Label191;
  TLabel *Label192;
  TLabel *Label193;
  TLabel *Label194;
  TLabel *Label195;
  TEdit *Snap_CloneItem_FromX;
  TEdit *Snap_CloneItem_ToX;
  TEdit *Snap_CloneItem_ToY;
  TEdit *Snap_CloneItem_FromY;
  TEdit *Snap_CloneItem_FromZ;
  TEdit *Snap_CloneItem_ToZ;
  TButton *Snap_CloneItem_From_V1;
  TButton *Snap_CloneItem_From_V2;
  TButton *Snap_CloneItem_To_V1;
  TButton *Snap_CloneItem_To_V2;
  TButton *Snap_CloneItem_Snap;
  TLabel *Label196;
  TLabel *Label197;
  TEdit *Size_CloneItem_Old_ScaleFactor;
  TEdit *Size_CloneItem_New_ScaleFactor;
  TButton *Size_CloneItem_Get;
  TButton *Size_CloneItem_Set;
  TLabel *Label198;
  TEdit *Add_CloneItem_ScaleFactor;
  TLabel *Label199;
  TEdit *Data_CloneItem_ScaleFactor;
  TGroupBox *GroupBox123;
  TEdit *CloneMesh_New_MeshName;
  TButton *CloneMesh_Change_MeshName;
  TGroupBox *GroupBox124;
  TEdit *CloneItem_New_ItemName;
  TButton *CloneItem_Change_ItemName;
  TGroupBox *GroupBox125;
  TLabel *Get_CloneItemName;
  TButton *ReadClonesFile;
  TButton *SaveClonesFile;
  TCheckBox *DisplayClones;
  TLabel *Label200;
  TLabel *Label201;
  TEdit *ImagePointSize;
  TEdit *ImageLineWidth;
  void __fastcall FormDestroy(TObject *Sender);
  void __fastcall Read3dFileClick(TObject *Sender);
  void __fastcall UseTexturesClick(TObject *Sender);
  void __fastcall AlwaysShowCameraCursorClick(TObject *Sender);
  void __fastcall TouchScreenInputClick(TObject *Sender);
  void __fastcall ViewFrontClick(TObject *Sender);
  void __fastcall ViewBackClick(TObject *Sender);
  void __fastcall ViewRightClick(TObject *Sender);
  void __fastcall ViewLeftClick(TObject *Sender);
  void __fastcall ViewTopClick(TObject *Sender);
  void __fastcall ViewBottomClick(TObject *Sender);
  void __fastcall GetCameraViewpointClick(TObject *Sender);
  void __fastcall SetCameraViewpointClick(TObject *Sender);
  void __fastcall GetCameraFieldOfViewClick(TObject *Sender);
  void __fastcall SetCameraFieldOfViewClick(TObject *Sender);
  void __fastcall SetCameraFOV1Click(TObject *Sender);
  void __fastcall SetCameraFOV2Click(TObject *Sender);
  void __fastcall SetCameraRotateSpeed1Click(TObject *Sender);
  void __fastcall SetCameraRotateSpeed2Click(TObject *Sender);
  void __fastcall GetCameraAnglesClick(TObject *Sender);
  void __fastcall SetCameraAnglesClick(TObject *Sender);
  void __fastcall SetCameraXAngle1Click(TObject *Sender);
  void __fastcall SetCameraXAngle2Click(TObject *Sender);
  void __fastcall SetCameraYAngle1Click(TObject *Sender);
  void __fastcall SetCameraYAngle2Click(TObject *Sender);
  void __fastcall SetCameraZAngle1Click(TObject *Sender);
  void __fastcall SetCameraZAngle2Click(TObject *Sender);
  void __fastcall SetCameraPanClick(TObject *Sender);
  void __fastcall SetCameraXPan1Click(TObject *Sender);
  void __fastcall SetCameraXPan2Click(TObject *Sender);
  void __fastcall SetCameraZPan1Click(TObject *Sender);
  void __fastcall SetCameraZPan2Click(TObject *Sender);
  void __fastcall ViewCenterXClick(TObject *Sender);
  void __fastcall ViewCenterZClick(TObject *Sender);
  void __fastcall ReversePanDirectionClick(TObject *Sender);
  void __fastcall GetCameraInitialDistanceClick(TObject *Sender);
  void __fastcall SetCameraInitialDistanceClick(TObject *Sender);
  void __fastcall GetCameraDistanceClick(TObject *Sender);
  void __fastcall SetCameraDistanceClick(TObject *Sender);
  void __fastcall GetCameraZoomSpeedClick(TObject *Sender);
  void __fastcall SetCameraZoomSpeed1Click(TObject *Sender);
  void __fastcall SetCameraZoomSpeed2Click(TObject *Sender);
  void __fastcall SetCameraZoomSpeed3Click(TObject *Sender);
  void __fastcall SetCameraZoomSpeed4Click(TObject *Sender);
  void __fastcall SetCameraZoomSpeed5Click(TObject *Sender);
  void __fastcall SetCameraZoomSpeed6Click(TObject *Sender);
  void __fastcall ZoomInClick(TObject *Sender);
  void __fastcall ZoomOutClick(TObject *Sender);
  void __fastcall ZoomResetClick(TObject *Sender);
  void __fastcall ReverseZoomDirectionClick(TObject *Sender);
  void __fastcall SwapEndsClick(TObject *Sender);
  void __fastcall Locations_OffClick(TObject *Sender);
  void __fastcall Locations_MeasureClick(TObject *Sender);
  void __fastcall Locations_PauseClick(TObject *Sender);
  void __fastcall Locations_ContinueClick(TObject *Sender);
  void __fastcall SetRubberbandParametersClick(TObject *Sender);
  void __fastcall SetObstructionParametersClick(TObject *Sender);
  void __fastcall Line_PanelClick(TObject *Sender);
  void __fastcall Vertex1_PanelClick(TObject *Sender);
  void __fastcall Vertex2_PanelClick(TObject *Sender);
  void __fastcall UseDepthTestClick(TObject *Sender);
  void __fastcall DisplayVerticesClick(TObject *Sender);
  void __fastcall Obstruction_PanelClick(TObject *Sender);
  void __fastcall DisplayObstructionVertexClick(TObject *Sender);
  void __fastcall CheckForObstructionClick(TObject *Sender);
  void __fastcall ClipAtObstructionPointClick(TObject *Sender);
  void __fastcall UpdateScreenXYClick(TObject *Sender);
  void __fastcall UpdateObstructionClick(TObject *Sender);
  void __fastcall TypeObjClick(TObject *Sender);
  void __fastcall TypeWrlClick(TObject *Sender);
  void __fastcall SetViewpoint_MiddleClick(TObject *Sender);
  void __fastcall SetViewpoint_LeftClick(TObject *Sender);
  void __fastcall LeftToggle_RotateClick(TObject *Sender);
  void __fastcall LeftToggle_PanClick(TObject *Sender);
  void __fastcall MultiReadClick(TObject *Sender);
  void __fastcall Axis_Lock_XClick(TObject *Sender);
  void __fastcall Axis_Lock_YClick(TObject *Sender);
  void __fastcall Axis_Lock_ZClick(TObject *Sender);
  void __fastcall Axis_Constrain_XClick(TObject *Sender);
  void __fastcall Axis_Constrain_YClick(TObject *Sender);
  void __fastcall Axis_Constrain_ZClick(TObject *Sender);
  void __fastcall Get_Axis_XClick(TObject *Sender);
  void __fastcall Set_Axis_XClick(TObject *Sender);
  void __fastcall Get_Axis_YClick(TObject *Sender);
  void __fastcall Set_Axis_YClick(TObject *Sender);
  void __fastcall Get_Axis_ZClick(TObject *Sender);
  void __fastcall Set_Axis_ZClick(TObject *Sender);
  void __fastcall ViewFromV1ToV2Click(TObject *Sender);
  void __fastcall ViewFromV2ToV1Click(TObject *Sender);
  void __fastcall ObserverLine_PanelClick(TObject *Sender);
  void __fastcall ObserverVertex_PanelClick(TObject *Sender);
  void __fastcall DisplayObserverLineClick(TObject *Sender);
  void __fastcall DisplayObserverVertexClick(TObject *Sender);
  void __fastcall SetObserverParametersClick(TObject *Sender);
  void __fastcall UseObserverClick(TObject *Sender);
  void __fastcall SetViewpoint1Click(TObject *Sender);
  void __fastcall SetViewpoint2Click(TObject *Sender);
  void __fastcall Zoom_Lock_NoneClick(TObject *Sender);
  void __fastcall Zoom_Lock_InClick(TObject *Sender);
  void __fastcall Zoom_Lock_OutClick(TObject *Sender);
  void __fastcall Zoom_Lock_BothClick(TObject *Sender);
  void __fastcall ZoomAutomaticClick(TObject *Sender);
  void __fastcall RotateX1ObjectClick(TObject *Sender);
  void __fastcall RotateX2ObjectClick(TObject *Sender);
  void __fastcall RotateX3ObjectClick(TObject *Sender);
  void __fastcall RotateY1ObjectClick(TObject *Sender);
  void __fastcall RotateY2ObjectClick(TObject *Sender);
  void __fastcall RotateY3ObjectClick(TObject *Sender);
  void __fastcall RotateZ1ObjectClick(TObject *Sender);
  void __fastcall RotateZ2ObjectClick(TObject *Sender);
  void __fastcall RotateZ3ObjectClick(TObject *Sender);
  void __fastcall Move_Camera_AheadClick(TObject *Sender);
  void __fastcall Move_Camera_BackClick(TObject *Sender);
  void __fastcall Move_Camera_RightClick(TObject *Sender);
  void __fastcall Move_Camera_LeftClick(TObject *Sender);
  void __fastcall Move_Camera_UpClick(TObject *Sender);
  void __fastcall Move_Camera_DownClick(TObject *Sender);
  void __fastcall Move_Camera_Turn_RightClick(TObject *Sender);
  void __fastcall Move_Camera_Turn_LeftClick(TObject *Sender);
  void __fastcall Roaming_GetClick(TObject *Sender);
  void __fastcall Roaming_SetClick(TObject *Sender);
  void __fastcall Add_CameraClick(TObject *Sender);
  void __fastcall Free_Roaming_CamerasClick(TObject *Sender);
  void __fastcall Roaming_As_DisplayedClick(TObject *Sender);
  void __fastcall Roaming_Get_FromClick(TObject *Sender);
  void __fastcall CameraID_ListClick(TObject *Sender);
  void __fastcall Active_CameraID_ListClick(TObject *Sender);
  void __fastcall ZeroCameraAngleXClick(TObject *Sender);
  void __fastcall ZeroCameraAngleYClick(TObject *Sender);
  void __fastcall ZeroCameraAngleZClick(TObject *Sender);
  void __fastcall ZeroCameraAngleXYZClick(TObject *Sender);
  void __fastcall Get_Axis_XYZClick(TObject *Sender);
  void __fastcall Set_Axis_XYZClick(TObject *Sender);
  void __fastcall Get_Roaming_AxisClick(TObject *Sender);
  void __fastcall Set_Roaming_AxisClick(TObject *Sender);
  void __fastcall Roaming_Up_DefaultClick(TObject *Sender);
  void __fastcall GetGPUMaxTextureSizeClick(TObject *Sender);
  void __fastcall GetUserMaxTextureSizeClick(TObject *Sender);
  void __fastcall SetUserMaxTextureSizeClick(TObject *Sender);
  void __fastcall Locations_DragClick(TObject *Sender);
  void __fastcall Locations_ClickClick(TObject *Sender);
  void __fastcall Line_Panel_MLClick(TObject *Sender);
  void __fastcall Vertex1_Panel_MLClick(TObject *Sender);
  void __fastcall Vertex2_Panel_MLClick(TObject *Sender);
  void __fastcall UseDepthTest_MLClick(TObject *Sender);
  void __fastcall DisplayVertices_MLClick(TObject *Sender);
  void __fastcall SetRubberbandParameters_MLClick(TObject *Sender);
  void __fastcall Obstruction_Panel_MLClick(TObject *Sender);
  void __fastcall DisplayObstructionVertex_MLClick(TObject *Sender);
  void __fastcall SetObstructionParameters_MLClick(TObject *Sender);
  void __fastcall ObserverLine_Panel_MLClick(TObject *Sender);
  void __fastcall ObserverVertex_Panel_MLClick(TObject *Sender);
  void __fastcall DisplayObserverLine_MLClick(TObject *Sender);
  void __fastcall DisplayObserverVertex_MLClick(TObject *Sender);
  void __fastcall SetObserverParameters_MLClick(TObject *Sender);
  void __fastcall DisplayMeasureListClick(TObject *Sender);
  void __fastcall AddCurrentMeasureClick(TObject *Sender);
  void __fastcall SaveImageClick(TObject *Sender);
  void __fastcall StatusBar_ShowClick(TObject *Sender);
  void __fastcall StatusBar_HideClick(TObject *Sender);
  void __fastcall FontColorPanelClick(TObject *Sender);
  void __fastcall SetFontColorClick(TObject *Sender);
  void __fastcall SetFontNameClick(TObject *Sender);
  void __fastcall SetFontSizeClick(TObject *Sender);
  void __fastcall SetFontStyleClick(TObject *Sender);
  void __fastcall SetStatusBarTextClick(TObject *Sender);
  void __fastcall GetMeasureCountClick(TObject *Sender);
  void __fastcall ClearMeasureListClick(TObject *Sender);
  void __fastcall GetMeasurementClick(TObject *Sender);
  void __fastcall Transfer_All_FromClick(TObject *Sender);
  void __fastcall Transfer_Random5Click(TObject *Sender);
  void __fastcall Transfer_All_ToClick(TObject *Sender);
  void __fastcall Transfer_ClearClick(TObject *Sender);
  void __fastcall No_Ctrl_Key_PanClick(TObject *Sender);
  void __fastcall Visible_AllClick(TObject *Sender);
  void __fastcall Visible_NoneClick(TObject *Sender);
  void __fastcall Visible_Set_OneClick(TObject *Sender);
  void __fastcall Only_One_VisibleClick(TObject *Sender);
  void __fastcall DeleteTest1Click(TObject *Sender);
  void __fastcall DeleteTest2Click(TObject *Sender);
  void __fastcall ReplaceTest1Click(TObject *Sender);
  void __fastcall ReplaceTest2Click(TObject *Sender);
  void __fastcall SelectTest1Click(TObject *Sender);
  void __fastcall SelectTest2Click(TObject *Sender);
  void __fastcall SelectCancelClick(TObject *Sender);
  void __fastcall SelectMeasureLineClick(TObject *Sender);
  void __fastcall CenterAtOriginClick(TObject *Sender);
  void __fastcall MeasureColor_DefaultClick(TObject *Sender);
  void __fastcall MeasureColor_Sniper1Click(TObject *Sender);
  void __fastcall MeasureColor_Sniper2Click(TObject *Sender);
  void __fastcall GetStatusbarTextClick(TObject *Sender);
  void __fastcall Export_MeasurementsClick(TObject *Sender);
  void __fastcall Import_MeasurementsClick(TObject *Sender);
  void __fastcall No_Reduce_Vert_Edge_PolyClick(TObject *Sender);
  void __fastcall InvertSelectedColorClick(TObject *Sender);
  void __fastcall DisplayBillboardsClick(TObject *Sender);
  void __fastcall MakeBillboardClick(TObject *Sender);
  void __fastcall No_Ctrl_Key_ZoomClick(TObject *Sender);
  void __fastcall SetCameraRotateDegreesClick(TObject *Sender);
  void __fastcall TwoSidedLightingClick(TObject *Sender);
  void __fastcall CullBackFacesClick(TObject *Sender);
  void __fastcall NoMouseButtonMoveActivityClick(TObject *Sender);
  void __fastcall NoMouseWheelActivityClick(TObject *Sender);
  void __fastcall No_Ctrl_Key_FOVClick(TObject *Sender);
  void __fastcall GetNoCtrlStepFOVClick(TObject *Sender);
  void __fastcall SetNoCtrlStepFOVClick(TObject *Sender);
  void __fastcall DeleteAllBillboardsClick(TObject *Sender);
  void __fastcall Billboard_Visible_AllClick(TObject *Sender);
  void __fastcall Billboard_Visible_OneClick(TObject *Sender);
  void __fastcall Billboard_Visible_Set_OneClick(TObject *Sender);
  void __fastcall Billboard_DeleteTest1Click(TObject *Sender);
  void __fastcall Billboard_DeleteTest2Click(TObject *Sender);
  void __fastcall Billboard_Only_One_VisibleClick(TObject *Sender);
  void __fastcall Billboard_SetPivot1Click(TObject *Sender);
  void __fastcall Billboard_SetPivot2Click(TObject *Sender);
  void __fastcall Billboard_GetPivot1Click(TObject *Sender);
  void __fastcall Billboard_GetPivot2Click(TObject *Sender);
  void __fastcall Billboard_GetHeight1Click(TObject *Sender);
  void __fastcall Billboard_GetHeight2Click(TObject *Sender);
  void __fastcall Billboard_SetHeight1Click(TObject *Sender);
  void __fastcall Billboard_SetHeight2Click(TObject *Sender);
  void __fastcall Billboard_GetIDClick(TObject *Sender);
  void __fastcall Billboard_SetIDClick(TObject *Sender);
  void __fastcall Billboard_GetImage1Click(TObject *Sender);
  void __fastcall Billboard_GetImage2Click(TObject *Sender);
  void __fastcall Billboard_SetImage1Click(TObject *Sender);
  void __fastcall Billboard_SetImage2Click(TObject *Sender);
  void __fastcall BillboardPosition_V1Click(TObject *Sender);
  void __fastcall BillboardPosition_V2Click(TObject *Sender);
  void __fastcall Billboard_PivotTypeGet1Click(TObject *Sender);
  void __fastcall Billboard_PivotTypeGet2Click(TObject *Sender);
  void __fastcall Billboard_PivotTypeSet1Click(TObject *Sender);
  void __fastcall Billboard_PivotTypeSet2Click(TObject *Sender);
  void __fastcall Billboard_GetOglReplace1Click(TObject *Sender);
  void __fastcall Billboard_GetOglReplace2Click(TObject *Sender);
  void __fastcall Billboard_SetOglReplace1Click(TObject *Sender);
  void __fastcall Billboard_SetOglReplace2Click(TObject *Sender);
  void __fastcall Billboard_GetStatic1Click(TObject *Sender);
  void __fastcall Billboard_GetStatic2Click(TObject *Sender);
  void __fastcall Billboard_SetStatic1Click(TObject *Sender);
  void __fastcall Billboard_SetStatic2Click(TObject *Sender);
  void __fastcall Billboard_GetRotation1Click(TObject *Sender);
  void __fastcall Billboard_GetRotation2Click(TObject *Sender);
  void __fastcall Billboard_SetRotation1Click(TObject *Sender);
  void __fastcall Billboard_SetRotation2Click(TObject *Sender);
  void __fastcall Billboard_Copy_DeleteClick(TObject *Sender);
  void __fastcall Billboard_ReplaceClick(TObject *Sender);
  void __fastcall BillboardPivot_V1Click(TObject *Sender);
  void __fastcall BillboardPivot_V2Click(TObject *Sender);
  void __fastcall Billboard_Cancel_SelectedClick(TObject *Sender);
  void __fastcall Billboard_InvertSelectedColorClick(TObject *Sender);
  void __fastcall Billboard_SetColorClick(TObject *Sender);
  void __fastcall Billboard_SetWidthClick(TObject *Sender);
  void __fastcall Billboard_Line_Get1Click(TObject *Sender);
  void __fastcall Billboard_Line_Get2Click(TObject *Sender);
  void __fastcall Billboard_Line_Set1Click(TObject *Sender);
  void __fastcall Billboard_Line_Set2Click(TObject *Sender);
  void __fastcall Billboard_Get_CountClick(TObject *Sender);
  void __fastcall Billboard_GetType1Click(TObject *Sender);
  void __fastcall Billboard_GetType2Click(TObject *Sender);
  void __fastcall Billboard_SetType1Click(TObject *Sender);
  void __fastcall Billboard_SetType2Click(TObject *Sender);
  void __fastcall Get_LocationClick(TObject *Sender);
  void __fastcall Integrated_GPUClick(TObject *Sender);
  void __fastcall Get_Intersection_PointClick(TObject *Sender);
  void __fastcall Locations_Depth_TestClick(TObject *Sender);
  void __fastcall ReadAddPosition_V1Click(TObject *Sender);
  void __fastcall ReadAddPosition_V2Click(TObject *Sender);
  void __fastcall Read_Add_Static_MeshClick(TObject *Sender);
  void __fastcall AdjustMesh_Pivot_CalculateClick(TObject *Sender);
  void __fastcall AdjustMesh_Pivot_GetClick(TObject *Sender);
  void __fastcall AdjustMesh_Pivot_SetClick(TObject *Sender);
  void __fastcall AdjustMesh_RotateClick(TObject *Sender);
  void __fastcall AdjustMesh_ScaleClick(TObject *Sender);
  void __fastcall AdjustMesh_Translate_RelativeClick(TObject *Sender);
  void __fastcall AdjustMesh_TranslateClick(TObject *Sender);
  void __fastcall AdjustMesh_DeleteClick(TObject *Sender);
  void __fastcall AdjustMesh_OnClick(TObject *Sender);
  void __fastcall AdjustMesh_OffClick(TObject *Sender);
  void __fastcall AdjustMesh_SizeClick(TObject *Sender);
  void __fastcall AdjustMesh_Pivot_V1Click(TObject *Sender);
  void __fastcall AdjustMesh_Pivot_V2Click(TObject *Sender);
  void __fastcall AdjustMesh_Translate_V1Click(TObject *Sender);
  void __fastcall AdjustMesh_Translate_V2Click(TObject *Sender);
  void __fastcall CloneMesh_AddClick(TObject *Sender);
  void __fastcall CloneMesh_DeleteOneClick(TObject *Sender);
  void __fastcall CloneMesh_DeleteAllClick(TObject *Sender);
  void __fastcall CloneMesh_GetClick(TObject *Sender);
  void __fastcall Add_CloneItem_GetMeshNameClick(TObject *Sender);
  void __fastcall Add_CloneItem_Location_V1Click(TObject *Sender);
  void __fastcall Add_CloneItem_Location_V2Click(TObject *Sender);
  void __fastcall Add_CloneItem_AddClick(TObject *Sender);
  void __fastcall CloneItem_GetMeshNameClick(TObject *Sender);
  void __fastcall CloneItem_Change_MeshNameClick(TObject *Sender);
  void __fastcall CloneItem_DeleteOneClick(TObject *Sender);
  void __fastcall CloneItem_DeleteAllClick(TObject *Sender);
  void __fastcall CloneItem_GetDataClick(TObject *Sender);
  void __fastcall CloneItem_Rotate_RelativeClick(TObject *Sender);
  void __fastcall CloneItem_RotateClick(TObject *Sender);
  void __fastcall CloneItem_Translate_V1Click(TObject *Sender);
  void __fastcall CloneItem_Translate_V2Click(TObject *Sender);
  void __fastcall CloneItem_Translate_RelativeClick(TObject *Sender);
  void __fastcall CloneItem_TranslateClick(TObject *Sender);
  void __fastcall CloneItem_Visible_OffClick(TObject *Sender);
  void __fastcall CloneItem_Visible_OnClick(TObject *Sender);
  void __fastcall CloneMesh_Size_GetClick(TObject *Sender);
  void __fastcall CloneMesh_Size_SetClick(TObject *Sender);
  void __fastcall CloneMesh_RotateX_RotateClick(TObject *Sender);
  void __fastcall CloneMesh_ChangeClick(TObject *Sender);
  void __fastcall Size_CloneItem_GetClick(TObject *Sender);
  void __fastcall Size_CloneItem_SetClick(TObject *Sender);
  void __fastcall Snap_CloneItem_From_V1Click(TObject *Sender);
  void __fastcall Snap_CloneItem_From_V2Click(TObject *Sender);
  void __fastcall Snap_CloneItem_To_V1Click(TObject *Sender);
  void __fastcall Snap_CloneItem_To_V2Click(TObject *Sender);
  void __fastcall Snap_CloneItem_SnapClick(TObject *Sender);
  void __fastcall CloneMesh_Change_MeshNameClick(TObject *Sender);
  void __fastcall CloneItem_Change_ItemNameClick(TObject *Sender);
  void __fastcall ReadClonesFileClick(TObject *Sender);
  void __fastcall SaveClonesFileClick(TObject *Sender);
  void __fastcall DisplayClonesClick(TObject *Sender);
private:  // User declarations
  void __fastcall Form_Setup(void);
  void __fastcall AppMessage(MSG &Msg, bool &Handled);
  void __fastcall MyIdleHandler(TObject *Sender, bool &Done);
  void __fastcall Start_DLL(void);
  void __fastcall Setup_Rubberband_Parameters(void);
  void __fastcall Setup_Obstruction_Point_Parameters(void);
  void __fastcall Setup_Observer_Parameters(void);
  void __fastcall Setup_Rubberband_Parameters_ML(void);
  void __fastcall Setup_Obstruction_Point_Parameters_ML(void);
  void __fastcall Setup_Observer_Parameters_ML(void);
  void __fastcall Stop_DLL(void);
  void __fastcall Read_3D_File(Int32 multiFlag);
  //
  void __fastcall Display_Locations0(void);
  void __fastcall Display_Locations1(void);
  void __fastcall Display_Locations2(void);
  void __fastcall Display_LocationsObstruction(void);
  void __fastcall Display_LocationsObserver(void);
  void __fastcall Get_Locations0(void);
  void __fastcall Get_Locations1(void);
  void __fastcall Get_Locations2(void);
  void __fastcall Get_LocationsObstruction(void);
  void __fastcall Get_LocationsObserver(void);
  //
  void __fastcall GetCameraStepFOV(void);
  void __fastcall GetCameraRotateSpeed(void);
  void __fastcall GetCameraStepAngle(void);
  void __fastcall GetCameraStepPan(void);
  void __fastcall Get_All_ActiveCamera_Data(void);
  //
  void __fastcall Do_RGB_Panel(Int32 index);
  void __fastcall Do_ColorDialog(Int32* inColorPtr, Int32* outColorPtr);
  void __fastcall Pick_Color(Int32 index);
  void __fastcall Display_Rubberband_Data(Int32 item);
  //
  void __fastcall Display_Roaming_Parameters(void);
  void __fastcall Get_Roaming_Parameters(void);
  void __fastcall Get_Move_AheadBack_Step(void);
  void __fastcall Get_Move_RightLeft_Step(void);
  void __fastcall Get_Move_UpDown_Step(void);
  void __fastcall Get_Move_Turn_Step(void);
  void __fastcall Set_CameraID_List(void);
  void __fastcall Set_ActiveCameraID_List(void);
  //
  MEASURE_DATA* __fastcall Alloc_Measure_Data(void);
  void __fastcall Clear_measureData_List(void);
  void __fastcall Display_Measure_Data(Int32 index, MEASURE_DATA* dataPtr);
  void __fastcall Measure_ID_List_Clear(void);
  void __fastcall Measure_ID_List_Add(char* idPtr);
  //
  double __fastcall Calculate_Length(double* vertex1_Ptr, double* vertex2_Ptr);
  void __fastcall Calculate_Distances(void);
  Int32 __fastcall Add_MeasureList_Item(MEASURE_DATA* dataPtr);
  Int32 __fastcall Get_MeasureList_Item(Int32 index, MEASURE_DATA* dataPtr);
  Int32 __fastcall Replace_MeasureList_Item(Int32 index, char* bufferPtr, MEASURE_DATA* dataPtr);
  //
  void __fastcall Get_Image_From_DLL(Int32 index, char* idPtr);
  //
  void __fastcall Read_Add_File(void);
  //
  void __fastcall Do_AdjustMesh_Translate(Int32 relativeFlag);
  //
  void __fastcall Add_Clone_Mesh(Int32 changeFlag);
  void __fastcall Add_Clone_Item(void);
  void __fastcall Do_CloneItem_Rotate(Int32 relativeFlag);
  void __fastcall Do_CloneItem_Translate(Int32 relativeFlag);
  //
  //
  Int32 sender_W8057_Flag;
  Int32 shift_W8057_Flag;
  Int32 done_W8057_Flag;
  Int32 spare1;                //  padding for align
  Int32 dllOpen;
  Int32 fileLoaded;
  char inName[MAX_PATH];
  Int32 inputFileType;
  Int32 useTexturesFlag;
  Int32 alwaysShowCameraCursorFlag;
  Int32 touchScreenInputFlag;
  //
  float rgbLineArray[3];
  float rgbPoint1Array[3];
  float rgbPoint2Array[3];
  float rgbObstructionArray[3];
  float rgbLineObserverArray[3];
  float rgbPointObserverArray[3];
  float lineWidth;
  float observerWidth;
  float pointSize;
  float pointObstructionSize;
  float observerSize;
  Int32 depthTestFlag;
  Int32 pointsFlag;
  Int32 obstructionDisplayFlag;
  Int32 observerLineFlag;
  Int32 observerPointFlag;
  Int32 haveObserverFlag;
  Int32 haveObstructionMessageFlag;
  //
  Int32 screenX0;
  Int32 screenX1;
  Int32 screenX2;
  Int32 screenXObstruction;
  Int32 screenXObserver;
  Int32 screenY0;
  Int32 screenY1;
  Int32 screenY2;
  Int32 screenYObstruction;
  Int32 screenYObserver;
  Int32 hitFlag0;
  Int32 hitFlag1;
  Int32 hitFlag2;
  Int32 hitFlagObstruction;
  Int32 hitFlagObserver;
  double xyzDeltaArrayObserver[3];
  double rubberbandDistance;
  double obstructionDistance;
  double xyzArray0[3];
  double xyzArray1[3];
  double xyzArray2[3];
  double xyzArrayObstruction[3];
  double xyzArrayObserver[3];
  double obstructionSkipDistance;
  //
  double cameraXViewpoint;
  double cameraYViewpoint;
  double cameraZViewpoint;
  //
  double cameraFieldOfView;
  double cameraStepFOV;
  //
  double cameraXAngle;
  double cameraYAngle;
  double cameraZAngle;
  double cameraStepAngle;
  //
  Int32 cameraRotateSpeed;
  //
  Int32 cameraXPan;
  Int32 cameraZPan;
  Int32 cameraStepPan;
  Int32 reversePanDirectionFlag;
  //
  Int32 reverseZoomDirectionFlag;
  Int32 cameraZoomSpeed;
  Int32 zoomAutomaticFlag;
  Int32 roamingZoomLock;
  Int32 cameraTotal;
  Int32 activeCameraID;
  Int32 cameraID;
  Int32 getFromCameraID;
  Int32 roaming_asDisplayed_Flag;
  double cameraInitialDistance;
  double cameraDistance;
  //
  Int32 xRotation_Lock_Flag;
  Int32 yRotation_Lock_Flag;
  Int32 zRotation_Lock_Flag;
  Int32 xRotation_Constrain_Flag;
  Int32 yRotation_Constrain_Flag;
  Int32 zRotation_Constrain_Flag;
  double xRotation_Constrain_From;
  double xRotation_Constrain_To;
  double yRotation_Constrain_From;
  double yRotation_Constrain_To;
  double zRotation_Constrain_From;
  double zRotation_Constrain_To;
  //
  double roaming_Eye[3];
  double roaming_Center[3];
  double roaming_Up[3];
  //
  double moveAheadBack_Step;
  double moveRightLeft_Step;
  double moveUpDown_Step;
  double moveTurn_Step;
  //
  Int32 gpu_Max_Texture_Size;
  Int32 user_Max_Texture_Size;
  Int32 powerOf2_Max_Texture_Size;
  Int32 spare2;                      // padding for align
  //
  float rgbLineArray_ML[3];
  float rgbPoint1Array_ML[3];
  float rgbPoint2Array_ML[3];
  float rgbObstructionArray_ML[3];
  float rgbLineObserverArray_ML[3];
  float rgbPointObserverArray_ML[3];
  float lineWidth_ML;
  float observerWidth_ML;
  float pointSize_ML;
  float pointObstructionSize_ML;
  float observerSize_ML;
  Int32 depthTestFlag_ML;
  Int32 pointsFlag_ML;
  Int32 obstructionDisplayFlag_ML;
  Int32 observerLineFlag_ML;
  Int32 observerPointFlag_ML;
  //
#ifdef USING_WIN32
  Int32 w32_pad1;        // padding for align
#endif
  Int32 fontColor;
  Int32 measureColorFlag;
  TList* measureData_ListPtr;
  MEASURE_BILLBOARD billboardData;
  UInt8* billboardImagePtr;
  Int32 billboardImageWidth;
  Int32 billboardImageHeight;
  Int32 billboardAlphaFlag;
  //
  char clonesFilename[MAX_PATH];
public:  // User declarations
  __fastcall TTest_At3dx_LocationsForm(TComponent* Owner);
};
//---------------------------------------------------------------------------
extern PACKAGE TTest_At3dx_LocationsForm *Test_At3dx_LocationsForm;
//---------------------------------------------------------------------------
#endif
