//----------------------------------------------------------------------------
//  Project Test_At3dx_Read_Write
//  MicroMouse Productions
//  Copyright  2005-2020. All Rights Reserved.
//  Author: Wayne Hogue
//
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <stdlib.h>
#include <stdio.h>

#include "Test_At3dx_Read_WriteApp.h"
#include "at3dx_my_functions.h"
#include "at3ddll.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTest_At3dx_Read_WriteForm *Test_At3dx_Read_WriteForm;
//---------------------------------------------------------------------------

__fastcall TTest_At3dx_Read_WriteForm::TTest_At3dx_Read_WriteForm(TComponent* Owner)
  : TForm(Owner)
{
  unused_W8057_Flag = 0;

  memset(inName, 0, sizeof(char) * MAX_PATH);

  // Save 2D lines if they exist in the file
  //
  save2D_Flag    = 1;

  // Save UV coordinates if they exist in the file
  //
  saveUV_Flag    = 1;

  // Save Quads if they exist in the file
  //
  saveQuads_Flag = 1;

  // Option to turn triangles into quads
  //
  makeQuads_Flag = 0;

  angleBetweenNormals = 5.0;
  angleBetweenEdges   = 130.0;

  // --

  Form_Setup();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_Read_WriteForm::Form_Setup(void)
{
#if defined(_WIN64)
  #ifdef _DEBUG
    const char* cptr = " (x64-Debug)";
  #else
    const char* cptr = " (x64)";
  #endif
#else
  #ifdef _DEBUG
    const char* cptr = " (x32-Debug)";
  #else
    const char* cptr = " (x32)";
  #endif
#endif

  Caption = Caption + cptr;

  Save2DLines->Checked = (save2D_Flag) ? 1 : 0;

  SaveUV->Checked = (saveUV_Flag) ? 1 : 0;

  SaveQuads->Checked = (saveQuads_Flag) ? 1 : 0;

  MakeQuads->Checked = (makeQuads_Flag) ? 1 : 0;

  AngleBetweenNormals->Text = angleBetweenNormals;
  AngleBetweenEdges->Text   = angleBetweenEdges;
}
//---------------------------------------------------------------------------


void __fastcall TTest_At3dx_Read_WriteForm::Save2DLinesClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  save2D_Flag = (Save2DLines->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_Read_WriteForm::SaveUVClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  saveUV_Flag = (SaveUV->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_Read_WriteForm::SaveQuadsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  saveQuads_Flag = (SaveQuads->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_Read_WriteForm::MakeQuadsClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  makeQuads_Flag = (MakeQuads->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_Read_WriteForm::ReadWriteFileClick(TObject *Sender)
{
  unused_W8057_Flag = (Sender) ? 1 : 0;

  Read_Write_3D_File();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_Read_WriteForm::Read_Write_3D_File(void)
{
  // start the DLL
  //
  int dllOpen = At3d_Start();

  if (!dllOpen) return;

  // View window will not be used so do not draw to it
  //
  At3d_WriteFlag(LIBFLAG_NO_DRAW, 1);

  // Read OBJ file
  //
  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if (inName[0] != 0)
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ext0);
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  AnsiString filter = "Wavefront";
  filter += "|*";
  filter += ".obj";
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  int fileLoaded = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;
    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);

    my_strcpy(inName, MAX_PATH, fileName.c_str());

    // before reading make sure flags are set properly
    // (note: flags maintain their setting until changed with this function)
    //
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_VERTICES, 1);
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_FACES, 1);
    At3d_WriteFlag(LIBFLAG_SHOW_READ_MESSAGES, 0);

    // read file
    //
    fileLoaded = At3d_Read(INTYPE_WAVEFRONT, inName, 1.0);
  }

  delete dlg;

  // Write LXO file
  //
  if (fileLoaded)
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    char outName[MAX_PATH];

    my_makepath(outName, MAX_PATH, drive0, dir0,  name0, ".lxo");

    At3d_WriteFlag(LIBFLAG_SAVE_2D, save2D_Flag);

    At3d_WriteFlag(LIBFLAG_UV_OUTPUT, saveUV_Flag);

    At3d_WriteFlag(LIBFLAG_USE_QUADFACE, saveQuads_Flag);

    if ((saveQuads_Flag) && (makeQuads_Flag))
    {
      try
      {
        angleBetweenNormals = AngleBetweenNormals->Text.ToDouble();
      }
      catch(...)
      {
        angleBetweenNormals = 0.0;
      }

      try
      {
        angleBetweenEdges = AngleBetweenEdges->Text.ToDouble();
      }
      catch(...)
      {
        angleBetweenEdges = 0.0;
      }

      At3d_SetTriQuadAngle(angleBetweenNormals);
      At3d_SetTriQuadEdgeAngle(angleBetweenEdges);
      At3d_MakeQuads();
    }

    At3d_Write(OUTTYPE_MODO, outName, 1.0);
  }

  // free all memory used by the DLL
  //
  At3d_Stop();
}
//---------------------------------------------------------------------------


