//----------------------------------------------------------------------------
//  Project Test_At3dx_Viewer
//  MicroMouse Productions
//  Copyright  2010-2020. All Rights Reserved.
//  Author: Wayne Hogue
//
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <stdlib.h>
#include <stdio.h>
#include <dos.h>

#include "Test_At3dx_LocationsApp.h"
#include "at3dx_my_functions.h"
#include "at3ddll.h"

#define COLOR_PANEL_LINE              0
#define COLOR_PANEL_VERTEX1           1
#define COLOR_PANEL_VERTEX2           2
#define COLOR_PANEL_OBSTRUCTION       3
#define COLOR_PANEL_OBSERVER_LINE     4
#define COLOR_PANEL_OBSERVER_VERTEX   5

#define COLOR_PANEL_LINE_ML              6
#define COLOR_PANEL_VERTEX1_ML           7
#define COLOR_PANEL_VERTEX2_ML           8
#define COLOR_PANEL_OBSTRUCTION_ML       9
#define COLOR_PANEL_OBSERVER_LINE_ML    10
#define COLOR_PANEL_OBSERVER_VERTEX_ML  11

#define DISPLAY_RUBBER_CLEAR_ALL       -1
//#define DISPLAY_RUBBER_MOUSE_HOVER      0
//#define DISPLAY_RUBBER_MOUSE_DOWN       1
//#define DISPLAY_RUBBER_MOUSE_MOVE       2
#define DISPLAY_RUBBER_MOUSE_UP         3
#define DISPLAY_RUBBER_MOUSE_SWAP       4
#define DISPLAY_RUBBER_MOUSE_CLIP       5
#define DISPLAY_RUBBER_OBSERVER_ADD     6
#define DISPLAY_RUBBER_OBSERVER_CANCEL  7

// used when adding static mesh or clone mesh
//
#define ROTATE_ON_X_AXIS_NO            0
#define ROTATE_ON_X_AXIS_MINUS_90      1
#define ROTATE_ON_X_AXIS_PLUS_90       2
#define ROTATE_ON_X_AXIS_180           3

//---------------------------------------------------------------------------

// To be able to send XYZ locations for where the mouse is clicked on a mesh
// the DLL has to have a message number to  use in the Windows OS PostMessage(...)
// this number must be >= WM_USER.  See Windows documentation for more
// information on the PostMessage(...) function
// Select a message number that does not conflict with any other message numbers
// that your application may be using
//
#define WM_MESH_POSITION    WM_USER + 7

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTest_At3dx_LocationsForm *Test_At3dx_LocationsForm;
//---------------------------------------------------------------------------

__fastcall TTest_At3dx_LocationsForm::TTest_At3dx_LocationsForm(TComponent* Owner)
  : TForm(Owner)
{
  sender_W8057_Flag = 0;
  shift_W8057_Flag  = 0;
  done_W8057_Flag   = 0;

  spare1 = 0;

  dllOpen    = 0;
  fileLoaded = 0;

  my_memset(inName, 0, sizeof(char) * MAX_PATH);

  inputFileType = 0;

  // Display textures
  //
  useTexturesFlag = 1;

  // default setting for camera cursor
  //
  alwaysShowCameraCursorFlag = 0;

  // if set, touch screen is available
  //
  touchScreenInputFlag = 0;

  // --

  rgbLineArray[0] = 1.0f;
  rgbLineArray[1] = 0.0f;
  rgbLineArray[2] = 0.0f;

  rgbPoint1Array[0] = 1.0f;
  rgbPoint1Array[1] = 0.0f;
  rgbPoint1Array[2] = 1.0f;

  rgbPoint2Array[0] = 0.0f;
  rgbPoint2Array[1] = 1.0f;
  rgbPoint2Array[2] = 1.0f;

  rgbObstructionArray[0] = 1.0f;
  rgbObstructionArray[1] = 1.0f;
  rgbObstructionArray[2] = 0.0f;

  rgbLineObserverArray[0] = 0.0f;
  rgbLineObserverArray[1] = 0.0f;
  rgbLineObserverArray[2] = 1.0f;

  rgbPointObserverArray[0] = 0.0f;
  rgbPointObserverArray[1] = 1.0f;
  rgbPointObserverArray[2] = 0.0f;

  lineWidth     = 2.0f;
  observerWidth = 2.0f;

  pointSize = 4.0f;
  pointObstructionSize = 4.0f;
  observerSize = 4.0f;

  depthTestFlag = 1;
  pointsFlag    = 1;

  obstructionDisplayFlag     = 1;
  observerLineFlag           = 1;
  observerPointFlag          = 1;
  haveObserverFlag           = 0;
  haveObstructionMessageFlag = 0;

  // --

  screenX0 = 0;
  screenX1 = 0;
  screenX2 = 0;
  screenXObstruction = 0;
  screenXObserver    = 0;

  screenY0 = 0;
  screenY1 = 0;
  screenY2 = 0;
  screenYObstruction = 0;
  screenYObserver    = 0;

  hitFlag0 = MESH_LOCATION_NO_DATA;
  hitFlag1 = MESH_LOCATION_NO_DATA;
  hitFlag2 = MESH_LOCATION_NO_DATA;
  hitFlagObstruction = MESH_LOCATION_NO_DATA;
  hitFlagObserver    = MESH_LOCATION_NO_DATA;

  for (Int32 i=0; i<3; i++)
  {
    xyzArray0[i] = 0.0;
    xyzArray1[i] = 0.0;
    xyzArray2[i] = 0.0;
    xyzArrayObstruction[i] = 0.0;
    xyzArrayObserver[i]    = 0.0;

    xyzDeltaArrayObserver[i] = 0.0;
  }

  rubberbandDistance  = 0.0;
  obstructionDistance = 0.0;

  obstructionSkipDistance = 0.0;

  // --

  cameraXViewpoint = 0.0;
  cameraYViewpoint = 0.0;
  cameraZViewpoint = 0.0;

  cameraFieldOfView = 30.0;
  cameraStepFOV     = 1.0;

  cameraRotateSpeed = CAMERA_ROTATE_SPEED_2;

  cameraXAngle      = 0.0;
  cameraYAngle      = 0.0;
  cameraZAngle      = 0.0;
  cameraStepAngle   = 5.0;

  cameraXPan    = 0;
  cameraZPan    = 0;
  cameraStepPan = 1;

  reversePanDirectionFlag  = 0;

  reverseZoomDirectionFlag = 0;

  cameraZoomSpeed = CAMERA_ZOOM_SPEED_6;

  roamingZoomLock = ROAMING_LOCK_ZOOM_BOTH;

  zoomAutomaticFlag = 1;

  cameraTotal     = 0;
  activeCameraID  = 0;
  cameraID        = 0;
  getFromCameraID = 0;

  roaming_asDisplayed_Flag = 0;

  cameraInitialDistance    = 0.0;
  cameraDistance           = 0.0;

  xRotation_Lock_Flag = 0;
  yRotation_Lock_Flag = 0;
  zRotation_Lock_Flag = 0;

  xRotation_Constrain_Flag = 0;
  yRotation_Constrain_Flag = 0;
  zRotation_Constrain_Flag = 0;

  xRotation_Constrain_From = 0.0;
  xRotation_Constrain_To   = 0.0;

  yRotation_Constrain_From = 0.0;
  yRotation_Constrain_To   = 0.0;

  zRotation_Constrain_From = 0.0;
  zRotation_Constrain_To   = 0.0;

  for (Int32 i=0; i<3; i++)
  {
    roaming_Eye[i]    = 0.0;
    roaming_Center[i] = 0.0;
    roaming_Up[i]     = 0.0;
  }

  moveAheadBack_Step = 1.0;
  moveRightLeft_Step = 0.1;
  moveUpDown_Step    = 1.0;
  moveTurn_Step      = 5.0;

  gpu_Max_Texture_Size      = 0;
  user_Max_Texture_Size     = 0;
  powerOf2_Max_Texture_Size = 0;

  spare2 = 0;

  // --

  rgbLineArray_ML[0] = 0.616f;
  rgbLineArray_ML[1] = 0.094f;
  rgbLineArray_ML[2] = 0.067f;

  rgbPoint1Array_ML[0] = 0.714f;
  rgbPoint1Array_ML[1] = 0.047f;
  rgbPoint1Array_ML[2] = 0.714f;

  rgbPoint2Array_ML[0] = 0.0f;
  rgbPoint2Array_ML[1] = 0.616f;
  rgbPoint2Array_ML[2] = 0.616f;

  rgbObstructionArray_ML[0] = 0.733f;
  rgbObstructionArray_ML[1] = 0.733f;
  rgbObstructionArray_ML[2] = 0.0f;

  rgbLineObserverArray_ML[0] = 0.0f;
  rgbLineObserverArray_ML[1] = 0.502f;
  rgbLineObserverArray_ML[2] = 1.0f;

  rgbPointObserverArray_ML[0] = 0.0f;
  rgbPointObserverArray_ML[1] = 0.502f;
  rgbPointObserverArray_ML[2] = 0.0f;

  lineWidth_ML     = 2.0f;
  observerWidth_ML = 2.0f;

  pointSize_ML = 4.0f;
  pointObstructionSize_ML = 4.0f;
  observerSize_ML = 4.0f;

  depthTestFlag_ML = 1;
  pointsFlag_ML    = 1;

  obstructionDisplayFlag_ML = 1;
  observerLineFlag_ML       = 1;
  observerPointFlag_ML      = 1;

#ifdef USING_WIN32
  w32_pad1 = 0;
#endif

  fontColor = 0;
  measureColorFlag = 0;

  measureData_ListPtr = MY_NULL_PTR;

  my_memset(&billboardData, 0, sizeof(MEASURE_BILLBOARD));

  billboardImagePtr    = MY_NULL_PTR;
  billboardImageWidth  = 0;
  billboardImageHeight = 0;
  billboardAlphaFlag   = 0;

  // --

  my_memset(clonesFilename, 0, sizeof(char) * MAX_PATH);

  // --

  Form_Setup();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Form_Setup(void)
{
#if defined(_WIN64)
  #ifdef _DEBUG
    const char* cptr = " (x64-Debug)";
  #else
    const char* cptr = " (x64)";
  #endif
#else
  #ifdef _DEBUG
    const char* cptr = " (x32-Debug)";
  #else
    const char* cptr = " (x32)";
  #endif
#endif

  Caption = Caption + cptr;

  Application->OnMessage = AppMessage;
  Application->OnIdle    = MyIdleHandler;

  // --

  try
  {
    measureData_ListPtr = new TList;
  }
  catch (...)
  {
    measureData_ListPtr = MY_NULL_PTR;
  }

  // clear all location data on display
  //
  Display_Rubberband_Data(DISPLAY_RUBBER_CLEAR_ALL);

  // set intial file type to read
  //
  if (!inputFileType)
  {
    TypeObj->Checked = 1;
  }
  else
  {
    TypeWrl->Checked = 1;
  }

  // --

  Start_DLL();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::FormDestroy(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Clear_measureData_List();
  if (measureData_ListPtr) delete measureData_ListPtr;

  if (billboardData.imagePtr) delete[] billboardData.imagePtr;

  if (billboardImagePtr) delete[] billboardImagePtr;

  // free all memory used by the DLL
  //
  if (dllOpen) Stop_DLL();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AppMessage(MSG &Msg, bool &Handled)
{
  if ( (Msg.message == WM_MESH_POSITION) &&
       (dllOpen) )
  {
    if (Msg.lParam == 1)
    {
      if (Msg.wParam == 3)
      {
        // event: left button up
        //
        Display_Rubberband_Data(DISPLAY_RUBBER_MOUSE_UP);
      }
      else if ( (Msg.wParam > 999) &&
                (Msg.wParam < 2000) )
      {
        // measurement line has been selected
        //
        MEASURE_DATA data;

        Int32 result = Get_MeasureList_Item( static_cast<Int32>(Msg.wParam) - 1000, &data );

        Display_Measure_Data(result, &data);
      }
      else if (Msg.wParam > 1999)
      {
        // measurement billboard has been selected
        //
        char buffer[LEN_LAYER_NAME];

        if (At3d_Measure_Billboard_Get_ID(static_cast<Int32>(Msg.wParam) - 2000, buffer) > 0)
        {
          Billboard_Selected_ID->Text = AnsiString(buffer);
        }
      }
    }

    Handled = true;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MyIdleHandler(TObject *Sender, bool &Done)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;
  done_W8057_Flag   = (Done) ? 1 : 0;

  // --

  ZoomAutomatic->Down = zoomAutomaticFlag > 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read3dFileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Screen->Cursor = TCursor(crHourGlass);
  Read_3D_File(0);
  Screen->Cursor = TCursor(crDefault);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MultiReadClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Screen->Cursor = TCursor(crHourGlass);
  Read_3D_File(1);
  Screen->Cursor = TCursor(crDefault);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Start_DLL(void)
{
  if (dllOpen) return;

  dllOpen = At3d_Start();

  if (!dllOpen) return;

  // To detect mouse scroll wheel messages the DLL has to know the
  // handle to the main window of the application which
  // receives all the messages from the Windows Operating System
  //
  //At3d_SetAppHandle(Handle);

  // To be able to send XYZ locations for where the mouse is clicked on a mesh
  // the DLL has to know the handle to the main window of the application which
  // receives all the messages from the Windows Operating System
  // plus a message number to indicate the type of message being sent
  //
  At3d_Init_Mesh_Location(Handle, WM_MESH_POSITION);

  // Set parameters for rubberband line
  //
  Setup_Rubberband_Parameters();

  // Set parameters for obstuction point on rubberband line
  //
  Setup_Obstruction_Point_Parameters();

  // set parameters for observer on rubberband line
  //
  Setup_Observer_Parameters();

  // set deltas for obseerver on rubberband line
  //
  Observer_DeltaX->Text = xyzDeltaArrayObserver[0];
  Observer_DeltaY->Text = xyzDeltaArrayObserver[1];
  Observer_DeltaZ->Text = xyzDeltaArrayObserver[2];

  // Set size of DLL window
  //
  At3d_Size(600, 500);

  // Set position of DLL window
  //
  At3d_Position(16, 26);

  // Make DLL window a child of a window in the application
  //
  At3d_SetParent(ViewPanel->Handle);

  // If the DLL window does not have a parent
  // At3d_SetParent(0);
  // a button will appear on the Windows Taskbar
  // use this function to set the name that will appear
  //
  At3d_SetCaption("DLL (at3d)");

  // Use the OpenGL view window
  //
  At3d_WriteFlag(LIBFLAG_USE_OPENGL, 1);

  // Light both sides of the polygons
  //
  At3d_WriteFlag(LIBFLAG_OGL_2SIDED_LIGHTING, 1);

  // Cull back faces of polygons
  //
  At3d_WriteFlag(LIBFLAG_OGL_CULL_BACKFACES,1);

  UseTextures->Checked = (useTexturesFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_OGL_USE_TEXTURES, useTexturesFlag);

  // Display default texture if there is no texture file assigned
  //
  At3d_WriteFlag(LIBFLAG_OGL_USE_DEFAULT_TEXTURE, 1);

  // Let texture colors be affected by the lighting
  //
  At3d_WriteFlag(LIBFLAG_OGL_REPLACE, 0);

  // Do not blend polygon color with the texture colors
  //
  At3d_WriteFlag(LIBFLAG_OGL_WHITE_BASE, 1);

  AlwaysShowCameraCursor->Checked = (alwaysShowCameraCursorFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_ALWAYS_SHOW_CAMERA_CURSOR, alwaysShowCameraCursorFlag);

  TouchScreenInput->Checked = (touchScreenInputFlag) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_TOUCH_SCREEN_INPUT, touchScreenInputFlag);

  // Measuring distances (LIBFLAG_DISPLAY_MESH_LOCATION)
  // between locations on mesh is OFF by default
  //
  Locations_Off->Checked = 1;

  // Measuring mode of selecting vertices is DRAG by default
  //
  Locations_Drag->Checked = 1;

  // --

  CameraXViewpoint->Text = cameraXViewpoint;
  CameraYViewpoint->Text = cameraYViewpoint;
  CameraZViewpoint->Text = cameraZViewpoint;

  CameraFieldOfView->Text = cameraFieldOfView;
  CameraStepFOV->Text     = cameraStepFOV;

  if (cameraRotateSpeed == CAMERA_ROTATE_SPEED_1)
  {
    CameraRotateSpeed->Text = "R1";
  }
  else
  {
    CameraRotateSpeed->Text = "R2";
  }

  CameraXAngle->Text = cameraXAngle;
  CameraYAngle->Text = cameraYAngle;
  CameraZAngle->Text = cameraZAngle;

  CameraStepAngle->Text = cameraStepAngle;

  CameraXPan->Text = cameraXPan;
  CameraZPan->Text = cameraZPan;

  CameraStepPan->Text = cameraStepPan;

  CameraInitialDistance->Text = cameraInitialDistance;
  CameraDistance->Text        = cameraDistance;

  if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_1)
  {
    CameraZoomSpeed->Text = "Z1";
  }
  else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_2)
  {
    CameraZoomSpeed->Text = "Z2";
  }
  else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_3)
  {
    CameraZoomSpeed->Text = "Z3";
  }
  else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_4)
  {
    CameraZoomSpeed->Text = "Z4";
  }
  else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_5)
  {
    CameraZoomSpeed->Text = "Z5";
  }
  else
  {
    CameraZoomSpeed->Text = "Z6";
  }

  At3d_WriteFlag(LIBFLAG_CAMERA_ZOOM_SPEED_AUTOMATIC, zoomAutomaticFlag);

  if (roamingZoomLock == ROAMING_LOCK_ZOOM_NONE)
  {
    Zoom_Lock_None->Checked = 1;
  }
  else if (roamingZoomLock == ROAMING_LOCK_ZOOM_IN)
  {
    Zoom_Lock_In->Checked = 1;
  }
  else if (roamingZoomLock == ROAMING_LOCK_ZOOM_OUT)
  {
    Zoom_Lock_Out->Checked = 1;
  }
  else //if (roamingZoomLock == ROAMING_LOCK_ZOOM_BOTH)
  {
    Zoom_Lock_Both->Checked = 1;
  }

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);

  // create roaming cameras
  //
  cameraTotal = At3d_Add_Roaming_Cameras(3);
  Total_Cameras->Text = cameraTotal;
  if (cameraTotal > 1)
  {
    cameraID = 1;
  }
  CameraID->Text = cameraID;
  Get_From_CameraID->Text = getFromCameraID;

  Set_CameraID_List();
  Set_ActiveCameraID_List();

  Display_Roaming_Parameters();

  Move_AheadBack_Step->Text = moveAheadBack_Step;
  Move_RightLeft_Step->Text = moveRightLeft_Step;
  Move_UpDown_Step->Text    = moveUpDown_Step;
  Move_Turn_Step->Text      = moveTurn_Step;

  // Set parameters for rubberband line, obstruction point and observer
  // for a series of measurements stored in a list
  //
  Setup_Rubberband_Parameters_ML();
  Setup_Obstruction_Point_Parameters_ML();
  Setup_Observer_Parameters_ML();

  // --

  // By default, set viewpoint is set to use middle mouse button
  //
  SetViewpoint_Middle->Checked = 1;

  // By default, left button toggle is set to rotate
  //
  LeftToggle_Rotate->Checked = 1;

  // --

  // get axis rotation parameters
  //
  Get_Axis_XClick(MY_NULL_PTR);
  Get_Axis_YClick(MY_NULL_PTR);
  Get_Axis_ZClick(MY_NULL_PTR);

  // --

  ImageWidth->Text  = 600;
  ImageHeight->Text = 500;

  ImageLineWidth->Text = 1.0;
  ImagePointSize->Text = 1.0;

  // --

  GPUMaxTextureSize->Text       = gpu_Max_Texture_Size;
  UserMaxTextureSize->Text      = user_Max_Texture_Size;
  PowerOf2_MaxTextureSize->Text = powerOf2_Max_Texture_Size;

  // --

  StatusBar_Left->Text   = 60;
  StatusBar_Top->Text    = 474;
  StatusBar_Width->Text  = 480;
  StatusBar_Height->Text = 22;

  FontColorPanel->Color = static_cast<TColor>(fontColor);
  FontSize->Text = 10;

  measureColorFlag = 0;
  MeasureColor_Default->Checked = 1;

  // --

  DisplayBillboards->Checked = 0;

  BillboardFontColor1->Checked = 1;
  BillboardAlpha->Checked = 1;

  // --

  BillboardPivotTypeList->Items->BeginUpdate();

  try
  {
    BillboardPivotTypeList->Clear();

    BillboardPivotTypeList->Items->Add("Center");
    BillboardPivotTypeList->Items->Add("Bottom Left");
    BillboardPivotTypeList->Items->Add("Bottom Center");
    BillboardPivotTypeList->Items->Add("Bottom Right");
    BillboardPivotTypeList->Items->Add("Top Left");
    BillboardPivotTypeList->Items->Add("Top Center");
    BillboardPivotTypeList->Items->Add("Top Right");
    BillboardPivotTypeList->Items->Add("Left Center");
    BillboardPivotTypeList->Items->Add("Right Center");

    BillboardPivotTypeList->ItemIndex = 1;

    BillboardPivotTypeList->Items->EndUpdate();
  }
  catch(...)
  {
    BillboardPivotTypeList->Items->EndUpdate();
  }

  // --

  BillboardStatic->Checked = 0;
  BillboardVisible->Checked = 1;
  BillboardOglReplace->Checked = 1;

  BillboardType->Text = 0;

  BillboardLineWidth->Text = 1.0;

  BillboardHeight->Text = 1.0;

  BillboardPositionX->Text = 0.0;
  BillboardPositionY->Text = 0.0;
  BillboardPositionZ->Text = 0.0;

  BillboardPivotX->Text = 0.0;
  BillboardPivotY->Text = 0.0;
  BillboardPivotZ->Text = 0.0;

  BillboardRotateX->Text = 0.0;
  BillboardRotateY->Text = 0.0;
  BillboardRotateZ->Text = 0.0;

  Billboard_Selected_Red->Text   = 1.0;
  Billboard_Selected_Green->Text = 1.0;
  Billboard_Selected_Blue->Text  = 1.0;

  Billboard_Selected_Width->Text = 4.0;

  Billboard_Line_Red->Text   = 0.0;
  Billboard_Line_Green->Text = 0.0;
  Billboard_Line_Blue->Text  = 0.0;

  Billboard_Line_Width->Text = 0.0;

  // --

  Billboard_PivotTypeList->Items->BeginUpdate();

  try
  {
    Billboard_PivotTypeList->Clear();

    Billboard_PivotTypeList->Items->Add("Center");
    Billboard_PivotTypeList->Items->Add("Bottom Left");
    Billboard_PivotTypeList->Items->Add("Bottom Center");
    Billboard_PivotTypeList->Items->Add("Bottom Right");
    Billboard_PivotTypeList->Items->Add("Top Left");
    Billboard_PivotTypeList->Items->Add("Top Center");
    Billboard_PivotTypeList->Items->Add("Top Right");
    Billboard_PivotTypeList->Items->Add("Left Center");
    Billboard_PivotTypeList->Items->Add("Right Center");

    Billboard_PivotTypeList->ItemIndex = -1;

    Billboard_PivotTypeList->Items->EndUpdate();
  }
  catch(...)
  {
    Billboard_PivotTypeList->Items->EndUpdate();
  }

  // --

  Get_ScreenX->Text = 0;
  Get_ScreenY->Text = 0;

  Get_MeshX->Caption = "";
  Get_MeshY->Caption = "";
  Get_MeshZ->Caption = "";

  Get_LayerName->Caption = "";
  Get_CloneItemName->Caption = "";

  // --

  LineMesh_V1_X->Text = 0.0;
  LineMesh_V1_Y->Text = 0.0;
  LineMesh_V1_Z->Text = 0.0;

  LineMesh_V2_X->Text = 0.0;
  LineMesh_V2_Y->Text = 0.0;
  LineMesh_V2_Z->Text = 0.0;

  LineMesh_SkipDistance->Text = 0.0;

  LineMesh_Intersection_X->Text = 0.0;
  LineMesh_Intersection_Y->Text = 0.0;
  LineMesh_Intersection_Z->Text = 0.0;

  // --

  ReadAddPositionX->Text = 0.0;
  ReadAddPositionY->Text = 0.0;
  ReadAddPositionZ->Text = 0.0;

  ReadAddScaleFactor->Text = 1.0;

  ReadAddRotateX_No->Checked = 1;

  AdjustMesh_PivotX->Text = 0.0;
  AdjustMesh_PivotY->Text = 0.0;
  AdjustMesh_PivotZ->Text = 0.0;

  AdjustMesh_RotateX->Text = 0.0;
  AdjustMesh_RotateY->Text = 0.0;
  AdjustMesh_RotateZ->Text = 0.0;

  AdjustMesh_ScaleX->Text = 1.0;
  AdjustMesh_ScaleY->Text = 1.0;
  AdjustMesh_ScaleZ->Text = 1.0;

  AdjustMesh_TranslateX->Text = 0.0;
  AdjustMesh_TranslateY->Text = 0.0;
  AdjustMesh_TranslateZ->Text = 0.0;

  CloneMesh_SizeX->Text = 0.0;
  CloneMesh_SizeY->Text = 0.0;
  CloneMesh_SizeZ->Text = 0.0;

  CloneMesh_Old_ScaleFactor->Text = 1.0;
  CloneMesh_New_ScaleFactor->Text = 1.0;

  CloneMesh_RotateX_Minus90->Checked = 1;

  // --

  Add_CloneMesh_ScaleFactor->Text = 1.0;

  Add_CloneMesh_RotateX_No->Checked = 1;

  Add_CloneItem_LocationX->Text = 0.0;
  Add_CloneItem_LocationY->Text = 0.0;
  Add_CloneItem_LocationZ->Text = 0.0;

  Add_CloneItem_RotationX->Text = 0.0;
  Add_CloneItem_RotationY->Text = 0.0;
  Add_CloneItem_RotationZ->Text = 0.0;

  Add_CloneItem_ScaleFactor->Text = 1.0;

  CloneItem_RotateX->Text = 0.0;
  CloneItem_RotateY->Text = 0.0;
  CloneItem_RotateZ->Text = 0.0;

  CloneItem_TranslateX->Text = 0.0;
  CloneItem_TranslateY->Text = 0.0;
  CloneItem_TranslateZ->Text = 0.0;

  Data_CloneItem_LocationX->Text = 0.0;
  Data_CloneItem_LocationY->Text = 0.0;
  Data_CloneItem_LocationZ->Text = 0.0;

  Data_CloneItem_RotationX->Text = 0.0;
  Data_CloneItem_RotationY->Text = 0.0;
  Data_CloneItem_RotationZ->Text = 0.0;

  Data_CloneItem_ScaleFactor->Text = 1.0;

  Snap_CloneItem_FromX->Text = 0.0;
  Snap_CloneItem_FromY->Text = 0.0;
  Snap_CloneItem_FromZ->Text = 0.0;

  Snap_CloneItem_ToX->Text = 0.0;
  Snap_CloneItem_ToY->Text = 0.0;
  Snap_CloneItem_ToZ->Text = 0.0;

  Size_CloneItem_OriginalX->Text = 0.0;
  Size_CloneItem_OriginalY->Text = 0.0;
  Size_CloneItem_OriginalZ->Text = 0.0;

  Size_CloneItem_CurrentX->Text = 0.0;
  Size_CloneItem_CurrentY->Text = 0.0;
  Size_CloneItem_CurrentZ->Text = 0.0;

  Size_CloneItem_Old_ScaleFactor->Text = 1.0;

  Size_CloneItem_New_ScaleFactor->Text = 1.0;

  // --

  Int32 cloneVisibleFlag = At3d_ReadFlag(LIBFLAG_DISPLAY_CLONES);
  DisplayClones->Checked = (cloneVisibleFlag) ? 1 : 0;

  // --

  CameraRotateDegrees->Text = 0.5;

  At3d_WriteFlag(LIBFLAG_OGL_2SIDED_LIGHTING, 0);

  At3d_WriteFlag(LIBFLAG_OGL_CULL_BACKFACES, 0);

  // --

  // Make DLL window visible
  //
  At3d_Show();

  // set tab sheet
  //
  PageControl1->ActivePage = LocationsSheet;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Setup_Rubberband_Parameters()
{
  if (!dllOpen) return;

  Do_RGB_Panel(COLOR_PANEL_LINE);
  Do_RGB_Panel(COLOR_PANEL_VERTEX1);
  Do_RGB_Panel(COLOR_PANEL_VERTEX2);

  LineWidth->Text = static_cast<double>(lineWidth);
  PointSize->Text = static_cast<double>(pointSize);

  UseDepthTest->Checked = (depthTestFlag) ? 1 : 0;

  DisplayVertices->Checked = (pointsFlag) ? 1 : 0;

  At3d_Set_Mesh_Location_Parameters(&rgbLineArray[0], &rgbPoint1Array[0],
                                    &rgbPoint2Array[0], lineWidth, pointSize,
                                    depthTestFlag, pointsFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Setup_Obstruction_Point_Parameters(void)
{
  if (!dllOpen) return;

  Do_RGB_Panel(COLOR_PANEL_OBSTRUCTION);

  ObstructionPointSize->Text = static_cast<double>(pointObstructionSize);

  DisplayObstructionVertex->Checked = (obstructionDisplayFlag) ? 1 : 0;

  ObstructionSkipDistance->Text = obstructionSkipDistance;

  At3d_Set_Mesh_Obstruction_Parameters(rgbObstructionArray, pointObstructionSize,
                                       obstructionDisplayFlag, obstructionSkipDistance);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Setup_Observer_Parameters(void)
{
  if (!dllOpen) return;

  Do_RGB_Panel(COLOR_PANEL_OBSERVER_LINE);
  Do_RGB_Panel(COLOR_PANEL_OBSERVER_VERTEX);

  ObserverLineWidth->Text = static_cast<double>(observerWidth);
  ObserverPointSize->Text = static_cast<double>(observerSize);

  DisplayObserverLine->Checked   = (observerLineFlag) ? 1 : 0;
  DisplayObserverVertex->Checked = (observerPointFlag) ? 1 : 0;

  At3d_Set_Mesh_Location_Observer_Parameters( rgbLineObserverArray, rgbPointObserverArray,
                                              observerWidth, observerSize,
                                              observerLineFlag, observerPointFlag );
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Setup_Rubberband_Parameters_ML()
{
  if (!dllOpen) return;

  Do_RGB_Panel(COLOR_PANEL_LINE_ML);
  Do_RGB_Panel(COLOR_PANEL_VERTEX1_ML);
  Do_RGB_Panel(COLOR_PANEL_VERTEX2_ML);

  LineWidth_ML->Text = static_cast<double>(lineWidth_ML);
  PointSize_ML->Text = static_cast<double>(pointSize_ML);

  UseDepthTest_ML->Checked = (depthTestFlag_ML) ? 1 : 0;

  DisplayVertices_ML->Checked = (pointsFlag_ML) ? 1 : 0;

  At3d_MeasureList_Set_Parameters(&rgbLineArray_ML[0], &rgbPoint1Array_ML[0],
                                  &rgbPoint2Array_ML[0], lineWidth_ML, pointSize_ML,
                                  depthTestFlag_ML, pointsFlag_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Setup_Obstruction_Point_Parameters_ML(void)
{
  if (!dllOpen) return;

  Do_RGB_Panel(COLOR_PANEL_OBSTRUCTION_ML);

  ObstructionPointSize_ML->Text = static_cast<double>(pointObstructionSize_ML);

  DisplayObstructionVertex_ML->Checked = (obstructionDisplayFlag_ML) ? 1 : 0;

  At3d_MeasureList_Set_Obstruction_Parameters(rgbObstructionArray_ML,
                                              pointObstructionSize_ML,
                                              obstructionDisplayFlag_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Setup_Observer_Parameters_ML(void)
{
  if (!dllOpen) return;

  Do_RGB_Panel(COLOR_PANEL_OBSERVER_LINE_ML);
  Do_RGB_Panel(COLOR_PANEL_OBSERVER_VERTEX_ML);

  ObserverLineWidth_ML->Text = static_cast<double>(observerWidth_ML);
  ObserverPointSize_ML->Text = static_cast<double>(observerSize_ML);

  DisplayObserverLine_ML->Checked   = (observerLineFlag_ML) ? 1 : 0;
  DisplayObserverVertex_ML->Checked = (observerPointFlag_ML) ? 1 : 0;

  At3d_MeasureList_Set_Observer_Parameters( rgbLineObserverArray_ML,
                                            rgbPointObserverArray_ML,
                                            observerWidth_ML, observerSize_ML,
                                            observerLineFlag_ML, observerPointFlag_ML );
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Stop_DLL(void)
{
  if (!dllOpen) return;

  // free all memory used by the DLL
  //
  At3d_Stop();
  dllOpen = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read_3D_File(Int32 multiFlag)
{
  if (!dllOpen) return;

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if (inName[0] != '\0')
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ext0);
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr;
  const char* inputExtensionPtr;
  Int32 inputType;

  if (!inputFileType)
  {
    inputNamePtr      = "Wavefront";
    inputExtensionPtr = ".obj";
    inputType         = INTYPE_WAVEFRONT;
  }
  else
  {
    inputNamePtr      = "VRML";
    inputExtensionPtr = ".wrl";
    inputType         = INTYPE_VRML;
  }

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  if (multiFlag)
  {
    dlg->Options = dlg->Options << ofAllowMultiSelect;
  }

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;
    Int32 errorFlag = 0;
    Int32 count = 0;
    char** filesList = MY_NULL_PTR;

    if (multiFlag)
    {
      count = dlg->Files->Count;

      try
      {
        filesList = new char*[static_cast<size_t>(count)];
      }
      catch(...)
      {
        filesList = MY_NULL_PTR;
      }

      if (filesList)
      {
        my_memset(filesList, 0, sizeof(char**) * static_cast<size_t>(count));

        for (Int32 i=0; i<count; i++)
        {
          try
          {
            filesList[i] = new char[LEN_FILE_PATH];
          }
          catch(...)
          {
            filesList[i] = MY_NULL_PTR;
            errorFlag++;
            break;
          }
        }

        if (!errorFlag)
        {
          for (Int32 i=0; i<count; i++)
          {
            my_copystring_to_ansi_from_vcl(&fileName, dlg->Files->Strings[i]);
            my_strcpy(filesList[i], MAX_PATH, fileName.c_str());
          }
        }
      }
    }
    else
    {
      my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
      my_strcpy(inName, MAX_PATH, fileName.c_str());
    }

    // before reading make sure flags are set properly
    // (note: flags maintain their setting until changed with this function)
    //
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_VERTICES, 1);
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_FACES, 1);
    At3d_WriteFlag(LIBFLAG_SHOW_READ_MESSAGES, 0);

    // Set to display messages in statusbar for
    // removing duplicate vertices, edges and polygons
    //
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATES_MSG, 1);

    // read file
    //
    if (multiFlag)
    {
      if ( (filesList) && (!errorFlag) )
      {
        fileLoaded = At3d_Multi_Read(inputType, count, LEN_FILE_PATH,
                                      &filesList[0], 1.0);
      }
    }
    else
    {
      fileLoaded = At3d_Read(inputType, inName, 1.0);
    }

    if (fileLoaded)
    {
      Get_All_ActiveCamera_Data();

      // Measuring distances (LIBFLAG_DISPLAY_MESH_LOCATION)
      // between locations on mesh is turned OFF when a file is cleared from
      // memory when a new file is read
      //
      // Had to add the PostMessage(...) functions for the radio buttons
      // to work correctly if Measure, Pause or Continue radio button
      // had been selected before the file was read
      //
      PostMessage(Locations_Off->Handle, WM_LBUTTONDOWN, 0, 0);
      PostMessage(Locations_Off->Handle, WM_LBUTTONUP, 0, 0);
      Locations_Off->Checked = 1;
    }

    if (filesList)
    {
      for (Int32 i=0; i<count; i++)
      {
        if (filesList[i])
        {
          delete[] filesList[i];
        }
      }

      delete[] filesList;
    }
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UseTexturesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  useTexturesFlag = (UseTextures->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_OGL_USE_TEXTURES, useTexturesFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AlwaysShowCameraCursorClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  alwaysShowCameraCursorFlag = (AlwaysShowCameraCursor->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_ALWAYS_SHOW_CAMERA_CURSOR, alwaysShowCameraCursorFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_OffClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (Locations_Off->Checked)
  {
    At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_LOCATION, DISPLAY_MESH_LOCATION_OFF);

    // clear all location data on display
    //
    Display_Rubberband_Data(DISPLAY_RUBBER_CLEAR_ALL);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_MeasureClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (Locations_Measure->Checked)
  {
    At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_LOCATION, DISPLAY_MESH_LOCATION_MEASURE);

    UpdateScreenXYClick(MY_NULL_PTR);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_PauseClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (Locations_Pause->Checked)
  {
    At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_LOCATION, DISPLAY_MESH_LOCATION_PAUSE);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_ContinueClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (Locations_Continue->Checked)
  {
    At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_LOCATION, DISPLAY_MESH_LOCATION_CONTINUE);

    UpdateScreenXYClick(MY_NULL_PTR);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Locations0(void)
{
  // now empty of code
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Locations0(void)
{
  Int32 result = At3d_Get_Mesh_Location(0, &hitFlag0, &xyzArray0[0],
                                        &screenX0, &screenY0);

  if (!result)
  {
    hitFlag0 = MESH_LOCATION_NO_DATA;
  }
  
  Display_Locations0();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Locations1(void)
{
  if (hitFlag1 == MESH_LOCATION_NO_DATA)
  {
    Location1_ScreenX->Caption = "";
    Location1_ScreenY->Caption = "";

    Location1_MeshX->Caption = "";
    Location1_MeshY->Caption = "";
    Location1_MeshZ->Caption = "";
  }
  else
  {
    Location1_ScreenX->Caption = screenX1;
    Location1_ScreenY->Caption = screenY1;

    if (hitFlag1 == MESH_LOCATION_HAS_XYZ)
    {
      Location1_MeshX->Caption = xyzArray1[0];
      Location1_MeshY->Caption = xyzArray1[1];
      Location1_MeshZ->Caption = xyzArray1[2];

      if (haveObserverFlag)
      {
        Get_LocationsObserver();
      }
    }
    else
    {
      Location1_MeshX->Caption = "";
      Location1_MeshY->Caption = "";
      Location1_MeshZ->Caption = "";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Locations1(void)
{
  Int32 result = At3d_Get_Mesh_Location(1, &hitFlag1, &xyzArray1[0],
                                        &screenX1, &screenY1);

  if (!result)
  {
    hitFlag1 = MESH_LOCATION_NO_DATA;
  }

  Display_Locations1();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Locations2(void)
{
  if (hitFlag2 == MESH_LOCATION_NO_DATA)
  {
    Location2_ScreenX->Caption = "";
    Location2_ScreenY->Caption = "";

    Location2_MeshX->Caption = "";
    Location2_MeshY->Caption = "";
    Location2_MeshZ->Caption = "";

    Locations_Distance->Caption = "";
  }
  else
  {
    Location2_ScreenX->Caption = screenX2;
    Location2_ScreenY->Caption = screenY2;

    if (hitFlag2 == MESH_LOCATION_HAS_XYZ)
    {
      Location2_MeshX->Caption = xyzArray2[0];
      Location2_MeshY->Caption = xyzArray2[1];
      Location2_MeshZ->Caption = xyzArray2[2];

      Locations_Distance->Caption = rubberbandDistance;
    }
    else
    {
      Location2_MeshX->Caption = "";
      Location2_MeshY->Caption = "";
      Location2_MeshZ->Caption = "";

      Locations_Distance->Caption = "";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Locations2(void)
{
  Int32 result = At3d_Get_Mesh_Location(2, &hitFlag2, &xyzArray2[0],
                                        &screenX2, &screenY2);
  if (!result)
  {
    hitFlag2 = MESH_LOCATION_NO_DATA;
  }
  else
  {
    Calculate_Distances();
  }

  Display_Locations2();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_LocationsObstruction(void)
{
  if (hitFlagObstruction == MESH_LOCATION_HAS_XYZ)
  {
    haveObstructionMessageFlag = 0;

    Obstruction_ScreenX->Caption = screenXObstruction;
    Obstruction_ScreenY->Caption = screenYObstruction;

    Obstruction_MeshX->Caption = xyzArrayObstruction[0];
    Obstruction_MeshY->Caption = xyzArrayObstruction[1];
    Obstruction_MeshZ->Caption = xyzArrayObstruction[2];

    Obstruction_Distance->Caption = obstructionDistance;
  }
  else
  {
    if (haveObstructionMessageFlag)
    {
      haveObstructionMessageFlag = 0;
    }
    else
    {
      ObstructionMessage->Caption = "";
    }

    Obstruction_ScreenX->Caption = "";
    Obstruction_ScreenY->Caption = "";

    Obstruction_MeshX->Caption = "";
    Obstruction_MeshY->Caption = "";
    Obstruction_MeshZ->Caption = "";

    Obstruction_Distance->Caption = "";
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_LocationsObstruction(void)
{
  Int32 result = At3d_Get_Mesh_Location(3, &hitFlagObstruction, &xyzArrayObstruction[0],
                                        &screenXObstruction, &screenYObstruction);

  if ( (result) &&
       (hitFlagObstruction == MESH_LOCATION_HAS_XYZ) )
  {
    Calculate_Distances();
  }
  else
  {
    hitFlagObstruction = MESH_LOCATION_NO_DATA;
  }

  Display_LocationsObstruction();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_LocationsObserver(void)
{
  if (hitFlagObserver == MESH_LOCATION_NO_DATA)
  {
    Observer_ScreenX->Caption = "";
    Observer_ScreenY->Caption = "";

    Observer_MeshX->Caption = "";
    Observer_MeshY->Caption = "";
    Observer_MeshZ->Caption = "";
  }
  else
  {
    Observer_ScreenX->Caption = screenXObserver;
    Observer_ScreenY->Caption = screenYObserver;

    if (hitFlagObserver == MESH_LOCATION_HAS_XYZ)
    {
      Observer_MeshX->Caption = xyzArrayObserver[0];
      Observer_MeshY->Caption = xyzArrayObserver[1];
      Observer_MeshZ->Caption = xyzArrayObserver[2];
    }
    else
    {
      Observer_MeshX->Caption = "";
      Observer_MeshY->Caption = "";
      Observer_MeshZ->Caption = "";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_LocationsObserver(void)
{
  Int32 result = At3d_Get_Mesh_Location(4, &hitFlagObserver, &xyzArrayObserver[0],
                                        &screenXObserver, &screenYObserver);

  if (!result)
  {
    hitFlagObserver = MESH_LOCATION_NO_DATA;
  }
  else
  {
    Calculate_Distances();
  }

  Display_LocationsObserver();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::TouchScreenInputClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  touchScreenInputFlag = (TouchScreenInput->Checked) ? 1 : 0;

  At3d_WriteFlag(LIBFLAG_TOUCH_SCREEN_INPUT, touchScreenInputFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewFrontClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_ViewFront();
  GetCameraAnglesClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewBackClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_ViewBack();
  GetCameraAnglesClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewRightClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_ViewRight();
  GetCameraAnglesClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewLeftClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_ViewLeft();
  GetCameraAnglesClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewTopClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_ViewTop();
  GetCameraAnglesClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewBottomClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_ViewBottom();
  GetCameraAnglesClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraViewpointClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_GetCameraViewPoint(&cameraXViewpoint,
                                         &cameraYViewpoint,
                                         &cameraZViewpoint);

  if (result)
  {
    CameraXViewpoint->Text = cameraXViewpoint;
    CameraYViewpoint->Text = cameraYViewpoint;
    CameraZViewpoint->Text = cameraZViewpoint;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraViewpointClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    cameraXViewpoint = CameraXViewpoint->Text.ToDouble();
  }
  catch(...)
  {
    cameraXViewpoint = 0.0;
  }

  try
  {
    cameraYViewpoint = CameraYViewpoint->Text.ToDouble();
  }
  catch(...)
  {
    cameraYViewpoint = 0.0;
  }

  try
  {
    cameraZViewpoint = CameraZViewpoint->Text.ToDouble();
  }
  catch(...)
  {
    cameraZViewpoint = 0.0;
  }

  At3d_SetCameraViewPoint(cameraXViewpoint,
                          cameraYViewpoint,
                          cameraZViewpoint);

  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraFieldOfViewClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_GetCameraFieldOfView(&cameraFieldOfView);

  if (result)
  {
    CameraFieldOfView->Text = cameraFieldOfView;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraFieldOfViewClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    cameraFieldOfView = CameraFieldOfView->Text.ToDouble();
  }
  catch(...)
  {
    cameraFieldOfView = 30.0;
  }

  Int32 result = At3d_SetCameraFieldOfView(0, cameraFieldOfView);

  if (result) GetCameraFieldOfViewClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraStepFOV(void)
{
  try
  {
    cameraStepFOV = CameraStepFOV->Text.ToDouble();
  }
  catch(...)
  {
    cameraStepFOV = 1.0;
  }

  CameraStepFOV->Text = cameraStepFOV;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraFOV1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  GetCameraStepFOV();
  At3d_SetCameraFieldOfView(1, -cameraStepFOV);
  GetCameraFieldOfViewClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraFOV2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  GetCameraStepFOV();
  At3d_SetCameraFieldOfView(1, cameraStepFOV);
  GetCameraFieldOfViewClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraRotateSpeed(void)
{
  Int32 result = At3d_GetCameraRotateSpeed(&cameraRotateSpeed);

  if (result)
  {
    if (cameraRotateSpeed == CAMERA_ROTATE_SPEED_1)
    {
      CameraRotateSpeed->Text = "R1";
    }
    else
    {
      CameraRotateSpeed->Text = "R2";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraRotateSpeed1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_SetCameraRotateSpeed(CAMERA_ROTATE_SPEED_1);

  if (result) GetCameraRotateSpeed();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraRotateSpeed2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_SetCameraRotateSpeed(CAMERA_ROTATE_SPEED_2);

  if (result) GetCameraRotateSpeed();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraAnglesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);

  CameraXAngle->Text = cameraXAngle;
  CameraYAngle->Text = cameraYAngle;
  CameraZAngle->Text = cameraZAngle;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraAnglesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    cameraXAngle = CameraXAngle->Text.ToDouble();
  }
  catch(...)
  {
    cameraXAngle = 0.0;
  }

  try
  {
    cameraYAngle = CameraYAngle->Text.ToDouble();
  }
  catch(...)
  {
    cameraYAngle = 0.0;
  }

  try
  {
    cameraZAngle = CameraZAngle->Text.ToDouble();
  }
  catch(...)
  {
    cameraZAngle = 0.0;
  }

  At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);

  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraStepAngle(void)
{
  try
  {
    cameraStepAngle = CameraStepAngle->Text.ToDouble();
  }
  catch(...)
  {
    cameraStepAngle = 1.0;
  }

  CameraStepAngle->Text = cameraStepAngle;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraXAngle1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  GetCameraStepAngle();
  At3d_RotateCamera(1, -cameraStepAngle, 0.0, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraXAngle2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  GetCameraStepAngle();
  At3d_RotateCamera(1, cameraStepAngle, 0.0, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraYAngle1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, -cameraStepAngle, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraYAngle2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, cameraStepAngle, 0.0);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZAngle1Click(TObject *Sender)
{
   sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, 0.0, -cameraStepAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZAngle2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  GetCameraStepAngle();
  At3d_RotateCamera(1, 0.0, 0.0, cameraStepAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZeroCameraAngleXClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);
  cameraXAngle = 0.0;
  At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZeroCameraAngleYClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);
  cameraYAngle = 0.0;
  At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZeroCameraAngleZClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);
  cameraZAngle = 0.0;
  At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZeroCameraAngleXYZClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_GetCameraAngles(&cameraXAngle, &cameraYAngle, &cameraZAngle);
  cameraXAngle = 0.0;
  cameraYAngle = 0.0;
  cameraZAngle = 0.0;
  At3d_RotateCamera(0, cameraXAngle, cameraYAngle, cameraZAngle);
  GetCameraAnglesClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraPanClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    cameraXPan = CameraXPan->Text.ToInt();
  }
  catch(...)
  {
    cameraXPan = 0;
  }

  try
  {
    cameraZPan = CameraZPan->Text.ToInt();
  }
  catch(...)
  {
    cameraZPan = 0;
  }

  At3d_PanCamera(0, cameraXPan, cameraZPan);

  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraStepPan(void)
{
  try
  {
    cameraStepPan = CameraStepPan->Text.ToInt();
  }
  catch(...)
  {
    cameraStepPan = 1;
  }

  CameraStepPan->Text = cameraStepPan;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraXPan1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  GetCameraStepPan();
  At3d_PanCamera(1, -cameraStepPan, 0);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraXPan2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  GetCameraStepPan();
  At3d_PanCamera(1, cameraStepPan, 0);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZPan1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  GetCameraStepPan();
  At3d_PanCamera(1, 0, -cameraStepPan);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZPan2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  GetCameraStepPan();
  At3d_PanCamera(1, 0, cameraStepPan);
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewCenterXClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_CenterX();
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewCenterZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_CenterZ();
  GetCameraViewpointClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReversePanDirectionClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  reversePanDirectionFlag = (ReversePanDirection->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_PAN_DIRECTION, reversePanDirectionFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraInitialDistanceClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_GetCameraInitialDistance(&cameraInitialDistance);

  if (result)
  {
    CameraInitialDistance->Text = cameraInitialDistance;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraInitialDistanceClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    cameraInitialDistance = CameraInitialDistance->Text.ToDouble();
  }
  catch(...)
  {
    cameraInitialDistance = 1.0;
  }

  Int32 result = At3d_SetCameraInitialDistance(cameraInitialDistance);

  if (result) GetCameraInitialDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraDistanceClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_GetCameraDistance(&cameraDistance);

  if (result)
  {
    CameraDistance->Text = cameraDistance;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraDistanceClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    cameraDistance = CameraDistance->Text.ToDouble();
  }
  catch(...)
  {
    cameraDistance = 1.0;
  }

  Int32 result = At3d_SetCameraDistance(cameraDistance);

  if (result) GetCameraDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetCameraZoomSpeedClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_GetCameraZoomSpeed(&cameraZoomSpeed);

  if (result)
  {
    if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_1)
    {
      CameraZoomSpeed->Text = "Z1";
    }
    else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_2)
    {
      CameraZoomSpeed->Text = "Z2";
    }
    else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_3)
    {
      CameraZoomSpeed->Text = "Z3";
    }
    else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_4)
    {
      CameraZoomSpeed->Text = "Z4";
    }
    else if (cameraZoomSpeed == CAMERA_ZOOM_SPEED_5)
    {
      CameraZoomSpeed->Text = "Z5";
    }
    else
    {
      CameraZoomSpeed->Text = "Z6";
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_1);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_2);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed3Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_3);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed4Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_4);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed5Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_5);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraZoomSpeed6Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_SetCameraZoomSpeed(CAMERA_ZOOM_SPEED_6);

  if (result) GetCameraZoomSpeedClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZoomAutomaticClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  zoomAutomaticFlag = (zoomAutomaticFlag) ? 0 : 1;

  At3d_WriteFlag(LIBFLAG_CAMERA_ZOOM_SPEED_AUTOMATIC, zoomAutomaticFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZoomInClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_ZoomIn();
  GetCameraDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZoomOutClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_ZoomOut();
  GetCameraDistanceClick(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ZoomResetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_ZoomReset();

  Get_All_ActiveCamera_Data();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReverseZoomDirectionClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  reverseZoomDirectionFlag = (ReverseZoomDirection->Checked) ? 1 : 0;
  At3d_WriteFlag(LIBFLAG_REVERSE_ZOOM_DIRECTION, reverseZoomDirectionFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_All_ActiveCamera_Data(void)
{
  GetCameraViewpointClick(MY_NULL_PTR);

  GetCameraFieldOfViewClick(MY_NULL_PTR);

  GetCameraAnglesClick(MY_NULL_PTR);

  GetCameraInitialDistanceClick(MY_NULL_PTR);

  GetCameraDistanceClick(MY_NULL_PTR);

  GetCameraZoomSpeedClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SwapEndsClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (At3d_Swap_Ends_Mesh_Location() > 0)
  {
    UpdateObstructionClick(MY_NULL_PTR);
    Display_Rubberband_Data(DISPLAY_RUBBER_MOUSE_SWAP);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_RGB_Panel(Int32 index)
{
  TPanel* panelPtr;
  Int32 red;
  Int32 green;
  Int32 blue;

  if (index == COLOR_PANEL_LINE)
  {
    panelPtr = Line_Panel;

    red   = my_change_float_to_256_int(rgbLineArray[0]);
    green = my_change_float_to_256_int(rgbLineArray[1]);
    blue  = my_change_float_to_256_int(rgbLineArray[2]);
  }
  else if (index == COLOR_PANEL_VERTEX1)
  {
    panelPtr = Vertex1_Panel;

    red   = my_change_float_to_256_int(rgbPoint1Array[0]);
    green = my_change_float_to_256_int(rgbPoint1Array[1]);
    blue  = my_change_float_to_256_int(rgbPoint1Array[2]);
  }
  else if (index == COLOR_PANEL_VERTEX2)
  {
    panelPtr = Vertex2_Panel;

    red   = my_change_float_to_256_int(rgbPoint2Array[0]);
    green = my_change_float_to_256_int(rgbPoint2Array[1]);
    blue  = my_change_float_to_256_int(rgbPoint2Array[2]);
  }
  else if (index == COLOR_PANEL_OBSTRUCTION)
  {
    panelPtr = Obstruction_Panel;

    red   = my_change_float_to_256_int(rgbObstructionArray[0]);
    green = my_change_float_to_256_int(rgbObstructionArray[1]);
    blue  = my_change_float_to_256_int(rgbObstructionArray[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE)
  {
    panelPtr = ObserverLine_Panel;

    red   = my_change_float_to_256_int(rgbLineObserverArray[0]);
    green = my_change_float_to_256_int(rgbLineObserverArray[1]);
    blue  = my_change_float_to_256_int(rgbLineObserverArray[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX)
  {
    panelPtr = ObserverVertex_Panel;

    red   = my_change_float_to_256_int(rgbPointObserverArray[0]);
    green = my_change_float_to_256_int(rgbPointObserverArray[1]);
    blue  = my_change_float_to_256_int(rgbPointObserverArray[2]);
  }
  else if (index == COLOR_PANEL_LINE_ML)
  {
    panelPtr = Line_Panel_ML;

    red   = my_change_float_to_256_int(rgbLineArray_ML[0]);
    green = my_change_float_to_256_int(rgbLineArray_ML[1]);
    blue  = my_change_float_to_256_int(rgbLineArray_ML[2]);
  }
  else if (index == COLOR_PANEL_VERTEX1_ML)
  {
    panelPtr = Vertex1_Panel_ML;

    red   = my_change_float_to_256_int(rgbPoint1Array_ML[0]);
    green = my_change_float_to_256_int(rgbPoint1Array_ML[1]);
    blue  = my_change_float_to_256_int(rgbPoint1Array_ML[2]);
  }
  else if (index == COLOR_PANEL_VERTEX2_ML)
  {
    panelPtr = Vertex2_Panel_ML;

    red   = my_change_float_to_256_int(rgbPoint2Array_ML[0]);
    green = my_change_float_to_256_int(rgbPoint2Array_ML[1]);
    blue  = my_change_float_to_256_int(rgbPoint2Array_ML[2]);
  }
  else if (index == COLOR_PANEL_OBSTRUCTION_ML)
  {
    panelPtr = Obstruction_Panel_ML;

    red   = my_change_float_to_256_int(rgbObstructionArray_ML[0]);
    green = my_change_float_to_256_int(rgbObstructionArray_ML[1]);
    blue  = my_change_float_to_256_int(rgbObstructionArray_ML[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE_ML)
  {
    panelPtr = ObserverLine_Panel_ML;

    red   = my_change_float_to_256_int(rgbLineObserverArray_ML[0]);
    green = my_change_float_to_256_int(rgbLineObserverArray_ML[1]);
    blue  = my_change_float_to_256_int(rgbLineObserverArray_ML[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX_ML)
  {
    panelPtr = ObserverVertex_Panel_ML;

    red   = my_change_float_to_256_int(rgbPointObserverArray_ML[0]);
    green = my_change_float_to_256_int(rgbPointObserverArray_ML[1]);
    blue  = my_change_float_to_256_int(rgbPointObserverArray_ML[2]);
  }
  else
  {
    return;
  }

  panelPtr->Color = static_cast<TColor>(red + (green << 8) + (blue << 16));
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_ColorDialog(Int32* inColorPtr, Int32* outColorPtr)
{
  if( (!inColorPtr) || (!outColorPtr) ) return;

  // --

  TColorDialog* ColorDlg;
  try
  {
    ColorDlg = new TColorDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    ColorDlg = MY_NULL_PTR;
  }

  if (ColorDlg)
  {
    ColorDlg->Options = ColorDlg->Options << cdFullOpen;
    ColorDlg->Color = static_cast<TColor>((inColorPtr[0] + (inColorPtr[1] << 8) + (inColorPtr[2] << 16)));

    ColorDlg->Execute();

    outColorPtr[0] = static_cast<UInt8>(ColorDlg->Color & 0xFF);
    outColorPtr[1] = static_cast<UInt8>((ColorDlg->Color & 0xFF00) >> 8);
    outColorPtr[2] = static_cast<UInt8>((ColorDlg->Color & 0xFF0000) >> 16);

    delete ColorDlg;
  }
  else
  {
    for (Int32 i=0; i<3; i++)
    {
      outColorPtr[i] = inColorPtr[i];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Pick_Color(Int32 index)
{
  Int32 temp1[3];
  Int32 temp2[3];

  if (index == COLOR_PANEL_LINE)
  {
    temp1[0] = my_change_float_to_256_int(rgbLineArray[0]);
    temp1[1] = my_change_float_to_256_int(rgbLineArray[1]);
    temp1[2] = my_change_float_to_256_int(rgbLineArray[2]);
  }
  else if (index == COLOR_PANEL_VERTEX1)
  {
    temp1[0] = my_change_float_to_256_int(rgbPoint1Array[0]);
    temp1[1] = my_change_float_to_256_int(rgbPoint1Array[1]);
    temp1[2] = my_change_float_to_256_int(rgbPoint1Array[2]);
  }
  else if (index == COLOR_PANEL_VERTEX2)
  {
    temp1[0] = my_change_float_to_256_int(rgbPoint2Array[0]);
    temp1[1] = my_change_float_to_256_int(rgbPoint2Array[1]);
    temp1[2] = my_change_float_to_256_int(rgbPoint2Array[2]);
  }
  else if (index == COLOR_PANEL_OBSTRUCTION)
  {
    temp1[0] = my_change_float_to_256_int(rgbObstructionArray[0]);
    temp1[1] = my_change_float_to_256_int(rgbObstructionArray[1]);
    temp1[2] = my_change_float_to_256_int(rgbObstructionArray[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE)
  {
    temp1[0] = my_change_float_to_256_int(rgbLineObserverArray[0]);
    temp1[1] = my_change_float_to_256_int(rgbLineObserverArray[1]);
    temp1[2] = my_change_float_to_256_int(rgbLineObserverArray[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX)
  {
    temp1[0] = my_change_float_to_256_int(rgbPointObserverArray[0]);
    temp1[1] = my_change_float_to_256_int(rgbPointObserverArray[1]);
    temp1[2] = my_change_float_to_256_int(rgbPointObserverArray[2]);
  }
  else if (index == COLOR_PANEL_LINE_ML)
  {
    temp1[0] = my_change_float_to_256_int(rgbLineArray_ML[0]);
    temp1[1] = my_change_float_to_256_int(rgbLineArray_ML[1]);
    temp1[2] = my_change_float_to_256_int(rgbLineArray_ML[2]);
  }
  else if (index == COLOR_PANEL_VERTEX1_ML)
  {
    temp1[0] = my_change_float_to_256_int(rgbPoint1Array_ML[0]);
    temp1[1] = my_change_float_to_256_int(rgbPoint1Array_ML[1]);
    temp1[2] = my_change_float_to_256_int(rgbPoint1Array_ML[2]);
  }
  else if (index == COLOR_PANEL_VERTEX2_ML)
  {
    temp1[0] = my_change_float_to_256_int(rgbPoint2Array_ML[0]);
    temp1[1] = my_change_float_to_256_int(rgbPoint2Array_ML[1]);
    temp1[2] = my_change_float_to_256_int(rgbPoint2Array_ML[2]);
  }
  else if (index == COLOR_PANEL_OBSTRUCTION_ML)
  {
    temp1[0] = my_change_float_to_256_int(rgbObstructionArray_ML[0]);
    temp1[1] = my_change_float_to_256_int(rgbObstructionArray_ML[1]);
    temp1[2] = my_change_float_to_256_int(rgbObstructionArray_ML[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE_ML)
  {
    temp1[0] = my_change_float_to_256_int(rgbLineObserverArray_ML[0]);
    temp1[1] = my_change_float_to_256_int(rgbLineObserverArray_ML[1]);
    temp1[2] = my_change_float_to_256_int(rgbLineObserverArray_ML[2]);
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX_ML)
  {
    temp1[0] = my_change_float_to_256_int(rgbPointObserverArray_ML[0]);
    temp1[1] = my_change_float_to_256_int(rgbPointObserverArray_ML[1]);
    temp1[2] = my_change_float_to_256_int(rgbPointObserverArray_ML[2]);
  }
  else
  {
    return;
  }

  // --

  Do_ColorDialog(temp1, temp2);

  // --

  float red   = my_change_256_int_to_float(temp2[0]);
  float green = my_change_256_int_to_float(temp2[1]);
  float blue  = my_change_256_int_to_float(temp2[2]);

  if (index == COLOR_PANEL_LINE)
  {
    rgbLineArray[0] = red;
    rgbLineArray[1] = green;
    rgbLineArray[2] = blue;
  }
  else if (index == COLOR_PANEL_VERTEX1)
  {
    rgbPoint1Array[0] = red;
    rgbPoint1Array[1] = green;
    rgbPoint1Array[2] = blue;
  }
  else if (index == COLOR_PANEL_VERTEX2)
  {
    rgbPoint2Array[0] = red;
    rgbPoint2Array[1] = green;
    rgbPoint2Array[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSTRUCTION)
  {
    rgbObstructionArray[0] = red;
    rgbObstructionArray[1] = green;
    rgbObstructionArray[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE)
  {
    rgbLineObserverArray[0] = red;
    rgbLineObserverArray[1] = green;
    rgbLineObserverArray[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX)
  {
    rgbPointObserverArray[0] = red;
    rgbPointObserverArray[1] = green;
    rgbPointObserverArray[2] = blue;
  }
  else if (index == COLOR_PANEL_LINE_ML)
  {
    rgbLineArray_ML[0] = red;
    rgbLineArray_ML[1] = green;
    rgbLineArray_ML[2] = blue;
  }
  else if (index == COLOR_PANEL_VERTEX1_ML)
  {
    rgbPoint1Array_ML[0] = red;
    rgbPoint1Array_ML[1] = green;
    rgbPoint1Array_ML[2] = blue;
  }
  else if (index == COLOR_PANEL_VERTEX2_ML)
  {
    rgbPoint2Array_ML[0] = red;
    rgbPoint2Array_ML[1] = green;
    rgbPoint2Array_ML[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSTRUCTION_ML)
  {
    rgbObstructionArray_ML[0] = red;
    rgbObstructionArray_ML[1] = green;
    rgbObstructionArray_ML[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSERVER_LINE_ML)
  {
    rgbLineObserverArray_ML[0] = red;
    rgbLineObserverArray_ML[1] = green;
    rgbLineObserverArray_ML[2] = blue;
  }
  else if (index == COLOR_PANEL_OBSERVER_VERTEX_ML)
  {
    rgbPointObserverArray_ML[0] = red;
    rgbPointObserverArray_ML[1] = green;
    rgbPointObserverArray_ML[2] = blue;
  }

  // --

  Do_RGB_Panel(index);

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Line_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_LINE);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Vertex1_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_VERTEX1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Vertex2_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_VERTEX2);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UseDepthTestClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  depthTestFlag = (UseDepthTest->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayVerticesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  pointsFlag = (DisplayVertices->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetRubberbandParametersClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    lineWidth = my_double_to_float(LineWidth->Text.ToDouble());
  }
  catch(...)
  {
    lineWidth = 1.0f;
  }

  try
  {
    pointSize = my_double_to_float(PointSize->Text.ToDouble());
  }
  catch(...)
  {
    pointSize = 1.0f;
  }

  // --

  At3d_Set_Mesh_Location_Parameters(&rgbLineArray[0], &rgbPoint1Array[0],
                                    &rgbPoint2Array[0], lineWidth, pointSize,
                                    depthTestFlag, pointsFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Obstruction_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSTRUCTION);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObstructionVertexClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  obstructionDisplayFlag = (DisplayObstructionVertex->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetObstructionParametersClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    pointObstructionSize = my_double_to_float(ObstructionPointSize->Text.ToDouble());
  }
  catch(...)
  {
    pointObstructionSize = 1.0f;
  }

  try
  {
    obstructionSkipDistance = ObstructionSkipDistance->Text.ToDouble();
  }
  catch(...)
  {
    obstructionSkipDistance = 0.0;
  }

  // --

  At3d_Set_Mesh_Obstruction_Parameters(rgbObstructionArray, pointObstructionSize,
                                       obstructionDisplayFlag, obstructionSkipDistance);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CheckForObstructionClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result = At3d_Get_Mesh_Obstruction_Point(&xyzArrayObstruction[0],
                                                 &screenXObstruction,
                                                 &screenYObstruction);

  if (result < 0)
  {
    hitFlagObstruction = MESH_LOCATION_NO_DATA;
    haveObstructionMessageFlag = 1;
    ObstructionMessage->Caption = "Error";
  }
  else if (!result)
  {
    hitFlagObstruction = MESH_LOCATION_NO_DATA;
    haveObstructionMessageFlag = 1;
    ObstructionMessage->Caption = "No obstruction point";
  }
  else
  {
    hitFlagObstruction = MESH_LOCATION_HAS_XYZ;
    haveObstructionMessageFlag = 1;
    ObstructionMessage->Caption = "Have obstruction point";

    Calculate_Distances();
  }

  Display_LocationsObstruction();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ClipAtObstructionPointClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (At3d_Clip_At_Mesh_Obstruction_Point() > 0)
  {
    Display_Rubberband_Data(DISPLAY_RUBBER_MOUSE_CLIP);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UpdateScreenXYClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 result;

  if (At3d_Update_Mesh_Location_ScreenXY() > 0)
  {
    result = At3d_Get_Mesh_Location(1, &hitFlag1, &xyzArray1[0],
                                    &screenX1, &screenY1);

    if ( (result) &&
         (hitFlag1 == MESH_LOCATION_HAS_XYZ) )
    {
      Location1_ScreenX->Caption = screenX1;
      Location1_ScreenY->Caption = screenY1;
    }

    // --

    result = At3d_Get_Mesh_Location(2, &hitFlag2, &xyzArray2[0],
                                    &screenX2, &screenY2);

    if ( (result) && (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
    {
      Location2_ScreenX->Caption = screenX2;
      Location2_ScreenY->Caption = screenY2;
    }

    // --

    result = At3d_Get_Mesh_Location(3, &hitFlagObstruction, &xyzArrayObstruction[0],
                                    &screenXObstruction, &screenYObstruction);

    if ( (result) && (hitFlagObstruction == MESH_LOCATION_HAS_XYZ) )
    {
      Obstruction_ScreenX->Caption = screenXObstruction;
      Obstruction_ScreenY->Caption = screenYObstruction;
    }

    // --

    result = At3d_Get_Mesh_Location(4, &hitFlagObserver, &xyzArrayObserver[0],
                                    &screenXObserver, &screenYObserver);

    if ( (result) && (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      Observer_ScreenX->Caption = screenXObserver;
      Observer_ScreenY->Caption = screenYObserver;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UpdateObstructionClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_LocationsObstruction();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::TypeObjClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  inputFileType = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::TypeWrlClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  inputFileType = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetViewpoint_MiddleClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_SET_VIEWPOINT_MOUSE, SET_VIEWPOINT_MOUSE_MIDDLE);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetViewpoint_LeftClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_SET_VIEWPOINT_MOUSE, SET_VIEWPOINT_MOUSE_LEFT);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::LeftToggle_RotateClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_LEFT_MOUSE_ROTATE_PAN, LEFT_MOUSE_TOGGLE_ROTATE);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::LeftToggle_PanClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_LEFT_MOUSE_ROTATE_PAN, LEFT_MOUSE_TOGGLE_PAN);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Lock_XClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  xRotation_Lock_Flag = (Axis_Lock_X->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Lock_YClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  yRotation_Lock_Flag = (Axis_Lock_Y->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Lock_ZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  zRotation_Lock_Flag = (Axis_Lock_Z->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Constrain_XClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  xRotation_Constrain_Flag = (Axis_Constrain_X->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Constrain_YClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  yRotation_Constrain_Flag = (Axis_Constrain_Y->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Axis_Constrain_ZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  zRotation_Constrain_Flag = (Axis_Constrain_Z->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Axis_XClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_Get_Rotation_Lock(cameraID, 0, &xRotation_Lock_Flag, &xRotation_Constrain_Flag,
                         &xRotation_Constrain_From, &xRotation_Constrain_To);

  // --

  Axis_Lock_X->Checked = (xRotation_Lock_Flag) ? 1 : 0;

  Axis_Constrain_X->Checked = (xRotation_Constrain_Flag) ? 1 : 0;

  Constrain_From_X->Text = xRotation_Constrain_From;

  Constrain_To_X->Text = xRotation_Constrain_To;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Axis_XClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    xRotation_Constrain_From = Constrain_From_X->Text.ToDouble();
  }
  catch(...)
  {
    xRotation_Constrain_From = -180.0;
  }

  try
  {
    xRotation_Constrain_To = Constrain_To_X->Text.ToDouble();
  }
  catch(...)
  {
    xRotation_Constrain_To = 180.0;
  }

  // --

  At3d_Set_Rotation_Lock(cameraID, 0, xRotation_Lock_Flag, xRotation_Constrain_Flag,
                         xRotation_Constrain_From, xRotation_Constrain_To);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Axis_YClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_Get_Rotation_Lock(cameraID, 1, &yRotation_Lock_Flag, &yRotation_Constrain_Flag,
                         &yRotation_Constrain_From, &yRotation_Constrain_To);

  // --

  Axis_Lock_Y->Checked = (yRotation_Lock_Flag) ? 1 : 0;

  Axis_Constrain_Y->Checked = (yRotation_Constrain_Flag) ? 1 : 0;

  Constrain_From_Y->Text = yRotation_Constrain_From;

  Constrain_To_Y->Text = yRotation_Constrain_To;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Axis_YClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    yRotation_Constrain_From = Constrain_From_Y->Text.ToDouble();
  }
  catch(...)
  {
    yRotation_Constrain_From = -180.0;
  }

  try
  {
    yRotation_Constrain_To = Constrain_To_Y->Text.ToDouble();
  }
  catch(...)
  {
    yRotation_Constrain_To = 180.0;
  }

  // --

  At3d_Set_Rotation_Lock(cameraID, 1, yRotation_Lock_Flag, yRotation_Constrain_Flag,
                         yRotation_Constrain_From, yRotation_Constrain_To);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Axis_ZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_Get_Rotation_Lock(cameraID, 2, &zRotation_Lock_Flag, &zRotation_Constrain_Flag,
                         &zRotation_Constrain_From, &zRotation_Constrain_To);

  // --

  Axis_Lock_Z->Checked = (zRotation_Lock_Flag) ? 1 : 0;

  Axis_Constrain_Z->Checked = (zRotation_Constrain_Flag) ? 1 : 0;

  Constrain_From_Z->Text = zRotation_Constrain_From;

  Constrain_To_Z->Text = zRotation_Constrain_To;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Axis_XYZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Get_Axis_XClick(MY_NULL_PTR);
  Get_Axis_YClick(MY_NULL_PTR);
  Get_Axis_ZClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Axis_ZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    zRotation_Constrain_From = Constrain_From_Z->Text.ToDouble();
  }
  catch(...)
  {
    zRotation_Constrain_From = -180.0;
  }

  try
  {
    zRotation_Constrain_To = Constrain_To_Z->Text.ToDouble();
  }
  catch(...)
  {
    zRotation_Constrain_To = 180.0;
  }

  // --

  At3d_Set_Rotation_Lock(cameraID, 2, zRotation_Lock_Flag, zRotation_Constrain_Flag,
                         zRotation_Constrain_From, zRotation_Constrain_To);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Axis_XYZClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Set_Axis_XClick(MY_NULL_PTR);
  Set_Axis_YClick(MY_NULL_PTR);
  Set_Axis_ZClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewFromV1ToV2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      roaming_Eye[0] = xyzArrayObserver[0];
      roaming_Eye[1] = xyzArrayObserver[1];
      roaming_Eye[2] = xyzArrayObserver[2];
    }
    else
    {
      roaming_Eye[0] = xyzArray1[0];
      roaming_Eye[1] = xyzArray1[1];
      roaming_Eye[2] = xyzArray1[2];
    }

    roaming_Center[0] = xyzArray2[0];
    roaming_Center[1] = xyzArray2[1];
    roaming_Center[2] = xyzArray2[2];

    roaming_Up[0] = 0.0;
    roaming_Up[1] = 1.0;
    roaming_Up[2] = 0.0;

    Display_Roaming_Parameters();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ViewFromV2ToV1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    roaming_Eye[0] = xyzArray2[0];
    roaming_Eye[1] = xyzArray2[1];
    roaming_Eye[2] = xyzArray2[2];

    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      roaming_Center[0] = xyzArrayObserver[0];
      roaming_Center[1] = xyzArrayObserver[1];
      roaming_Center[2] = xyzArrayObserver[2];
    }
    else
    {
      roaming_Center[0] = xyzArray1[0];
      roaming_Center[1] = xyzArray1[1];
      roaming_Center[2] = xyzArray1[2];
    }

    roaming_Up[0] = 0.0;
    roaming_Up[1] = 1.0;
    roaming_Up[2] = 0.0;

    Display_Roaming_Parameters();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ObserverLine_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSERVER_LINE);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ObserverVertex_PanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSERVER_VERTEX);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObserverLineClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  observerLineFlag = (DisplayObserverLine->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObserverVertexClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  observerPointFlag = (DisplayObserverVertex->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetObserverParametersClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    observerWidth = my_double_to_float(ObserverLineWidth->Text.ToDouble());
  }
  catch(...)
  {
    observerWidth = 1.0f;
  }

  try
  {
    observerSize = my_double_to_float(ObserverPointSize->Text.ToDouble());
  }
  catch(...)
  {
    observerSize = 1.0f;
  }

  // --

  At3d_Set_Mesh_Location_Observer_Parameters( rgbLineObserverArray, rgbPointObserverArray,
                                              observerWidth, observerSize,
                                              observerLineFlag, observerPointFlag );
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UseObserverClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (UseObserver->Checked)
  {
    try
    {
      xyzDeltaArrayObserver[0] = Observer_DeltaX->Text.ToDouble();
    }
    catch(...)
    {
      xyzDeltaArrayObserver[0] = 0.0;
    }

    // --

    try
    {
      xyzDeltaArrayObserver[1] = Observer_DeltaY->Text.ToDouble();
    }
    catch(...)
    {
      xyzDeltaArrayObserver[1] = 0.0;
    }

    // --

    try
    {
      xyzDeltaArrayObserver[2] = Observer_DeltaZ->Text.ToDouble();
    }
    catch(...)
    {
      xyzDeltaArrayObserver[2] = 0.0;
    }

    // --

    if (At3d_Add_Mesh_Location_Observer(&xyzDeltaArrayObserver[0]) > 0)
    {
      haveObserverFlag = 1;
    }
    else
    {
      // error if return is -1
      // x,y and z are zero if return is zero
      //
      haveObserverFlag = 0;
    }

    Display_Rubberband_Data(DISPLAY_RUBBER_OBSERVER_ADD);
  }
  else
  {
    At3d_Cancel_Mesh_Location_Observer();

    haveObserverFlag = 0;

    Display_Rubberband_Data(DISPLAY_RUBBER_OBSERVER_CANCEL);
  }

  At3d_RenderOpenGLScene();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Rubberband_Data(Int32 item)
{
  if (item == DISPLAY_RUBBER_CLEAR_ALL)
  {
    hitFlag0 = MESH_LOCATION_NO_DATA;
    hitFlag1 = MESH_LOCATION_NO_DATA;
    hitFlag2 = MESH_LOCATION_NO_DATA;
    hitFlagObstruction = MESH_LOCATION_NO_DATA;
    hitFlagObserver = MESH_LOCATION_NO_DATA;

    Display_Locations0();
    Display_Locations1();
    Display_Locations2();
    Display_LocationsObstruction();
    Display_LocationsObserver();
  }
  else if (item == DISPLAY_RUBBER_MOUSE_UP)
  {
    // event: left button up
    //
    hitFlagObstruction = MESH_LOCATION_NO_DATA;
    Display_LocationsObstruction();

    Get_Locations1();
    Get_Locations2();
  }
  else if (item == DISPLAY_RUBBER_MOUSE_SWAP)
  {
    Get_Locations1();
    Get_Locations2();
  }
  else if (item == DISPLAY_RUBBER_MOUSE_CLIP)
  {
    hitFlagObstruction = MESH_LOCATION_NO_DATA;
    Display_LocationsObstruction();

    Get_Locations2();
  }
  else if (item == DISPLAY_RUBBER_OBSERVER_ADD)
  {
    if (haveObserverFlag)
    {
      hitFlagObstruction = MESH_LOCATION_NO_DATA;
      Display_LocationsObstruction();

      Get_LocationsObserver();
      Get_Locations2();
    }
    else
    {
      hitFlagObserver = MESH_LOCATION_NO_DATA;
      Display_LocationsObserver();
    }
  }
  else if (item == DISPLAY_RUBBER_OBSERVER_CANCEL)
  {
    hitFlagObserver = MESH_LOCATION_NO_DATA;
    Display_LocationsObserver();

    hitFlagObstruction = MESH_LOCATION_NO_DATA;
    Display_LocationsObstruction();

    Get_Locations2();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetViewpoint1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (hitFlag1 == MESH_LOCATION_HAS_XYZ)
  {
    At3d_SetCameraViewPoint(xyzArray1[0], xyzArray1[1], xyzArray1[2]);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetViewpoint2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (hitFlag2 == MESH_LOCATION_HAS_XYZ)
  {
    At3d_SetCameraViewPoint(xyzArray2[0], xyzArray2[1], xyzArray2[2]);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Zoom_Lock_NoneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  roamingZoomLock = ROAMING_LOCK_ZOOM_NONE;

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Zoom_Lock_InClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  roamingZoomLock = ROAMING_LOCK_ZOOM_IN;

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Zoom_Lock_OutClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  roamingZoomLock = ROAMING_LOCK_ZOOM_OUT;

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Zoom_Lock_BothClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  roamingZoomLock = ROAMING_LOCK_ZOOM_BOTH;

  At3d_WriteFlag(LIBFLAG_ROAMING_LOCK_ZOOM, roamingZoomLock);

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateX1ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectX_90Minus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateX2ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectX_90Plus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateX3ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectX_180();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateY1ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectY_90Minus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateY2ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectY_90Plus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateY3ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectY_180();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateZ1ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectZ_90Minus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateZ2ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectZ_90Plus();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::RotateZ3ObjectClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 1);

  At3d_RotateObjectZ_180();

  At3d_WriteFlag(LIBFLAG_ORIGIN_ROTATE, 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Move_AheadBack_Step(void)
{
  try
  {
    moveAheadBack_Step = fabs( Move_AheadBack_Step->Text.ToDouble() );
  }
  catch(...)
  {
    moveAheadBack_Step = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_AheadClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Get_Move_AheadBack_Step();
  At3d_Move_Roaming_AheadBack(moveAheadBack_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_BackClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Get_Move_AheadBack_Step();
  At3d_Move_Roaming_AheadBack(-moveAheadBack_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Move_RightLeft_Step(void)
{
  try
  {
    moveRightLeft_Step = fabs( Move_RightLeft_Step->Text.ToDouble() );
  }
  catch(...)
  {
    moveRightLeft_Step = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_RightClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Get_Move_RightLeft_Step();
  At3d_Move_Roaming_RightLeft(moveRightLeft_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_LeftClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Get_Move_RightLeft_Step();
  At3d_Move_Roaming_RightLeft(-moveRightLeft_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Move_UpDown_Step(void)
{
  try
  {
    moveUpDown_Step = fabs( Move_UpDown_Step->Text.ToDouble() );
  }
  catch(...)
  {
    moveUpDown_Step = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_UpClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Get_Move_UpDown_Step();
  At3d_Move_Roaming_UpDown(moveUpDown_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_DownClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Get_Move_UpDown_Step();
  At3d_Move_Roaming_UpDown(-moveUpDown_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Move_Turn_Step(void)
{
  try
  {
    moveTurn_Step = fabs( Move_Turn_Step->Text.ToDouble() );
  }
  catch(...)
  {
    moveTurn_Step = 0.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_Turn_RightClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Get_Move_Turn_Step();
  At3d_Move_Roaming_Turn(moveTurn_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Move_Camera_Turn_LeftClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Get_Move_Turn_Step();
  At3d_Move_Roaming_Turn(-moveTurn_Step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Roaming_Parameters(void)
{
  Roaming_Eye_X->Text = roaming_Eye[0];
  Roaming_Eye_Y->Text = roaming_Eye[1];
  Roaming_Eye_Z->Text = roaming_Eye[2];

  Roaming_Center_X->Text = roaming_Center[0];
  Roaming_Center_Y->Text = roaming_Center[1];
  Roaming_Center_Z->Text = roaming_Center[2];

  Roaming_Up_X->Text = roaming_Up[0];
  Roaming_Up_Y->Text = roaming_Up[1];
  Roaming_Up_Z->Text = roaming_Up[2];
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Roaming_Parameters(void)
{
  try
  {
    roaming_Eye[0] = Roaming_Eye_X->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Eye[0] = 0.0;
  }

  try
  {
    roaming_Eye[1] = Roaming_Eye_Y->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Eye[1] = 0.0;
  }

  try
  {
    roaming_Eye[2] = Roaming_Eye_Z->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Eye[2] = 0.0;
  }

  // --

  try
  {
    roaming_Center[0] = Roaming_Center_X->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Center[0] = 0.0;
  }

  try
  {
    roaming_Center[1] = Roaming_Center_Y->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Center[1] = 0.0;
  }

  try
  {
    roaming_Center[2] = Roaming_Center_Z->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Center[2] = 0.0;
  }

  // --

  try
  {
    roaming_Up[0] = Roaming_Up_X->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Up[0] = 0.0;
  }

  try
  {
    roaming_Up[1] = Roaming_Up_Y->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Up[1] = 1.0;
  }

  try
  {
    roaming_Up[2] = Roaming_Up_Z->Text.ToDouble();
  }
  catch(...)
  {
    roaming_Up[2] = 0.0;
  }

  // --

  // just displaying again to check for errors
  //
  Display_Roaming_Parameters();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_GetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (At3d_Get_Roaming_Data(cameraID, &roaming_Eye[0], &roaming_Center[0],
                            &roaming_Up[0], roaming_asDisplayed_Flag) > 0)
  {
    Display_Roaming_Parameters();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_As_DisplayedClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  roaming_asDisplayed_Flag = (Roaming_As_Displayed->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_Get_FromClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    getFromCameraID = Get_From_CameraID->Text.ToInt();
  }
  catch(...)
  {
    getFromCameraID = 0;
  }

  if (At3d_Get_Roaming_Data(getFromCameraID, &roaming_Eye[0], &roaming_Center[0],
                            &roaming_Up[0], roaming_asDisplayed_Flag) > 0)
  {
    Display_Roaming_Parameters();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_SetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Get_Roaming_Parameters();

  At3d_Set_Roaming_Data(cameraID, &roaming_Eye[0], &roaming_Center[0], &roaming_Up[0]);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_CameraClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  cameraTotal = At3d_Add_Roaming_Cameras(1);
  Total_Cameras->Text = cameraTotal;

  Set_CameraID_List();
  Set_ActiveCameraID_List();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Free_Roaming_CamerasClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_Free_All_Roaming_Cameras();

  cameraTotal = At3d_Roaming_Camera_Count();
  Total_Cameras->Text = cameraTotal;

  Set_CameraID_List();
  Set_ActiveCameraID_List();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_CameraID_List(void)
{
  CameraID_List->Items->BeginUpdate();

  try
  {
    CameraID_List->Clear();

    for (Int32 i=0; i<cameraTotal; i++)
    {
      CameraID_List->Items->Add(i);
    }

    if (!cameraTotal)
    {
      cameraID = -1;
      CameraID_List->ItemIndex = -1;
    }
    else
    {
      if (cameraID >= cameraTotal) cameraID = 0;
      CameraID_List->ItemIndex = cameraID;
    }

    CameraID_List->Items->EndUpdate();
  }
  catch(...)
  {
    CameraID_List->Items->EndUpdate();
  }

  CameraID->Text = cameraID;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CameraID_ListClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  cameraID = CameraID_List->ItemIndex;

  CameraID->Text = cameraID;

  Get_Roaming_AxisClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_ActiveCameraID_List(void)
{
  Active_CameraID_List->Items->BeginUpdate();

  try
  {
    Active_CameraID_List->Clear();

    for (Int32 i=0; i<cameraTotal; i++)
    {
      Active_CameraID_List->Items->Add(i);
    }

    if (!cameraTotal)
    {
      cameraID = -1;
      Active_CameraID_List->ItemIndex = -1;
    }
    else
    {
      if (activeCameraID >= cameraTotal) activeCameraID = 0;
      Active_CameraID_List->ItemIndex = activeCameraID;
    }

    Active_CameraID_List->Items->EndUpdate();
  }
  catch(...)
  {
    Active_CameraID_List->Items->EndUpdate();
  }

  Active_CameraID->Text = activeCameraID;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Active_CameraID_ListClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  activeCameraID = Active_CameraID_List->ItemIndex;

  Active_CameraID->Text = activeCameraID;

  At3d_Switch_OGL_Camera(activeCameraID);

  Get_All_ActiveCamera_Data();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Roaming_AxisClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Roaming_GetClick(MY_NULL_PTR);
  Get_Axis_XYZClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Set_Roaming_AxisClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Roaming_SetClick(MY_NULL_PTR);
  Set_Axis_XYZClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Roaming_Up_DefaultClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  roaming_Up[0] = 0.0;
  roaming_Up[1] = 1.0;
  roaming_Up[2] = 0.0;

  Display_Roaming_Parameters();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetGPUMaxTextureSizeClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  gpu_Max_Texture_Size = At3d_Get_GPU_Max_Texture_Size();

  GPUMaxTextureSize->Text = gpu_Max_Texture_Size;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetUserMaxTextureSizeClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  user_Max_Texture_Size = At3d_Get_Max_Texture_Size();

  UserMaxTextureSize->Text = user_Max_Texture_Size;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetUserMaxTextureSizeClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  powerOf2_Max_Texture_Size = PowerOf2_MaxTextureSize->Text.ToInt();

  Int32 result = At3d_Set_Max_Texture_Size(powerOf2_Max_Texture_Size);
  if (result) GetUserMaxTextureSizeClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_DragClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_SELECT, DISPLAY_MESH_SELECT_DRAG);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_ClickClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_SELECT, DISPLAY_MESH_SELECT_CLICK);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Line_Panel_MLClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_LINE_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Vertex1_Panel_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_VERTEX1_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Vertex2_Panel_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_VERTEX2_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::UseDepthTest_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  depthTestFlag_ML = (UseDepthTest_ML->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayVertices_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  pointsFlag_ML = (DisplayVertices_ML->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetRubberbandParameters_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    lineWidth_ML = my_double_to_float(LineWidth_ML->Text.ToDouble());
  }
  catch(...)
  {
    lineWidth_ML = 1.0f;
  }

  try
  {
    pointSize_ML = my_double_to_float(PointSize_ML->Text.ToDouble());
  }
  catch(...)
  {
    pointSize_ML = 1.0f;
  }

  // --

  At3d_MeasureList_Set_Parameters(&rgbLineArray_ML[0], &rgbPoint1Array_ML[0],
                                  &rgbPoint2Array_ML[0],
                                  lineWidth_ML, pointSize_ML,
                                  depthTestFlag_ML, pointsFlag_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Obstruction_Panel_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSTRUCTION_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObstructionVertex_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  obstructionDisplayFlag_ML = (DisplayObstructionVertex_ML->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetObstructionParameters_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    pointObstructionSize_ML = my_double_to_float(ObstructionPointSize_ML->Text.ToDouble());
  }
  catch(...)
  {
    pointObstructionSize_ML = 1.0f;
  }

  // --

  At3d_MeasureList_Set_Obstruction_Parameters(rgbObstructionArray_ML,
                                              pointObstructionSize_ML,
                                              obstructionDisplayFlag_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ObserverLine_Panel_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSERVER_LINE_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ObserverVertex_Panel_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Pick_Color(COLOR_PANEL_OBSERVER_VERTEX_ML);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObserverLine_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  observerLineFlag_ML = (DisplayObserverLine_ML->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayObserverVertex_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  observerPointFlag_ML = (DisplayObserverVertex_ML->Checked) ? 1 : 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetObserverParameters_MLClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  try
  {
    observerWidth_ML = my_double_to_float(ObserverLineWidth_ML->Text.ToDouble());
  }
  catch(...)
  {
    observerWidth_ML = 1.0f;
  }

  try
  {
    observerSize_ML = my_double_to_float(ObserverPointSize_ML->Text.ToDouble());
  }
  catch(...)
  {
    observerSize_ML = 1.0f;
  }

  // --

  At3d_MeasureList_Set_Observer_Parameters( rgbLineObserverArray_ML, rgbPointObserverArray_ML,
                                            observerWidth_ML, observerSize_ML,
                                            observerLineFlag_ML, observerPointFlag_ML );
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayMeasureListClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_DISPLAY_MEASURE_LIST,
                 (DisplayMeasureList->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AddCurrentMeasureClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, MeasureID->Text);

  char buffer[LEN_LAYER_NAME];
  my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  buffer[LEN_LAYER_NAME - 1] = '\0';

  float width;
  float rgbColor[3];

  if (!measureColorFlag)
  {
    width = lineWidth_ML;

    rgbColor[0] = rgbLineArray_ML[0];
    rgbColor[1] = rgbLineArray_ML[1];
    rgbColor[2] = rgbLineArray_ML[2];
  }
  else if (measureColorFlag == 1)
  {
    width = 3.0f;

    rgbColor[0] = 1.0f;
    rgbColor[1] = 0.5f;
    rgbColor[2] = 0.0f;
  }
  else
  {
    width = 4.0f;

    rgbColor[0] = 1.0f;
    rgbColor[1] = 0.0f;
    rgbColor[2] = 0.0f;
  }

  if (At3d_MeasureList_Add_Current(buffer, width, &rgbColor[0]) > -1)
  {
    Measure_ID_List_Add(buffer);
  }

  GetMeasureCountClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SaveImageClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if ( (!fileLoaded) ||
       (inName[0] == '\0') )
  {
    return;
  }

  // --

  Int32 width;
  Int32 height;
  Int32 maxSize = At3d_Get_GPU_Max_Texture_Size();

  try
  {
    width = ImageWidth->Text.ToInt();
  }
  catch(...)
  {
    return;
  }

  try
  {
    height = ImageHeight->Text.ToInt();
  }
  catch(...)
  {
    return;
  }

  float line_width;
  float point_size;

  try
  {
    line_width = my_double_to_float(ImageLineWidth->Text.ToDouble());
  }
  catch(...)
  {
    line_width = 1.0f;
  }

  try
  {
    point_size = my_double_to_float(ImagePointSize->Text.ToDouble());
  }
  catch(...)
  {
    point_size = 1.0f;
  }

  if ( (width > 0) &&
       (width <= maxSize) &&
       (height > 0) &&
       (height <= maxSize) &&
       (line_width > 0.0f) &&
       (point_size > 0.0f) )
  {
    char filename[_MAX_PATH];
    char drive0[_MAX_DRIVE];
    char dir0[_MAX_DIR];
    char name0[_MAX_FNAME];
    char ext0[_MAX_EXT];

    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(filename, _MAX_PATH, drive0);
    my_strcat(filename, _MAX_PATH, dir0);
    my_strcat(filename, _MAX_PATH, name0);

    // adding a suffix to avoid overwriting a JPG file used as a UV texture
    // if it had the same name as the model file
    //
    my_strcat(filename, _MAX_PATH, "-view_window");

    my_strcat(filename, _MAX_PATH, ".jpg");

    At3d_Save_Thumbnail_LineWidth_PointSize(line_width, point_size);

    At3d_Save_Thumbnail_Image(filename, width, height);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::StatusBar_ShowClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 left;
  Int32 top;
  Int32 width;
  Int32 height;

  try
  {
    left = StatusBar_Left->Text.ToInt();
  }
  catch(...)
  {
    return;
  }

  try
  {
    top = StatusBar_Top->Text.ToInt();
  }
  catch(...)
  {
    return;
  }

  try
  {
    width = StatusBar_Width->Text.ToInt();
  }
  catch(...)
  {
    return;
  }

  try
  {
    height = StatusBar_Height->Text.ToInt();
  }
  catch(...)
  {
    return;
  }

  At3d_StatusBar_Show( left, top, width, height );
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::StatusBar_HideClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_StatusBar_Hide();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::FontColorPanelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  TColorDialog* ColorDlg;
  try
  {
    ColorDlg = new TColorDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    ColorDlg = MY_NULL_PTR;
  }

  if (ColorDlg)
  {
    ColorDlg->Options = ColorDlg->Options << cdFullOpen;
    ColorDlg->Color = static_cast<TColor>(fontColor);

    ColorDlg->Execute();

    fontColor = ColorDlg->Color;
    FontColorPanel->Color = static_cast<TColor>(fontColor);

    delete ColorDlg;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetFontColorClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_StatusBar_Set_Font_Color(fontColor);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetFontNameClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, FontName->Text);

  char buffer[LEN_LAYER_NAME];
  my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  buffer[LEN_LAYER_NAME - 1] = '\0';

  At3d_StatusBar_Set_Font_Name(buffer);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetFontSizeClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 size;

  try
  {
    size = FontSize->Text.ToInt();
  }
  catch(...)
  {
    return;
  }

  At3d_StatusBar_Set_Font_Size(size);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetFontStyleClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 boldFlag      = (FontBold->Checked)      ? 1 : 0;
  Int32 italicFlag    = (FontItalic->Checked)    ? 1 : 0;
  Int32 underlineFlag = (FontUnderline->Checked) ? 1 : 0;
  Int32 strikeOutFlag = (FontStrikeOut->Checked) ? 1 : 0;

  At3d_StatusBar_Set_Font_Style(boldFlag, italicFlag, underlineFlag, strikeOutFlag);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetStatusBarTextClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, StatusBar_Text->Text);

  char buffer[LEN_LAYER_NAME];
  my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  buffer[LEN_LAYER_NAME - 1] = '\0';

  At3d_StatusBar_Set_Text(buffer);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetMeasureCountClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 count = At3d_MeasureList_Get_Count();

  MeasureCount->Text = count;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ClearMeasureListClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_MeasureList_Clear();

  Measure_ID_List_Clear();

  GetMeasureCountClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetMeasurementClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 index;
  MEASURE_DATA data;

  try
  {
    index = Get_Index->Text.ToInt();
  }
  catch(...)
  {
    return;
  }

  if (index == -1)
  {
    AnsiString tempID;
    my_copystring_to_ansi_from_vcl(&tempID, Get_ID->Text);

    my_strncpy(data.id, LEN_LAYER_NAME, tempID.c_str(), LEN_LAYER_NAME - 1);
    data.id[LEN_LAYER_NAME - 1] = '\0';
  }

  // --

  Int32 result = Get_MeasureList_Item(index, &data);

  Display_Measure_Data(result, &data);
}
//---------------------------------------------------------------------------

MEASURE_DATA* __fastcall TTest_At3dx_LocationsForm::Alloc_Measure_Data(void)
{
  MEASURE_DATA* ptr = MY_NULL_PTR;

  try
  {
    ptr = new MEASURE_DATA;

    if (ptr)
    {
      my_memset(ptr, 0, sizeof(MEASURE_DATA));
    }
  }
  catch(...)
  {
    ptr = MY_NULL_PTR;
  }

  return(ptr);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Clear_measureData_List(void)
{
  if (!measureData_ListPtr) return;

  // --

  Int32 iEnd = measureData_ListPtr->Count;
  if (iEnd)
  {
    for(Int32 i=0; i<iEnd; i++)
    {
      MEASURE_DATA* ptr = reinterpret_cast<MEASURE_DATA*>(measureData_ListPtr->Items[i]);

      if (ptr)
      {
        delete ptr;
      }
    }

    measureData_ListPtr->Clear();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Display_Measure_Data(Int32 index,
                                                         MEASURE_DATA* dataPtr)
{
  if (!dataPtr) return;

  // --

  if (index == -1)
  {
    my_memset(dataPtr, 0, sizeof(MEASURE_DATA));
  }

  Measure_Index->Text = index;

  Measure_ID->Text = dataPtr->id;

  Measure_Visible_Flag->Checked     = (dataPtr->visible_Flag) ? 1 : 0;
  Measure_Observer_Flag->Checked    = (dataPtr->observer_Flag) ? 1 : 0;
  Measure_Obstruction_Flag->Checked = (dataPtr->obstruction_Flag) ? 1 : 0;

  // --

  Measure_Vertex1_X->Caption = dataPtr->vertex1[0];
  Measure_Vertex1_Y->Caption = dataPtr->vertex1[1];
  Measure_Vertex1_Z->Caption = dataPtr->vertex1[2];

  Measure_Vertex2_X->Caption = dataPtr->vertex2[0];
  Measure_Vertex2_Y->Caption = dataPtr->vertex2[1];
  Measure_Vertex2_Z->Caption = dataPtr->vertex2[2];

  // --

  Measure_Distance->Caption = dataPtr->distance;

  // --

  Measure_Line_Width->Text = static_cast<double>(dataPtr->lineWidth);

  Int32 red   = my_change_float_to_256_int(dataPtr->rgb_Line_Array[0]);
  Int32 green = my_change_float_to_256_int(dataPtr->rgb_Line_Array[1]);
  Int32 blue  = my_change_float_to_256_int(dataPtr->rgb_Line_Array[2]);

  Measure_Line_Color->Color = static_cast<TColor>(red + (green << 8) + (blue << 16));

  // --

  Measure_Observer_X->Caption = dataPtr->observer[0];
  Measure_Observer_Y->Caption = dataPtr->observer[1];
  Measure_Observer_Z->Caption = dataPtr->observer[2];

  // --

  Measure_Obstruction_X->Caption = dataPtr->obstruction[0];
  Measure_Obstruction_Y->Caption = dataPtr->obstruction[1];
  Measure_Obstruction_Z->Caption = dataPtr->obstruction[2];

  // --

  Measure_Obstruction_Distance->Caption = dataPtr->obstruction_Distance;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Transfer_All_FromClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!measureData_ListPtr) return;

  // --

  Int32 fromCount = At3d_MeasureList_Get_Count();

  if (!fromCount)
  {
    return;
  }

  // --

  for (Int32 i=0; i<fromCount; i++)
  {
    MEASURE_DATA* ptr = Alloc_Measure_Data();

    if (ptr)
    {
      measureData_ListPtr->Add(ptr);

      Get_MeasureList_Item(i, ptr);
    }
  }

  Transfer_Count->Text = measureData_ListPtr->Count;

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Transfer_Random5Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!measureData_ListPtr) return;

  // --

  Int32 iEnd = measureData_ListPtr->Count;

  if (iEnd < 5)
  {
    return;
  }

  for (Int32 i=0; i<5; i++)
  {
    Int32 index;

    if (!i)
    {
      index = 3;
    }
    else if (i == 1)
    {
      index = 1;
    }
    else if (i == 2)
    {
      index = 4;
    }
    else if (i == 3)
    {
      index = 2;
    }
    else
    {
      index = 0;
    }

    MEASURE_DATA* ptr = reinterpret_cast<MEASURE_DATA*>(measureData_ListPtr->Items[index]);

    if ( Add_MeasureList_Item(ptr) > -1 )
    {
      Measure_ID_List_Add(ptr->id);
    }
  }

  // --

  GetMeasureCountClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Transfer_All_ToClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!measureData_ListPtr) return;

  // --

  Int32 iEnd = measureData_ListPtr->Count;

  if (!iEnd)
  {
    return;
  }

  // --

  for (Int32 i=0; i<iEnd; i++)
  {
    MEASURE_DATA* ptr = reinterpret_cast<MEASURE_DATA*>(measureData_ListPtr->Items[i]);

    if (ptr)
    {
      if ( Add_MeasureList_Item(ptr) > -1 )
      {
        Measure_ID_List_Add(ptr->id);
      }
    }
  }

  GetMeasureCountClick(MY_NULL_PTR);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Transfer_ClearClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!measureData_ListPtr) return;

  // --

  Clear_measureData_List();
  Transfer_Count->Text = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::No_Ctrl_Key_PanClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  At3d_WriteFlag(LIBFLAG_NO_CTRL_KEY_PAN, (No_Ctrl_Key_Pan->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Measure_ID_List_Clear(void)
{
  Measure_ID_List->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Measure_ID_List_Add(char* idPtr)
{
  Measure_ID_List->Items->Add(idPtr);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Visible_AllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_MeasureList_Set_Visible_All(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Visible_NoneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_MeasureList_Set_Visible_All(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Visible_Set_OneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Measure_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      At3d_MeasureList_Set_Visible_One(-1, buffer, (Measure_ID_Visible->Checked) ? 1 : 0);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Only_One_VisibleClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    At3d_WriteFlag(LIBFLAG_NO_DRAW, 1);

    At3d_MeasureList_Set_Visible_All(0);

    At3d_MeasureList_Set_Visible_One(itemIndex, MY_NULL_PTR, 1);

    At3d_WriteFlag(LIBFLAG_NO_DRAW, 0);

    At3d_DrawWireframe();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DeleteTest1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Measure_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if ( At3d_MeasureList_Delete_Item(-1, buffer) > 0 )
      {
        Measure_ID_List->Items->Delete(itemIndex);
        GetMeasureCountClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DeleteTest2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    if ( At3d_MeasureList_Delete_Item(itemIndex, MY_NULL_PTR) > 0 )
    {
      Measure_ID_List->Items->Delete(itemIndex);
      GetMeasureCountClick(MY_NULL_PTR);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReplaceTest1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Measure_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      MEASURE_DATA data;

      my_strncpy(data.id, LEN_LAYER_NAME, buffer, LEN_LAYER_NAME - 1);

      if ( Get_MeasureList_Item(-1, &data) == itemIndex)
      {
        if (data.observer_Flag)
        {
          data.observer_DeltaXYZ[2] += 1.5;

          Replace_MeasureList_Item( -1, buffer, &data );
        }
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReplaceTest2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    MEASURE_DATA data;

    if ( Get_MeasureList_Item(itemIndex, &data) == itemIndex)
    {
      if (data.observer_Flag)
      {
        if (data.observer_DeltaXYZ[2] - 1.5 < 0.0)
        {
        }
        else
        {
          data.observer_DeltaXYZ[2] -= 1.5;

          my_strcat(data.id, LEN_LAYER_NAME, "7");

          if ( Replace_MeasureList_Item( itemIndex, MY_NULL_PTR, &data ) > 0 )
          {
            Measure_ID_List->Items->Strings[itemIndex] = data.id;
          }
        }
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SelectTest1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Measure_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      float rgbArray[3];

      rgbArray[0] = 0.0f;
      rgbArray[1] = 1.0f;
      rgbArray[2] = 0.0f;

      At3d_MeasureList_Set_Selected_Color(&rgbArray[0]);

      Int32 result = At3d_MeasureList_Set_Selected_Line(-1, buffer);

      if ( result > -1 )
      {
        MEASURE_DATA data;

        result = Get_MeasureList_Item(result, &data);

        Display_Measure_Data(result, &data);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SelectTest2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Measure_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    float rgbArray[3];

    rgbArray[0] = 1.0f;
    rgbArray[1] = 1.0f;
    rgbArray[2] = 1.0f;

    At3d_MeasureList_Set_Selected_Color(&rgbArray[0]);

    Int32 result = At3d_MeasureList_Set_Selected_Line(itemIndex, MY_NULL_PTR);

    if ( result > -1 )
    {
      MEASURE_DATA data;

      result = Get_MeasureList_Item(result, &data);

      Display_Measure_Data(result, &data);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SelectCancelClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_MeasureList_Cancel_Selected_Line();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SelectMeasureLineClick(TObject *Sender)

{
   sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_OGL_PICK_MEASURE_LINE, (SelectMeasureLine->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CenterAtOriginClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_SetObject_XYZCenter();
}
//---------------------------------------------------------------------------

double __fastcall TTest_At3dx_LocationsForm::Calculate_Length(double* vertex1_Ptr,
                                                              double* vertex2_Ptr)
{
  double distance = 0.0;

  if ( (!vertex1_Ptr) ||
       (!vertex2_Ptr) )
  {
    return(distance);
  }

  // --

  double x1 = *vertex1_Ptr++;
  double y1 = *vertex1_Ptr++;
  double z1 = *vertex1_Ptr;

  double x2 = *vertex2_Ptr++;
  double y2 = *vertex2_Ptr++;
  double z2 = *vertex2_Ptr;

  double x = x2 - x1;
  double y = y2 - y1;
  double z = z2 - z1;

  distance = sqrt( x * x + y * y + z * z );

  return(distance);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Calculate_Distances(void)
{
  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if (hitFlagObserver == MESH_LOCATION_HAS_XYZ)
    {
      rubberbandDistance = Calculate_Length(&xyzArrayObserver[0],
                                            &xyzArray2[0]);
    }
    else
    {
      rubberbandDistance = Calculate_Length(&xyzArray1[0],
                                            &xyzArray2[0]);
    }

    // --

    if (hitFlagObstruction == MESH_LOCATION_HAS_XYZ)
    {
      if (hitFlagObserver == MESH_LOCATION_HAS_XYZ)
      {
        obstructionDistance = Calculate_Length(&xyzArrayObserver[0],
                                               &xyzArrayObstruction[0]);
      }
      else
      {
        obstructionDistance = Calculate_Length(&xyzArray1[0],
                                               &xyzArrayObstruction[0]);
      }
    }
    else
    {
      obstructionDistance = 0.0;
    }
  }
  else
  {
    rubberbandDistance  = 0.0;
    obstructionDistance = 0.0;
  }
}
//---------------------------------------------------------------------------

Int32 __fastcall TTest_At3dx_LocationsForm::Add_MeasureList_Item(MEASURE_DATA* dataPtr)
{
  Int32 result = -1;

  if ( (!dllOpen) ||
       (!dataPtr) )
  {
    return(result);
  }

  // --

  result = At3d_MeasureList_Add_Item(dataPtr->id,
                                     dataPtr->visible_Flag,
                                     dataPtr->observer_Flag,
                                     dataPtr->obstruction_Flag,
                                     dataPtr->lineWidth,
                                     &dataPtr->rgb_Line_Array[0],
                                     &dataPtr->vertex1[0],
                                     &dataPtr->vertex2[0],
                                     &dataPtr->observer_DeltaXYZ[0],
                                     &dataPtr->obstruction[0]);

  return(result);
}
//---------------------------------------------------------------------------

Int32 __fastcall TTest_At3dx_LocationsForm::Get_MeasureList_Item(Int32 index, MEASURE_DATA* dataPtr)
{
  Int32 result = -1;

  if ( (!dllOpen) ||
       (!dataPtr) )
  {
    return(result);
  }

  // --

  result = At3d_MeasureList_Get_Item(index, dataPtr->id,
                                     &dataPtr->visible_Flag,
                                     &dataPtr->observer_Flag,
                                     &dataPtr->obstruction_Flag,
                                     &dataPtr->lineWidth,
                                     &dataPtr->rgb_Line_Array[0],
                                     &dataPtr->vertex1[0],
                                     &dataPtr->vertex2[0],
                                     &dataPtr->observer_DeltaXYZ[0],
                                     &dataPtr->obstruction[0]);

  if (result > -1)
  {
    if (dataPtr->observer_Flag)
    {
      dataPtr->observer[0] = dataPtr->vertex1[0] + dataPtr->observer_DeltaXYZ[0];
      dataPtr->observer[1] = dataPtr->vertex1[1] + dataPtr->observer_DeltaXYZ[1];
      dataPtr->observer[2] = dataPtr->vertex1[2] + dataPtr->observer_DeltaXYZ[2];

      dataPtr->distance = Calculate_Length(&dataPtr->observer[0],
                                           &dataPtr->vertex2[0]);
    }
    else
    {
      dataPtr->observer[0] = 0.0;
      dataPtr->observer[1] = 0.0;
      dataPtr->observer[2] = 0.0;

      dataPtr->distance = Calculate_Length(&dataPtr->vertex1[0],
                                           &dataPtr->vertex2[0]);
    }

    // --

    if (dataPtr->obstruction_Flag)
    {
      if (dataPtr->observer_Flag)
      {
        dataPtr->obstruction_Distance = Calculate_Length(&dataPtr->observer[0],
                                                         &dataPtr->obstruction[0]);
      }
      else
      {
        dataPtr->obstruction_Distance = Calculate_Length(&dataPtr->vertex1[0],
                                                         &dataPtr->obstruction[0]);
      }
    }
    else
    {
      dataPtr->obstruction_Distance = 0.0;
    }
  }



  return(result);
}
//---------------------------------------------------------------------------

Int32 __fastcall TTest_At3dx_LocationsForm::Replace_MeasureList_Item(Int32 index,
                                         char* bufferPtr, MEASURE_DATA* dataPtr)
{
  Int32 result = -1;

  if ( (!dllOpen) ||
       (!dataPtr) )
  {
    return(result);
  }

  // --

  result =  At3d_MeasureList_Replace_Item(index, bufferPtr,
                                          dataPtr->id,
                                          dataPtr->visible_Flag,
                                          dataPtr->observer_Flag,
                                          dataPtr->obstruction_Flag,
                                          dataPtr->lineWidth,
                                          &dataPtr->rgb_Line_Array[0],
                                          &dataPtr->vertex1[0],
                                          &dataPtr->vertex2[0],
                                          &dataPtr->observer_DeltaXYZ[0],
                                          &dataPtr->obstruction[0]);

  return(result);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MeasureColor_DefaultClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  measureColorFlag = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MeasureColor_Sniper1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  measureColorFlag = 1;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MeasureColor_Sniper2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  measureColorFlag = 2;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetStatusbarTextClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  char buffer[80];
  buffer[0] = '\0';

  At3d_StatusBar_Get_Text(buffer, 80);

  if (buffer[0] == '\0')
  {
    Statusbar_ReadText->Clear();
  }
  else
  {
    Statusbar_ReadText->Text = buffer;
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Export_MeasurementsClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  UInt32 bufferSize = sizeof(MEASURE_DATA);
  Int32 count = At3d_MeasureList_Get_Count(); // from DLL
  if (!count)
  {
    return;
  }

  // --

  if (inName[0] == 0)
  {
    return;
  }

  // --

  char mesName[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
               ext0, _MAX_EXT);

  my_makepath(mesName, _MAX_PATH, drive0, dir0, name0, ".mes");

  FILE* fh = fopen(mesName, "wb");

  if (!fh)
  {
    return;
  }

  // --

  MEASURE_DATA* ptr = Alloc_Measure_Data();

  if (ptr)
  {
    for (Int32 i = 0; i < count; i++)
    {
      At3d_MeasureList_Get_Item(i, ptr->id, &ptr->visible_Flag,
                                &ptr->observer_Flag, &ptr->obstruction_Flag,
                                &ptr->lineWidth, &ptr->rgb_Line_Array[0],
                                &ptr->vertex1[0], &ptr->vertex2[0],
                                &ptr->observer_DeltaXYZ[0], &ptr->obstruction[0]);

      fwrite(ptr, 1, bufferSize, fh);

    }

    delete ptr;
  }

  // --

  fclose(fh);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Import_MeasurementsClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (inName[0] == 0)
  {
    return;
  }

  // --

  UInt32 bufferSize = sizeof(MEASURE_DATA);

  char mesName[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
               ext0, _MAX_EXT);

  my_makepath(mesName, _MAX_PATH, drive0, dir0, name0, ".mes");

  FILE* fh = fopen(mesName, "rb");

  if (!fh)
  {
    return;
  }

  // --

  MEASURE_DATA* ptr = Alloc_Measure_Data();

  if (ptr)
  {
    for (;;)
    {
      size_t bytesRead = fread(ptr, bufferSize, 1, fh);
      if (!bytesRead)
      {
        break;
      }

      if (At3d_MeasureList_Add_Item(ptr->id, ptr->visible_Flag,
                                ptr->observer_Flag, ptr->obstruction_Flag,
                                ptr->lineWidth, &ptr->rgb_Line_Array[0],
                                &ptr->vertex1[0], &ptr->vertex2[0],
                                &ptr->observer_DeltaXYZ[0], &ptr->obstruction[0]) > -1)
      {
        Measure_ID_List_Add(ptr->id);
      }
    }

    delete ptr;
  }

  // --

  fclose(fh);

  // --

  GetMeasureCountClick(MY_NULL_PTR);

  if (At3d_MeasureList_Get_Count() > 0)
  {
    // check the checkbox which signals the DLL to display the measurments
    //
    DisplayMeasureList->Checked = 1;
  }

}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::No_Reduce_Vert_Edge_PolyClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_NO_DUPLICATE_VERT_EDGE_POLY_REDUCE,
                 (No_Reduce_Vert_Edge_Poly->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::InvertSelectedColorClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_INVERT_COLOR_SELECTED_MEASURE_LINE,
                 (InvertSelectedColor->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayBillboardsClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_DISPLAY_MEASURE_BILLBOARDS,
                 (DisplayBillboards->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::MakeBillboardClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, BillboardText->Text);

  Graphics::TBitmap* bmpPtr;

  try
  {
    bmpPtr = new Graphics::TBitmap;
  }
  catch(...)
  {
    return;
  }

  bmpPtr->PixelFormat = pf24bit;

  bmpPtr->Canvas->Font->Size = -48;

  if (BillboardFontColor1->Checked)
  {
    bmpPtr->Canvas->Font->Color = clRed;
  }
  else if (BillboardFontColor2->Checked)
  {
    bmpPtr->Canvas->Font->Color = clGreen;
  }
  else
  {
    bmpPtr->Canvas->Font->Color = clBlue;
  }

  bmpPtr->Canvas->Font->Style= TFontStyles() << System::Uitypes::fsBold;
  bmpPtr->Canvas->Brush->Style=bsSolid;
  bmpPtr->Canvas->Brush->Color=clBlack;

  Int32 imageWidth  = bmpPtr->Canvas->TextWidth(tempName.c_str());
  Int32 imageHeight = bmpPtr->Canvas->TextHeight(tempName.c_str());

  bmpPtr->Width  = imageWidth;
  bmpPtr->Height = imageHeight;

  bmpPtr->Canvas->TextOut(0, 0,tempName.c_str());

  // --

  Int32 alphaFlag = (BillboardAlpha->Checked) ? 1 : 0;

  Int32 rowBytes;

  if (alphaFlag)
  {
    rowBytes = imageWidth * 4;
  }
  else
  {
    rowBytes = ((3 * imageWidth + 3) / 4) * 4;
  }

  UInt8* imagePtr;

  try
  {
    imagePtr = new UInt8[static_cast<size_t>(rowBytes * imageHeight)];
  }
  catch(...)
  {
    imagePtr = MY_NULL_PTR;;
  }

  if (imagePtr)
  {
    for (Int32 y=0; y<imageHeight; y++)
    {
      UInt8* ptr0 = reinterpret_cast<UInt8*>(bmpPtr->ScanLine[y]);
      UInt8* ptr1 = imagePtr + y * rowBytes;

      for (Int32 x=0; x<imageWidth; x++)
      {
        Int32 count = *ptr0;
        *ptr1++ = *ptr0++;

        count += *ptr0;
        *ptr1++ = *ptr0++;

        count += *ptr0;
        *ptr1++ = *ptr0++;

        if (alphaFlag)
        {
          *ptr1++ = (count < 10) ? 0 : 255;
        }
      }
    }

    // --

    float rgb_Line_Array[3];

    if (BillboardFontColor1->Checked)
    {
      rgb_Line_Array[0] = 1.0f;
      rgb_Line_Array[1] = 0.0f;
      rgb_Line_Array[2] = 0.0f;
    }
    else if (BillboardFontColor2->Checked)
    {
      rgb_Line_Array[0] = 0.0f;
      rgb_Line_Array[1] = 1.0f;
      rgb_Line_Array[2] = 0.0f;
    }
    else
    {
      rgb_Line_Array[0] = 0.0f;
      rgb_Line_Array[1] = 0.0f;
      rgb_Line_Array[2] = 1.0f;
    }

    // --

    float billboardLineWidth;

    try
    {
      billboardLineWidth = my_double_to_float(BillboardLineWidth->Text.ToDouble());
    }
    catch(...)
    {
      billboardLineWidth = 1.0f;
    }

    // --

    double height;

    try
    {
      height = BillboardHeight->Text.ToDouble();
    }
    catch(...)
    {
      height = 1.0;
    }

    // --

    double position[3];

    try
    {
      position[0] = BillboardPositionX->Text.ToDouble();
    }
    catch(...)
    {
      position[0] = 0.0;
    }

    try
    {
      position[1] = BillboardPositionY->Text.ToDouble();
    }
    catch(...)
    {
      position[1] = 0.0;
    }

    try
    {
      position[2] = BillboardPositionZ->Text.ToDouble();
    }
    catch(...)
    {
      position[2] = 0.0;
    }

    // --

    double pivot[3];

    try
    {
      pivot[0] = BillboardPivotX->Text.ToDouble();
    }
    catch(...)
    {
      pivot[0] = 0.0;
    }

    try
    {
      pivot[1] = BillboardPivotY->Text.ToDouble();
    }
    catch(...)
    {
      pivot[1] = 0.0;
    }

    try
    {
      pivot[2] = BillboardPivotZ->Text.ToDouble();
    }
    catch(...)
    {
      pivot[2] = 0.0;
    }


    // --

    double rotation[3];

    try
    {
      rotation[0] = BillboardRotateX->Text.ToDouble();
    }
    catch(...)
    {
      rotation[0] = 0.0;
    }

    try
    {
      rotation[1] = BillboardRotateY->Text.ToDouble();
    }
    catch(...)
    {
      rotation[1] = 0.0;
    }

    try
    {
      rotation[2] = BillboardRotateZ->Text.ToDouble();
    }
    catch(...)
    {
      rotation[2] = 0.0;
    }

    // --

    Int32 typeFlag;

    try
    {
      typeFlag = BillboardType->Text.ToInt();
    }
    catch(...)
    {
      typeFlag = 0;
    }

    // --

    my_copystring_to_ansi_from_vcl(&tempName, BillboardID->Text);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    // --

    if (At3d_Measure_Billboard_Add_Item(buffer, imagePtr, imageWidth,
                                        imageHeight, alphaFlag,
                                        BillboardPivotTypeList->ItemIndex,
                                        (BillboardStatic->Checked) ? 1 : 0,
                                        (BillboardVisible->Checked) ? 1 : 0,
                                        (BillboardOglReplace->Checked) ? 1 : 0,
                                        typeFlag,
                                        &rgb_Line_Array[0], billboardLineWidth,
                                        height, &position[0], &pivot[0],
                                        &rotation[0]) > -1)
    {
      if (Billboard_ID_List->Items->Add(buffer) > -1)
      {
        Billboard_Get_CountClick(MY_NULL_PTR);
      }
    }
  }

  // --

  if (imagePtr) delete[] imagePtr;

  if (bmpPtr) delete bmpPtr;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DeleteAllBillboardsClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (At3d_Measure_Billboard_Delete_All() > 0)
  {
    Billboard_ID_List->Clear();
    Billboard_Get_CountClick(MY_NULL_PTR);
  }
}
//---------------------------------------------------------------------------


void __fastcall TTest_At3dx_LocationsForm::No_Ctrl_Key_ZoomClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_NO_CTRL_KEY_ZOOM, (No_Ctrl_Key_Zoom->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetCameraRotateDegreesClick(TObject *Sender)
{
   sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  double degrees;

  try
  {
    degrees = CameraRotateDegrees->Text.ToDouble();
  }
  catch(...)
  {
    degrees = 0.0;
  }

  At3d_Camera_Rotate_Set_Speed(degrees);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::TwoSidedLightingClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_OGL_2SIDED_LIGHTING, (TwoSidedLighting->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CullBackFacesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_OGL_CULL_BACKFACES, (CullBackFaces->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::NoMouseButtonMoveActivityClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_NO_MOUSE_BOTTON_MOVE_ACTIVITY, (NoMouseButtonMoveActivity->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::NoMouseWheelActivityClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_NO_MOUSE_WHEEL_ACTIVITY, (NoMouseWheelActivity->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::No_Ctrl_Key_FOVClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_NO_CTRL_KEY_FOV, (No_Ctrl_Key_FOV->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::GetNoCtrlStepFOVClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  float step = At3d_No_Ctrl_Key_FOV_Get_Step();

  NoCtrlStepFOV->Text = static_cast<double>(step);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SetNoCtrlStepFOVClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  float step;

  try
  {
    step = my_double_to_float( NoCtrlStepFOV->Text.ToDouble() );
  }
  catch(...)
  {
    step = 0.1f;
  }

  At3d_No_Ctrl_Key_FOV_Set_Step(step);
}
//---------------------------------------------------------------------------
void __fastcall TTest_At3dx_LocationsForm::Billboard_Visible_AllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_Measure_Billboard_Set_Visible_All(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Visible_OneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_Measure_Billboard_Set_Visible_All(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Visible_Set_OneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      At3d_Measure_Billboard_Set_Visible_One(-1, buffer, (Billboard_ID_Visible->Checked) ? 1 : 0);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Only_One_VisibleClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    At3d_WriteFlag(LIBFLAG_NO_DRAW, 1);

    At3d_Measure_Billboard_Set_Visible_All(0);

    At3d_Measure_Billboard_Set_Visible_One(itemIndex, MY_NULL_PTR, 1);

    At3d_WriteFlag(LIBFLAG_NO_DRAW, 0);

    At3d_DrawWireframe();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_DeleteTest1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if ( At3d_Measure_Billboard_Delete_Item(-1, buffer) > 0 )
      {
        Billboard_ID_List->Items->Delete(itemIndex);
        Billboard_Get_CountClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_DeleteTest2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    if ( At3d_Measure_Billboard_Delete_Item(itemIndex, MY_NULL_PTR) > 0 )
    {
      Billboard_ID_List->Items->Delete(itemIndex);
      Billboard_Get_CountClick(MY_NULL_PTR);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetPivot1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double position[3];
      double pivot[3];

      if (At3d_Measure_Billboard_Get_Pivot(-1, buffer, &position[0], &pivot[0]) > 0)
      {
        Billboard_PositionX->Text = position[0];
        Billboard_PositionY->Text = position[1];
        Billboard_PositionZ->Text = position[2];

        Billboard_PivotX->Text = pivot[0];
        Billboard_PivotY->Text = pivot[1];
        Billboard_PivotZ->Text = pivot[2];
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetPivot2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    double position[3];
    double pivot[3];

    if (At3d_Measure_Billboard_Get_Pivot(itemIndex, MY_NULL_PTR, &position[0], &pivot[0]) > 0)
    {
      Billboard_PositionX->Text = position[0];
      Billboard_PositionY->Text = position[1];
      Billboard_PositionZ->Text = position[2];

      Billboard_PivotX->Text = pivot[0];
      Billboard_PivotY->Text = pivot[1];
      Billboard_PivotZ->Text = pivot[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetPivot1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double position[3];

      try
      {
        position[0] = Billboard_PositionX->Text.ToDouble();
      }
      catch(...)
      {
        position[0] = 0.0;
      }

      try
      {
        position[1] = Billboard_PositionY->Text.ToDouble();
      }
      catch(...)
      {
        position[1] = 0.0;
      }

      try
      {
        position[2] = Billboard_PositionZ->Text.ToDouble();
      }
      catch(...)
      {
        position[2] = 0.0;
      }

      // --

      double pivot[3];

      try
      {
        pivot[0] = Billboard_PivotX->Text.ToDouble();
      }
      catch(...)
      {
        pivot[0] = 0.0;
      }

      try
      {
        pivot[1] = Billboard_PivotY->Text.ToDouble();
      }
      catch(...)
      {
        pivot[1] = 0.0;
      }

      try
      {
        pivot[2] = Billboard_PivotZ->Text.ToDouble();
      }
      catch(...)
      {
        pivot[2] = 0.0;
      }

      // --

      At3d_Measure_Billboard_Set_Pivot(-1, buffer, &position[0], &pivot[0]);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetPivot2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    double position[3];

    try
    {
      position[0] = Billboard_PositionX->Text.ToDouble();
    }
    catch(...)
    {
      position[0] = 0.0;
    }

    try
    {
      position[1] = Billboard_PositionY->Text.ToDouble();
    }
    catch(...)
    {
      position[1] = 0.0;
    }

    try
    {
      position[2] = Billboard_PositionZ->Text.ToDouble();
    }
    catch(...)
    {
      position[2] = 0.0;
    }

    // --

    double pivot[3];

    try
    {
      pivot[0] = Billboard_PivotX->Text.ToDouble();
    }
    catch(...)
    {
      pivot[0] = 0.0;
    }

    try
    {
      pivot[1] = Billboard_PivotY->Text.ToDouble();
    }
    catch(...)
    {
      pivot[1] = 0.0;
    }

    try
    {
      pivot[2] = Billboard_PivotZ->Text.ToDouble();
    }
    catch(...)
    {
      pivot[2] = 0.0;
    }

    // --

    At3d_Measure_Billboard_Set_Pivot(itemIndex, MY_NULL_PTR, &position[0], &pivot[0]);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetHeight1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double height;

      if (At3d_Measure_Billboard_Get_Height(-1, buffer, &height) > 0)
      {
        Billboard_Height->Text = height;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetHeight2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    double height;

    if (At3d_Measure_Billboard_Get_Height(itemIndex, MY_NULL_PTR, &height) > 0)
    {
      Billboard_Height->Text = height;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetHeight1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double height;

      try
      {
        height = Billboard_Height->Text.ToDouble();
      }
      catch(...)
      {
        height = 1.0;
      }

      At3d_Measure_Billboard_Set_Height(-1, buffer, height);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetHeight2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    double height;

    try
    {
      height = Billboard_Height->Text.ToDouble();
    }
    catch(...)
    {
      height = 1.0;
    }

    At3d_Measure_Billboard_Set_Height(itemIndex, MY_NULL_PTR, height);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetIDClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    char buffer[LEN_LAYER_NAME];

    if (At3d_Measure_Billboard_Get_ID(itemIndex, buffer) > 0)
    {
      Billboard_ID->Text = AnsiString(buffer);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetIDClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID->Text);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if ( At3d_Measure_Billboard_Set_ID(itemIndex, buffer) > 0 )
    {
      Billboard_ID_List->Items->Strings[itemIndex] = buffer;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Image_From_DLL(Int32 index, char* idPtr)
{
  if (!dllOpen) return;

  if (billboardImagePtr)
  {
    delete[] billboardImagePtr;
    billboardImagePtr = MY_NULL_PTR;
  }

  if ( At3d_Measure_Billboard_Get_Image_Size(index, idPtr, &billboardImageWidth,
                                             &billboardImageHeight,
                                             &billboardAlphaFlag) > 0 )
  {
    Int32 rowBytes;

    if (billboardAlphaFlag)
    {
      rowBytes = billboardImageWidth * 4;
    }
    else
    {
      rowBytes = ((3 * billboardImageWidth + 3) / 4) * 4;
    }

    Int32 imageSize = rowBytes * billboardImageHeight;

    try
    {
      billboardImagePtr = new UInt8[static_cast<size_t>(imageSize)];
    }
    catch(...)
    {
      billboardImagePtr = MY_NULL_PTR;;
    }

    if (billboardImagePtr)
    {
      At3d_Measure_Billboard_Get_Image(index, idPtr, billboardImagePtr, imageSize);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetImage1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Get_Image_From_DLL(-1, buffer);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetImage2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Get_Image_From_DLL(itemIndex, MY_NULL_PTR);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetImage1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (!billboardImagePtr)
  {
    return;
  }

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      At3d_Measure_Billboard_Set_Image(-1, buffer, billboardImagePtr,
                                       billboardImageWidth, billboardImageHeight,
                                       billboardAlphaFlag);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetImage2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (!billboardImagePtr)
  {
    return;
  }

  // --

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    At3d_Measure_Billboard_Set_Image(itemIndex, MY_NULL_PTR, billboardImagePtr,
                                     billboardImageWidth, billboardImageHeight,
                                     billboardAlphaFlag);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::BillboardPosition_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      BillboardPositionX->Text = xyzArrayObserver[0];
      BillboardPositionY->Text = xyzArrayObserver[1];
      BillboardPositionZ->Text = xyzArrayObserver[2];
    }
    else
    {
      BillboardPositionX->Text = xyzArray1[0];
      BillboardPositionY->Text = xyzArray1[1];
      BillboardPositionZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::BillboardPosition_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    BillboardPositionX->Text = xyzArray2[0];
    BillboardPositionY->Text = xyzArray2[1];
    BillboardPositionZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::BillboardPivot_V1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      BillboardPivotX->Text = xyzArrayObserver[0];
      BillboardPivotY->Text = xyzArrayObserver[1];
      BillboardPivotZ->Text = xyzArrayObserver[2];
    }
    else
    {
      BillboardPivotX->Text = xyzArray1[0];
      BillboardPivotY->Text = xyzArray1[1];
      BillboardPivotZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::BillboardPivot_V2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    BillboardPivotX->Text = xyzArray2[0];
    BillboardPivotY->Text = xyzArray2[1];
    BillboardPivotZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_PivotTypeGet1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 pivotType = 0;

      if (At3d_Measure_Billboard_Get_Pivot_Type(-1, buffer, &pivotType) > 0)
      {
        Billboard_PivotTypeList->ItemIndex = pivotType;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_PivotTypeGet2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Int32 pivotType = 0;

    if (At3d_Measure_Billboard_Get_Pivot_Type(itemIndex, MY_NULL_PTR, &pivotType) > 0)
    {
      Billboard_PivotTypeList->ItemIndex = pivotType;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_PivotTypeSet1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 pivotType = Billboard_PivotTypeList->ItemIndex;

      if (pivotType > -1)
      {
        At3d_Measure_Billboard_Set_Pivot_Type(-1, buffer, pivotType);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_PivotTypeSet2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Int32 pivotType = Billboard_PivotTypeList->ItemIndex;

    if (pivotType > -1)
    {
      At3d_Measure_Billboard_Set_Pivot_Type(itemIndex, MY_NULL_PTR, pivotType);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetOglReplace1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 oglReplace = 0;

      if (At3d_Measure_Billboard_Get_OglReplace(-1, buffer, &oglReplace) > 0)
      {
        Billboard_OglReplace->Checked = oglReplace;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetOglReplace2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Int32 oglReplace;

    if (At3d_Measure_Billboard_Get_OglReplace(itemIndex, MY_NULL_PTR, &oglReplace) > 0)
    {
      Billboard_OglReplace->Checked = oglReplace;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetOglReplace1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      At3d_Measure_Billboard_Set_OglReplace(-1, buffer, (Billboard_OglReplace->Checked) ? 1 : 0);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetOglReplace2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    At3d_Measure_Billboard_Set_OglReplace(itemIndex, MY_NULL_PTR, (Billboard_OglReplace->Checked) ? 1 : 0);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetStatic1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 staticFlag = 0;

      if (At3d_Measure_Billboard_Get_Static(-1, buffer, &staticFlag) > 0)
      {
        Billboard_Static->Checked = staticFlag;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetStatic2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Int32 staticFlag;

    if (At3d_Measure_Billboard_Get_Static(itemIndex, MY_NULL_PTR, &staticFlag) > 0)
    {
      Billboard_Static->Checked = staticFlag;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetStatic1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      At3d_Measure_Billboard_Set_Static(-1, buffer, (Billboard_Static->Checked) ? 1 : 0);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetStatic2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    At3d_Measure_Billboard_Set_Static(itemIndex, MY_NULL_PTR, (Billboard_Static->Checked) ? 1 : 0);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetRotation1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double rotation[3];

      if (At3d_Measure_Billboard_Get_Static_Angles(-1, buffer, &rotation[0]) > 0)
      {
        Billboard_RotateX->Text = rotation[0];
        Billboard_RotateY->Text = rotation[1];
        Billboard_RotateZ->Text = rotation[2];
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetRotation2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    double rotation[3];

    if (At3d_Measure_Billboard_Get_Static_Angles(itemIndex, MY_NULL_PTR, &rotation[0]) > 0)
    {
      Billboard_RotateX->Text = rotation[0];
      Billboard_RotateY->Text = rotation[1];
      Billboard_RotateZ->Text = rotation[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetRotation1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double rotation[3];

      try
      {
        rotation[0] = Billboard_RotateX->Text.ToDouble();
      }
      catch(...)
      {
        rotation[0] = 0.0;
      }

      try
      {
        rotation[1] = Billboard_RotateY->Text.ToDouble();
      }
      catch(...)
      {
        rotation[1] = 0.0;
      }

      try
      {
        rotation[2] = Billboard_RotateZ->Text.ToDouble();
      }
      catch(...)
      {
        rotation[2] = 0.0;
      }

      At3d_Measure_Billboard_Set_Static_Angles(-1, buffer, &rotation[0]);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetRotation2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    double rotation[3];

    try
    {
      rotation[0] = Billboard_RotateX->Text.ToDouble();
    }
    catch(...)
    {
      rotation[0] = 0.0;
    }

    try
    {
      rotation[1] = Billboard_RotateY->Text.ToDouble();
    }
    catch(...)
    {
      rotation[1] = 0.0;
    }

    try
    {
      rotation[2] = Billboard_RotateZ->Text.ToDouble();
    }
    catch(...)
    {
      rotation[2] = 0.0;
    }

    At3d_Measure_Billboard_Set_Static_Angles(itemIndex, MY_NULL_PTR, &rotation[0]);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Copy_DeleteClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // This function copies the billboard data from the DLL
  // and then the billboard is deleted from the DLL
  //
  if (billboardData.imagePtr)
  {
    delete[] billboardData.imagePtr;
    billboardData.imagePtr = MY_NULL_PTR;
    billboardData.imageSize = 0;
  }

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    my_strncpy(billboardData.id, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    billboardData.id[LEN_LAYER_NAME - 1] = '\0';

    Int32 index = (Billboard_Copy_Index->Checked) ? itemIndex : -1;

    if (index > -1) my_memset(billboardData.id, 0, LEN_LAYER_NAME);

    if (Billboard_Copy_Image->Checked)
    {
      Int32 width;
      Int32 height;
      Int32 alphaFlag;

      if ( At3d_Measure_Billboard_Get_Image_Size(index, billboardData.id,
                                                 &width, &height, &alphaFlag) > 0 )
      {
        Int32 rowBytes;

        if (alphaFlag)
        {
          rowBytes = width * 4;
        }
        else
        {
          rowBytes = ((3 * width + 3) / 4) * 4;
        }

        UInt8* imagePtr;
        Int32 imageSize = rowBytes * height;

        try
        {
          imagePtr = new UInt8[static_cast<size_t>(imageSize)];
        }
        catch(...)
        {
          imagePtr = MY_NULL_PTR;;
        }

        if (imagePtr)
        {
          billboardData.imagePtr  = imagePtr;
          billboardData.imageSize = imageSize;
        }
      }
    }

    if (At3d_Measure_Billboard_Get_Item(index, billboardData.id,
                                        billboardData.imagePtr,
                                        billboardData.imageSize,
                                        &billboardData.imageWidth,
                                        &billboardData.imageHeight,
                                        &billboardData.alphaFlag,
                                        &billboardData.pivotTypeFlag,
                                        &billboardData.staticFlag,
                                        &billboardData.visibleFlag,
                                        &billboardData.oglReplaceFlag,
                                        &billboardData.typeFlag,
                                        &billboardData.rgb_Line_Array[0],
                                        &billboardData.lineWidth,
                                        &billboardData.height,
                                        &billboardData.position[0],
                                        &billboardData.pivot[0],
                                        &billboardData.rotationStatic[0]) > 0)
    {
      if ( At3d_Measure_Billboard_Delete_Item(itemIndex, MY_NULL_PTR) > 0 )
      {
        Billboard_ID_List->Items->Delete(itemIndex);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_ReplaceClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  // This function copies the billboard data to an existing billboard in the DLL
  //
  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    Int32 index = (Billboard_Copy_Index->Checked) ? -1 : itemIndex;

    if (At3d_Measure_Billboard_Replace_Item(index, buffer,
                                            billboardData.id,
                                            billboardData.imagePtr,
                                            billboardData.imageWidth,
                                            billboardData.imageHeight,
                                            billboardData.alphaFlag,
                                            billboardData.pivotTypeFlag,
                                            billboardData.staticFlag,
                                            billboardData.visibleFlag,
                                            billboardData.oglReplaceFlag,
                                            billboardData.typeFlag,
                                            &billboardData.rgb_Line_Array[0],
                                            billboardData.lineWidth,
                                            billboardData.height,
                                            &billboardData.position[0],
                                            &billboardData.pivot[0],
                                            &billboardData.rotationStatic[0]) > 0)
    {
     Billboard_ID_List->Items->Strings[itemIndex] = billboardData.id;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Cancel_SelectedClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_Measure_Billboard_Cancel_Selected();
  Billboard_Selected_ID->Text = AnsiString();
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_InvertSelectedColorClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_INVERT_COLOR_SELECTED_MEASURE_BILLBOARD,
                 (Billboard_InvertSelectedColor->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetColorClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  float color[3];

  try
  {
    color[0] = my_double_to_float(Billboard_Selected_Red->Text.ToDouble());
  }
  catch(...)
  {
    color[0] = 1.0f;
  }

  try
  {
    color[1] = my_double_to_float(Billboard_Selected_Green->Text.ToDouble());
  }
  catch(...)
  {
    color[1] = 1.0f;
  }

  try
  {
    color[2] = my_double_to_float(Billboard_Selected_Blue->Text.ToDouble());
  }
  catch(...)
  {
    color[2] = 1.0f;
  }

  At3d_Measure_Billboard_Set_Selected_Color(&color[0]);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetWidthClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  float width;

  try
  {
    width = my_double_to_float(Billboard_Selected_Width->Text.ToDouble());
  }
  catch(...)
  {
    width = 1.0f;
  }

  At3d_Measure_Billboard_Set_Selected_Width(width);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Line_Get1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      float rgbArray[3];
      float width;

      if (At3d_Measure_Billboard_Get_Line(-1, buffer, &rgbArray[0],
                                          &width) > 0)
      {
        Billboard_Line_Red->Text   = static_cast<double>(rgbArray[0]);
        Billboard_Line_Green->Text = static_cast<double>(rgbArray[1]);
        Billboard_Line_Blue->Text  = static_cast<double>(rgbArray[2]);

        Billboard_Line_Width->Text = static_cast<double>(width);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Line_Get2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    float rgbArray[3];
    float width;;

    if (At3d_Measure_Billboard_Get_Line(itemIndex, MY_NULL_PTR, &rgbArray[0], &width) > 0)
    {
      Billboard_Line_Red->Text   = static_cast<double>(rgbArray[0]);
      Billboard_Line_Green->Text = static_cast<double>(rgbArray[1]);
      Billboard_Line_Blue->Text  = static_cast<double>(rgbArray[2]);

      Billboard_Line_Width->Text = static_cast<double>(width);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Line_Set1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      float rgbArray[3];

      try
      {
        rgbArray[0] = my_double_to_float(Billboard_Line_Red->Text.ToDouble());
      }
      catch(...)
      {
        rgbArray[0] = 0.0f;
      }

      try
      {
        rgbArray[1] = my_double_to_float(Billboard_Line_Green->Text.ToDouble());
      }
      catch(...)
      {
        rgbArray[1] = 0.0f;
      }

      try
      {
        rgbArray[2] = my_double_to_float(Billboard_Line_Blue->Text.ToDouble());
      }
      catch(...)
      {
        rgbArray[2] = 0.0f;
      }

      // --

      float width;

      try
      {
        width = my_double_to_float(Billboard_Line_Width->Text.ToDouble());
      }
      catch(...)
      {
        width = 1.0f;
      }

      // --

      At3d_Measure_Billboard_Set_Line(-1, buffer, &rgbArray[0], width);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Line_Set2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    float rgbArray[3];

    try
    {
      rgbArray[0] = my_double_to_float(Billboard_Line_Red->Text.ToDouble());
    }
    catch(...)
    {
      rgbArray[0] = 0.0f;
    }

    try
    {
      rgbArray[1] = my_double_to_float(Billboard_Line_Green->Text.ToDouble());
    }
    catch(...)
    {
      rgbArray[1] = 0.0f;
    }

    try
    {
      rgbArray[2] = my_double_to_float(Billboard_Line_Blue->Text.ToDouble());
    }
    catch(...)
    {
      rgbArray[2] = 0.0f;
    }

    // --

    float width;

    try
    {
      width = my_double_to_float(Billboard_Line_Width->Text.ToDouble());
    }
    catch(...)
    {
      width = 1.0f;
    }

    // --

    At3d_Measure_Billboard_Set_Line(itemIndex, MY_NULL_PTR, &rgbArray[0], width);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_Get_CountClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 count = At3d_Measure_Billboard_Get_Count();

  Billboard_Count->Text = count;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetType1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 type;

      if (At3d_Measure_Billboard_Get_Type(-1, buffer, &type) > 0)
      {
        Billboard_Type->Text = type;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_GetType2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Int32 type;

    if (At3d_Measure_Billboard_Get_Type(itemIndex, MY_NULL_PTR, &type) > 0)
    {
      Billboard_Type->Text = type;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetType1Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Billboard_ID_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 type;

      try
      {
        type = Billboard_Type->Text.ToInt();
      }
      catch(...)
      {
        type = 0;
      }

      At3d_Measure_Billboard_Set_Type(-1, buffer, type);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Billboard_SetType2Click(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = Billboard_ID_List->ItemIndex;

  if (itemIndex > -1)
  {
    Int32 type;

    try
    {
      type = Billboard_Type->Text.ToInt();
    }
    catch(...)
    {
      type = 0;
    }

    At3d_Measure_Billboard_Set_Type(itemIndex, MY_NULL_PTR, type);
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_LocationClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 screenX;
  Int32 screenY;

  try
  {
    screenX = Get_ScreenX->Text.ToInt();
  }
  catch(...)
  {
    screenX = 0;
  }

  try
  {
    screenY = Get_ScreenY->Text.ToInt();
  }
  catch(...)
  {
    screenY = 0;
  }

  Int32 hitFlag;
  double xyzArray[3];
  char layerName[LEN_LAYER_NAME];
  char cloneItemName[LEN_LAYER_NAME];

  Int32 result = At3d_Get_Mesh_Location_At_Point(screenX, screenY, &hitFlag,
                                                 &xyzArray[0], &layerName[0],
                                                 &cloneItemName[0]);

  if (result)
  {
    if (hitFlag == 2)
    {
      Get_MeshX->Caption = xyzArray[0];
      Get_MeshY->Caption = xyzArray[1];
      Get_MeshZ->Caption = xyzArray[2];

      Get_LayerName->Caption = layerName;
      Get_CloneItemName->Caption = cloneItemName;
    }
    else
    {
      Get_MeshX->Caption = "";
      Get_MeshY->Caption = "";
      Get_MeshZ->Caption = "";

      Get_LayerName->Caption = "";
      Get_CloneItemName->Caption = "";
    }
  }
  else
  {
    Get_MeshX->Caption = "";
    Get_MeshY->Caption = "";
    Get_MeshZ->Caption = "";

    Get_LayerName->Caption = "";
    Get_CloneItemName->Caption = "";
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Integrated_GPUClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_INTEGRATED_GPU, (Integrated_GPU->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Locations_Depth_TestClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_DISPLAY_MESH_DEPTH_TEST, (Locations_Depth_Test->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Get_Intersection_PointClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  double v1[3];

  try
  {
    v1[0] = LineMesh_V1_X->Text.ToDouble();
  }
  catch(...)
  {
    v1[0] = 0.0;
  }

  try
  {
    v1[1] = LineMesh_V1_Y->Text.ToDouble();
  }
  catch(...)
  {
    v1[1] = 0.0;
  }

  try
  {
    v1[2] = LineMesh_V1_Z->Text.ToDouble();
  }
  catch(...)
  {
    v1[2] = 0.0;
  }

  // --
  double v2[3];

  try
  {
    v2[0] = LineMesh_V2_X->Text.ToDouble();
  }
  catch(...)
  {
    v2[0] = 0.0;
  }

  try
  {
    v2[1] = LineMesh_V2_Y->Text.ToDouble();
  }
  catch(...)
  {
    v2[1] = 0.0;
  }

  try
  {
    v2[2] = LineMesh_V2_Z->Text.ToDouble();
  }
  catch(...)
  {
    v2[2] = 0.0;
  }

  // --

  double skipDistance;

  try
  {
    skipDistance = LineMesh_SkipDistance->Text.ToDouble();
  }
  catch(...)
  {
    skipDistance = 0.0;
  }

  // --

  double intersection[3];

  if ( At3d_Get_Line_Mesh_Intersection_Point(&v1[0], &v2[0], skipDistance, &intersection[0]) > 0 )
  {
    LineMesh_Intersection_X->Text = intersection[0];
    LineMesh_Intersection_Y->Text = intersection[1];
    LineMesh_Intersection_Z->Text = intersection[2];
  }
  else
  {
    LineMesh_Intersection_X->Text = AnsiString();
    LineMesh_Intersection_Y->Text = AnsiString();
    LineMesh_Intersection_Z->Text = AnsiString();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReadAddPosition_V1Click(TObject *Sender)
{
   sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      ReadAddPositionX->Text = xyzArrayObserver[0];
      ReadAddPositionY->Text = xyzArrayObserver[1];
      ReadAddPositionZ->Text = xyzArrayObserver[2];
    }
    else
    {
      ReadAddPositionX->Text = xyzArray1[0];
      ReadAddPositionY->Text = xyzArray1[1];
      ReadAddPositionZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReadAddPosition_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    ReadAddPositionX->Text = xyzArray2[0];
    ReadAddPositionY->Text = xyzArray2[1];
    ReadAddPositionZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read_Add_Static_MeshClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Screen->Cursor = TCursor(crHourGlass);
  Read_Add_File();
  Screen->Cursor = TCursor(crDefault);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Read_Add_File(void)
{
  if (!dllOpen) return;

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if (inName[0] != 0)
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ext0);
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr;
  const char* inputExtensionPtr;
  Int32 inputType;

  if (!inputFileType)
  {
    inputNamePtr      = "Wavefront";
    inputExtensionPtr = ".obj";
    inputType         = INTYPE_WAVEFRONT;
  }
  else
  {
    inputNamePtr      = "VRML";
    inputExtensionPtr = ".wrl";
    inputType         = INTYPE_VRML;
  }

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(inName, MAX_PATH, fileName.c_str());

    // before reading make sure flags are set properly
    // (note: flags maintain their setting until changed with this function)
    //
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_VERTICES, 1);
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_FACES, 1);
    At3d_WriteFlag(LIBFLAG_SHOW_READ_MESSAGES, 0);

    // --

    double scaleFactor;

    try
    {
      scaleFactor = ReadAddScaleFactor->Text.ToDouble();
    }
    catch(...)
    {
      scaleFactor = 1.0;
    }

    // --

    Int32 xRotationFlag;

    if (ReadAddRotateX_No->Checked)
    {
      xRotationFlag = ROTATE_ON_X_AXIS_NO;
    }
    else if (ReadAddRotateX_Minus90->Checked)
    {
      xRotationFlag = ROTATE_ON_X_AXIS_MINUS_90;
    }
    else if (ReadAddRotateX_Plus90->Checked)
    {
      xRotationFlag = ROTATE_ON_X_AXIS_PLUS_90;
    }
    else
    {
      xRotationFlag = ROTATE_ON_X_AXIS_180;
    }

    // --

    double position[3];

    try
    {
      position[0] = ReadAddPositionX->Text.ToDouble();
    }
    catch(...)
    {
      position[0] = 0.0;
    }

    try
    {
      position[1] = ReadAddPositionY->Text.ToDouble();
    }
    catch(...)
    {
      position[1] = 0.0;
    }

    try
    {
      position[2] = ReadAddPositionZ->Text.ToDouble();
    }
    catch(...)
    {
      position[2] = 0.0;
    }

    // --

    AnsiString tempPrefix;
    my_copystring_to_ansi_from_vcl(&tempPrefix, ReadAddPrefix->Text);

    char prefixBuffer[LEN_LAYER_NAME];
    my_strncpy(prefixBuffer, LEN_LAYER_NAME, tempPrefix.c_str(), LEN_LAYER_NAME - 1);
    prefixBuffer[LEN_LAYER_NAME - 1] = '\0';

    // --

    // read and add file
    //
    // the file read can contain more than one layer
    // all layers read can be saved to an OBJ or other 3D file if the layers are turned on
    //
    // a user defined prefix is added to every layer name
    // an underscore, '_', is automatically added to the prefix
    // this prefex is used with all functions that work with 'mesh static'
    //
    // the local origin of the object is placed at 'position'
    //
    // return value for all 'mesh static' functions
    // 0 = failure
    // 1 = success
    //
    if (At3d_Read_And_Add_Mesh_Static(inputType, inName, scaleFactor,
                                      xRotationFlag, &position[0], prefixBuffer))
    {
      AdjustMesh_List->Items->Add(prefixBuffer);
    }
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Pivot_CalculateClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      // when the pivot is calculated, X and Y are placed at the mid-point on
      // each axis and Z is placed at the lowest Z elevation
      //
      // if the original location of the pivot point is required, all
      // changes for it should be tracked and saved by the app as it can not
      // be saved to an OBJ or other 3D file.
      //
      double pivot[3];
      if (At3d_Calculate_Pivot_Mesh_Static(buffer, &pivot[0]) > 0)
      {
        AdjustMesh_PivotX->Text = pivot[0];
        AdjustMesh_PivotY->Text = pivot[1];
        AdjustMesh_PivotZ->Text = pivot[2];
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Pivot_GetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double pivot[3];
      if (At3d_Get_Pivot_Mesh_Static(buffer, &pivot[0]) > 0)
      {
        AdjustMesh_PivotX->Text = pivot[0];
        AdjustMesh_PivotY->Text = pivot[1];
        AdjustMesh_PivotZ->Text = pivot[2];
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Pivot_SetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  double pivot[3];

  try
  {
    pivot[0] = AdjustMesh_PivotX->Text.ToDouble();
  }
  catch(...)
  {
    pivot[0] = 0.0;
  }

  try
  {
    pivot[1] = AdjustMesh_PivotY->Text.ToDouble();
  }
  catch(...)
  {
    pivot[1] = 0.0;
  }

  try
  {
    pivot[2] = AdjustMesh_PivotZ->Text.ToDouble();
  }
  catch(...)
  {
    pivot[2] = 0.0;
  }

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Set_Pivot_Mesh_Static(buffer, &pivot[0]) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_RotateClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  double rotate[3];

  try
  {
    rotate[0] = AdjustMesh_RotateX->Text.ToDouble();
  }
  catch(...)
  {
    rotate[0] = 0.0;
  }

  try
  {
    rotate[1] = AdjustMesh_RotateY->Text.ToDouble();
  }
  catch(...)
  {
    rotate[1] = 0.0;
  }

  try
  {
    rotate[2] = AdjustMesh_RotateZ->Text.ToDouble();
  }
  catch(...)
  {
    rotate[2] = 0.0;
  }

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Rotate_Mesh_Static(buffer, &rotate[0]) > 0)
      {
        AdjustMesh_SizeClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_ScaleClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  double scale[3];

  try
  {
    scale[0] = AdjustMesh_ScaleX->Text.ToDouble();
  }
  catch(...)
  {
    scale[0] = 1.0;
  }

  try
  {
    scale[1] = AdjustMesh_ScaleY->Text.ToDouble();
  }
  catch(...)
  {
    scale[1] = 1.0;
  }

  try
  {
    scale[2] = AdjustMesh_ScaleZ->Text.ToDouble();
  }
  catch(...)
  {
    scale[2] = 1.0;
  }

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Scale_Mesh_Static(buffer, &scale[0]) > 0)
      {
        AdjustMesh_SizeClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_AdjustMesh_Translate(Int32 relativeFlag)
{
  if (!dllOpen) return;

  double translate[3];

  try
  {
    translate[0] = AdjustMesh_TranslateX->Text.ToDouble();
  }
  catch(...)
  {
    translate[0] = 0.0;
  }

  try
  {
    translate[1] = AdjustMesh_TranslateY->Text.ToDouble();
  }
  catch(...)
  {
    translate[1] = 1.0;
  }

  try
  {
    translate[2] = AdjustMesh_TranslateZ->Text.ToDouble();
  }
  catch(...)
  {
    translate[2] = 1.0;
  }

  // --

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Translate_Mesh_Static(buffer, relativeFlag, &translate[0]) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Translate_RelativeClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_AdjustMesh_Translate(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_TranslateClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_AdjustMesh_Translate(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_DeleteClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Delete_Mesh_Static(buffer) > 0)
      {
        AdjustMesh_List->Items->Delete(itemIndex);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_OnClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Set_Visible_Mesh_Static(buffer, 1) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_OffClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Set_Visible_Mesh_Static(buffer, 0) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_SizeClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = AdjustMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, AdjustMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double size[3];
      if (At3d_Get_Size_Mesh_Static(buffer, &size[0]) > 0)
      {
        AdjustMesh_SizeX->Text = size[0];
        AdjustMesh_SizeY->Text = size[1];
        AdjustMesh_SizeZ->Text = size[2];
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Pivot_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      AdjustMesh_PivotX->Text = xyzArrayObserver[0];
      AdjustMesh_PivotY->Text = xyzArrayObserver[1];
      AdjustMesh_PivotZ->Text = xyzArrayObserver[2];
    }
    else
    {
      AdjustMesh_PivotX->Text = xyzArray1[0];
      AdjustMesh_PivotY->Text = xyzArray1[1];
      AdjustMesh_PivotZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Pivot_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    AdjustMesh_PivotX->Text = xyzArray2[0];
    AdjustMesh_PivotY->Text = xyzArray2[1];
    AdjustMesh_PivotZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Translate_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      AdjustMesh_TranslateX->Text = xyzArrayObserver[0];
      AdjustMesh_TranslateY->Text = xyzArrayObserver[1];
      AdjustMesh_TranslateZ->Text = xyzArrayObserver[2];
    }
    else
    {
      AdjustMesh_TranslateX->Text = xyzArray1[0];
      AdjustMesh_TranslateY->Text = xyzArray1[1];
      AdjustMesh_TranslateZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::AdjustMesh_Translate_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    AdjustMesh_TranslateX->Text = xyzArray2[0];
    AdjustMesh_TranslateY->Text = xyzArray2[1];
    AdjustMesh_TranslateZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_AddClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Screen->Cursor = TCursor(crHourGlass);
  Add_Clone_Mesh(0);
  Screen->Cursor = TCursor(crDefault);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_ChangeClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Screen->Cursor = TCursor(crHourGlass);
  Add_Clone_Mesh(1);
  Screen->Cursor = TCursor(crDefault);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_Clone_Mesh(Int32 changeFlag)
{
  if (!dllOpen) return;

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if (inName[0] != 0)
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ext0);
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr;
  const char* inputExtensionPtr;
  //Int32 inputType;

  if (!inputFileType)
  {
    inputNamePtr      = "Wavefront";
    inputExtensionPtr = ".obj";
    //inputType         = INTYPE_WAVEFRONT;
  }
  else
  {
    inputNamePtr      = "VRML";
    inputExtensionPtr = ".wrl";
    //inputType         = INTYPE_VRML;
  }

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(inName, MAX_PATH, fileName.c_str());

    // before reading make sure flags are set properly
    // (note: flags maintain their setting until changed with this function)
    //
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_VERTICES, 1);
    At3d_WriteFlag(LIBFLAG_REMOVE_DUPLICATE_FACES, 1);
    At3d_WriteFlag(LIBFLAG_SHOW_READ_MESSAGES, 0);

    // --

    double scaleFactor;

    try
    {
      scaleFactor = Add_CloneMesh_ScaleFactor->Text.ToDouble();
    }
    catch(...)
    {
      scaleFactor = 1.0;
    }

    // --

    Int32 xRotationFlag;

    if (Add_CloneMesh_RotateX_No->Checked)
    {
      xRotationFlag = ROTATE_ON_X_AXIS_NO;
    }
    else if (Add_CloneMesh_RotateX_Minus90->Checked)
    {
      xRotationFlag = ROTATE_ON_X_AXIS_MINUS_90;
    }
    else if (Add_CloneMesh_RotateX_Plus90->Checked)
    {
      xRotationFlag = ROTATE_ON_X_AXIS_PLUS_90;
    }
    else
    {
      xRotationFlag = ROTATE_ON_X_AXIS_180;
    }

    // --

    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, Add_CloneMesh_Name->Text);

    char nameBuffer[LEN_LAYER_NAME];
    my_strncpy(nameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    nameBuffer[LEN_LAYER_NAME - 1] = '\0';
    At3d_CheckName(nameBuffer);
    Add_CloneMesh_Name->Text = nameBuffer;

    // --

    // add clone mesh
    //
    // the file containing the clone mesh can contain more than one layer
    //
    // meshes are displayed using clone items
    //
    // return value for all 'clone' functions
    // 0 = failure
    // 1 = success
    //
    if (changeFlag)
    {
      if (At3d_Change_File_Clone_Mesh(nameBuffer, inName, scaleFactor, xRotationFlag))
      {
        //
      }
    }
    else
    {
      if (At3d_Add_Clone_Mesh(nameBuffer, inName, scaleFactor, xRotationFlag))
      {
        CloneMesh_List->Items->Add(nameBuffer);
      }
    }
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_DeleteOneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Delete_Clone_Mesh(buffer) > 0)
      {
        CloneMesh_List->Items->Delete(itemIndex);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_DeleteAllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (At3d_Delete_All_Clone_Mesh() > 0)
  {
    CloneMesh_List->Clear();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_Change_MeshNameClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_List->Items->Strings[itemIndex]);

    char meshNameBuffer[LEN_LAYER_NAME];
    my_strncpy(meshNameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    meshNameBuffer[LEN_LAYER_NAME - 1] = '\0';

    // --

    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_New_MeshName->Text);

    char newMeshName[LEN_LAYER_NAME];
    my_strncpy(newMeshName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    newMeshName[LEN_LAYER_NAME - 1] = '\0';
    At3d_CheckName(newMeshName);
    CloneMesh_New_MeshName->Text = newMeshName;

    if ( (*meshNameBuffer != '\0') &&
         (*newMeshName  != '\0') )
    {
      if (At3d_Change_MeshName_Clone_Mesh(meshNameBuffer, newMeshName) > 0)
      {
        CloneMesh_List->Items->Strings[itemIndex] = newMeshName;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_GetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      char filename[LEN_FILE_PATH];
      Int32 xRotationFlag;
      double scaleFactor;

      if (At3d_Get_Data_Clone_Mesh(buffer, filename, &scaleFactor, &xRotationFlag) > 0)
      {
        Data_CloneMesh_Filename->Text = filename;
        Data_CloneMesh_ScaleFactor->Text = scaleFactor;
        Data_CloneMesh_RotationX->Text = xRotationFlag;
      }
      else
      {
        Data_CloneMesh_Filename->Text = "";
        Data_CloneMesh_ScaleFactor->Text = "";
        Data_CloneMesh_RotationX->Text = "";
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_Size_GetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double size[3];
      double scaleFactor;

      if (At3d_Get_Size_Clone_Mesh(buffer, &size[0], &scaleFactor) > 0)
      {
        CloneMesh_SizeX->Text = size[0];
        CloneMesh_SizeY->Text = size[1];
        CloneMesh_SizeZ->Text = size[2];

        CloneMesh_Old_ScaleFactor->Text = scaleFactor;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_Size_SetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      double scaleFactor;

      try
      {
        scaleFactor = CloneMesh_New_ScaleFactor->Text.ToDouble();
      }
      catch(...)
      {
        scaleFactor = 1.0;
      }

      if (At3d_Set_ScaleFactor_Clone_Mesh(buffer, scaleFactor) > 0)
      {
        CloneMesh_GetClick(MY_NULL_PTR);
        CloneMesh_Size_GetClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneMesh_RotateX_RotateClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneMesh_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      Int32 xRotationFlag;

      if (CloneMesh_RotateX_Minus90->Checked)
      {
        xRotationFlag = ROTATE_ON_X_AXIS_MINUS_90;
      }
      else if (CloneMesh_RotateX_Plus90->Checked)
      {
        xRotationFlag = ROTATE_ON_X_AXIS_PLUS_90;
      }
      else if (CloneMesh_RotateX_180->Checked)
      {
        xRotationFlag = ROTATE_ON_X_AXIS_180;
      }
      else
      {
        xRotationFlag = ROTATE_ON_X_AXIS_NO;
      }

      if (At3d_RotateX_Clone_Mesh(buffer, xRotationFlag) > 0)
      {
        CloneMesh_GetClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_CloneItem_GetMeshNameClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Int32 itemIndex = CloneMesh_List->ItemIndex;

  if (itemIndex > -1)
  {
    Add_CloneItem_MeshName->Text = CloneMesh_List->Items->Strings[itemIndex];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_CloneItem_Location_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      Add_CloneItem_LocationX->Text = xyzArrayObserver[0];
      Add_CloneItem_LocationY->Text = xyzArrayObserver[1];
      Add_CloneItem_LocationZ->Text = xyzArrayObserver[2];
    }
    else
    {
      Add_CloneItem_LocationX->Text = xyzArray1[0];
      Add_CloneItem_LocationY->Text = xyzArray1[1];
      Add_CloneItem_LocationZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_CloneItem_Location_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    Add_CloneItem_LocationX->Text = xyzArray2[0];
    Add_CloneItem_LocationY->Text = xyzArray2[1];
    Add_CloneItem_LocationZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_CloneItem_AddClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Screen->Cursor = TCursor(crHourGlass);
  Add_Clone_Item();
  Screen->Cursor = TCursor(crDefault);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Add_Clone_Item(void)
{
  if (!dllOpen) return;

  AnsiString tempName;
  my_copystring_to_ansi_from_vcl(&tempName, Add_CloneItem_ItemName->Text);

  char itemNameBuffer[LEN_LAYER_NAME];
  my_strncpy(itemNameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  itemNameBuffer[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(itemNameBuffer);
  Add_CloneItem_ItemName->Text = itemNameBuffer;

  // --

  my_copystring_to_ansi_from_vcl(&tempName, Add_CloneItem_MeshName->Text);

  char meshNameBuffer[LEN_LAYER_NAME];
  my_strncpy(meshNameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
  meshNameBuffer[LEN_LAYER_NAME - 1] = '\0';
  At3d_CheckName(meshNameBuffer);
  Add_CloneItem_MeshName->Text = meshNameBuffer;

  // --

  double location[3];

  try
  {
    location[0] = Add_CloneItem_LocationX->Text.ToDouble();
  }
    catch(...)
  {
    location[0] = 0.0;
  }

  try
  {
    location[1] = Add_CloneItem_LocationY->Text.ToDouble();
  }
  catch(...)
  {
    location[1] = 0.0;
  }

  try
  {
    location[2] = Add_CloneItem_LocationZ->Text.ToDouble();
  }
    catch(...)
  {
    location[2] = 0.0;
  }

  // --

  double rotation[3];

  try
  {
    rotation[0] = Add_CloneItem_RotationX->Text.ToDouble();
  }
  catch(...)
  {
    rotation[0] = 0.0;
  }

  try
  {
    rotation[1] = Add_CloneItem_RotationY->Text.ToDouble();
  }
  catch(...)
  {
    rotation[1] = 0.0;
  }

  try
  {
    rotation[2] = Add_CloneItem_RotationZ->Text.ToDouble();
  }
  catch(...)
  {
    rotation[2] = 0.0;
  }

  // --

  double scaleFactor;

  try
  {
    scaleFactor = Add_CloneItem_ScaleFactor->Text.ToDouble();
  }
  catch(...)
  {
    scaleFactor = 1.0;
  }

  // --

  Int32 visibleFlag = (Add_CloneItem_Visible->Checked) ? 1 : 0;

  // --

  if ( (*itemNameBuffer != '\0') &&
       (*meshNameBuffer != '\0') )
  {
    if (At3d_Add_Clone_Item(itemNameBuffer, meshNameBuffer,
                            &location[0], &rotation[0], scaleFactor, visibleFlag))
    {
      CloneItem_List->Items->Add(itemNameBuffer);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Change_ItemNameClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemNameBuffer[LEN_LAYER_NAME];
    my_strncpy(itemNameBuffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemNameBuffer[LEN_LAYER_NAME - 1] = '\0';

    // --

    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_New_ItemName->Text);

    char newItemName[LEN_LAYER_NAME];
    my_strncpy(newItemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    newItemName[LEN_LAYER_NAME - 1] = '\0';
    At3d_CheckName(newItemName);
    CloneItem_New_ItemName->Text = newItemName;

    if ( (*itemNameBuffer != '\0') &&
         (*newItemName  != '\0') )
    {
      if (At3d_Change_ItemName_Clone_Item(itemNameBuffer, newItemName) > 0)
      {
        CloneItem_List->Items->Strings[itemIndex] = newItemName;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_GetMeshNameClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  CloneItem_MeshName->Text = "";

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemName[LEN_LAYER_NAME];
    my_strncpy(itemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemName[LEN_LAYER_NAME - 1] = '\0';

    if (*itemName != '\0')
    {
      char meshName[LEN_LAYER_NAME];

      if (At3d_Get_Mesh_Clone_Item(itemName, meshName) > 0)
      {
        CloneItem_MeshName->Text = meshName;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Change_MeshNameClick(TObject *Sender)

{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemName[LEN_LAYER_NAME];
    my_strncpy(itemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemName[LEN_LAYER_NAME - 1] = '\0';

    // --

    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_MeshName->Text);

    char meshName[LEN_LAYER_NAME];
    my_strncpy(meshName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    meshName[LEN_LAYER_NAME - 1] = '\0';
    At3d_CheckName(meshName);
    CloneItem_MeshName->Text = meshName;

    if ( (*itemName != '\0') &&
         (*meshName != '\0') )
    {
      if (At3d_Change_Mesh_Clone_Item(itemName, meshName) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_DeleteOneClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Delete_Clone_Item(buffer) > 0)
      {
        CloneItem_List->Items->Delete(itemIndex);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_DeleteAllClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  if (At3d_Delete_All_Clone_Item() > 0)
  {
    CloneItem_List->Clear();
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_GetDataClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemName[LEN_LAYER_NAME];
    my_strncpy(itemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemName[LEN_LAYER_NAME - 1] = '\0';

    if (*itemName != '\0')
    {
      char meshName[LEN_LAYER_NAME];
      Int32 visibleFlag;
      double location[3];
      double rotation[3];
      double scaleFactor;

      if (At3d_Get_Data_Clone_Item(itemName, meshName,
                                   &location[0], &rotation[0],
                                   &scaleFactor, &visibleFlag) > 0)
      {
        Data_CloneItem_MeshName->Text = meshName;

        Data_CloneItem_LocationX->Text = location[0];
        Data_CloneItem_LocationY->Text = location[1];
        Data_CloneItem_LocationZ->Text = location[2];

        Data_CloneItem_RotationX->Text = rotation[0];
        Data_CloneItem_RotationY->Text = rotation[1];
        Data_CloneItem_RotationZ->Text = rotation[2];

        Data_CloneItem_ScaleFactor->Text = scaleFactor;

        Data_CloneItem_Visible->Checked = (visibleFlag) ? 1 : 0;
      }
      else
      {
        Data_CloneItem_MeshName->Text = "";

        Data_CloneItem_LocationX->Text = "";
        Data_CloneItem_LocationY->Text = "";
        Data_CloneItem_LocationZ->Text = "";

        Data_CloneItem_RotationX->Text = "";
        Data_CloneItem_RotationY->Text = "";
        Data_CloneItem_RotationZ->Text = "";

        Data_CloneItem_ScaleFactor->Text = 1.0;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Rotate_RelativeClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_CloneItem_Rotate(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_RotateClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_CloneItem_Rotate(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_CloneItem_Rotate(Int32 relativeFlag)
{
  if (!dllOpen) return;

  double rotate[3];

  try
  {
    rotate[0] = CloneItem_RotateX->Text.ToDouble();
  }
  catch(...)
  {
    rotate[0] = 0.0;
  }

  try
  {
    rotate[1] = CloneItem_RotateY->Text.ToDouble();
  }
  catch(...)
  {
    rotate[1] = 0.0;
  }

  try
  {
    rotate[2] = CloneItem_RotateZ->Text.ToDouble();
  }
  catch(...)
  {
    rotate[2] = 0.0;
  }

  // --

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Rotate_Clone_Item(buffer, relativeFlag, &rotate[0]) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Translate_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      CloneItem_TranslateX->Text = xyzArrayObserver[0];
      CloneItem_TranslateY->Text = xyzArrayObserver[1];
      CloneItem_TranslateZ->Text = xyzArrayObserver[2];
    }
    else
    {
      CloneItem_TranslateX->Text = xyzArray1[0];
      CloneItem_TranslateY->Text = xyzArray1[1];
      CloneItem_TranslateZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Translate_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    CloneItem_TranslateX->Text = xyzArray2[0];
    CloneItem_TranslateY->Text = xyzArray2[1];
    CloneItem_TranslateZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Translate_RelativeClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_CloneItem_Translate(1);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_TranslateClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  Do_CloneItem_Translate(0);
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Do_CloneItem_Translate(Int32 relativeFlag)
{
  if (!dllOpen) return;

  double translate[3];

  try
  {
    translate[0] = CloneItem_TranslateX->Text.ToDouble();
  }
  catch(...)
  {
    translate[0] = 0.0;
  }

  try
  {
    translate[1] = CloneItem_TranslateY->Text.ToDouble();
  }
  catch(...)
  {
    translate[1] = 0.0;
  }

  try
  {
    translate[2] = CloneItem_TranslateZ->Text.ToDouble();
  }
  catch(...)
  {
    translate[2] = 0.0;
  }

  // --

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Translate_Clone_Item(buffer, relativeFlag, &translate[0]) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Size_CloneItem_GetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemName[LEN_LAYER_NAME];
    my_strncpy(itemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemName[LEN_LAYER_NAME - 1] = '\0';

    if (*itemName != '\0')
    {
      double originalSize[3];
      double currentSize[3];
      double scaleFactor;

      if (At3d_Get_Size_Clone_Item(itemName, &originalSize[0],
                                   &currentSize[0], &scaleFactor) > 0)
      {
        Size_CloneItem_OriginalX->Text = originalSize[0];
        Size_CloneItem_OriginalY->Text = originalSize[1];
        Size_CloneItem_OriginalZ->Text = originalSize[2];

        Size_CloneItem_CurrentX->Text = currentSize[0];
        Size_CloneItem_CurrentY->Text = currentSize[1];
        Size_CloneItem_CurrentZ->Text = currentSize[2];

        Size_CloneItem_Old_ScaleFactor->Text = scaleFactor;
      }
      else
      {
        Size_CloneItem_OriginalX->Text = "";
        Size_CloneItem_OriginalY->Text = "";
        Size_CloneItem_OriginalZ->Text = "";

        Size_CloneItem_CurrentX->Text = "";
        Size_CloneItem_CurrentY->Text = "";
        Size_CloneItem_CurrentZ->Text = "";

        Size_CloneItem_Old_ScaleFactor->Text = "";
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Size_CloneItem_SetClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemName[LEN_LAYER_NAME];
    my_strncpy(itemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemName[LEN_LAYER_NAME - 1] = '\0';

    if (*itemName != '\0')
    {
      double scaleFactor;

      try
      {
        scaleFactor = Size_CloneItem_New_ScaleFactor->Text.ToDouble();
      }
      catch(...)
      {
        scaleFactor = 1.0;
      }

      if (At3d_Set_ScaleFactor_Clone_Item(itemName, scaleFactor) > 0)
      {
        Size_CloneItem_GetClick(MY_NULL_PTR);
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Snap_CloneItem_From_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      Snap_CloneItem_FromX->Text = xyzArrayObserver[0];
      Snap_CloneItem_FromY->Text = xyzArrayObserver[1];
      Snap_CloneItem_FromZ->Text = xyzArrayObserver[2];
    }
    else
    {
      Snap_CloneItem_FromX->Text = xyzArray1[0];
      Snap_CloneItem_FromY->Text = xyzArray1[1];
      Snap_CloneItem_FromZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Snap_CloneItem_From_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    Snap_CloneItem_FromX->Text = xyzArray2[0];
    Snap_CloneItem_FromY->Text = xyzArray2[1];
    Snap_CloneItem_FromZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Snap_CloneItem_To_V1Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    if ( (haveObserverFlag) &&
         (hitFlagObserver == MESH_LOCATION_HAS_XYZ) )
    {
      Snap_CloneItem_ToX->Text = xyzArrayObserver[0];
      Snap_CloneItem_ToY->Text = xyzArrayObserver[1];
      Snap_CloneItem_ToZ->Text = xyzArrayObserver[2];
    }
    else
    {
      Snap_CloneItem_ToX->Text = xyzArray1[0];
      Snap_CloneItem_ToY->Text = xyzArray1[1];
      Snap_CloneItem_ToZ->Text = xyzArray1[2];
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Snap_CloneItem_To_V2Click(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (hitFlag1 == MESH_LOCATION_HAS_XYZ) &&
       (hitFlag2 == MESH_LOCATION_HAS_XYZ) )
  {
    Snap_CloneItem_ToX->Text = xyzArray2[0];
    Snap_CloneItem_ToY->Text = xyzArray2[1];
    Snap_CloneItem_ToZ->Text = xyzArray2[2];
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::Snap_CloneItem_SnapClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char itemName[LEN_LAYER_NAME];
    my_strncpy(itemName, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    itemName[LEN_LAYER_NAME - 1] = '\0';

    if (*itemName != '\0')
    {
      double fromLocation[3];

      try
      {
        fromLocation[0] = Snap_CloneItem_FromX->Text.ToDouble();
      }
      catch(...)
      {
        fromLocation[0] = 0.0;
      }

      try
      {
        fromLocation[1] = Snap_CloneItem_FromY->Text.ToDouble();
      }
      catch(...)
      {
        fromLocation[1] = 0.0;
      }

      try
      {
        fromLocation[2] = Snap_CloneItem_FromZ->Text.ToDouble();
      }
      catch(...)
      {
        fromLocation[2] = 0.0;
      }

      // --

      double toLocation[3];

      try
      {
        toLocation[0] = Snap_CloneItem_ToX->Text.ToDouble();
      }
      catch(...)
      {
        toLocation[0] = 0.0;
      }

      try
      {
        toLocation[1] = Snap_CloneItem_ToY->Text.ToDouble();
      }
      catch(...)
      {
        toLocation[1] = 0.0;
      }

      try
      {
        toLocation[2] = Snap_CloneItem_ToZ->Text.ToDouble();
      }
      catch(...)
      {
        toLocation[2] = 0.0;
      }

      // --

      if (At3d_Snap_Clone_Item(itemName, &fromLocation[0], &toLocation[0]) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Visible_OnClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Set_Visible_Clone_Item(buffer, 1) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::CloneItem_Visible_OffClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  Int32 itemIndex = CloneItem_List->ItemIndex;

  if (itemIndex > -1)
  {
    AnsiString tempName;
    my_copystring_to_ansi_from_vcl(&tempName, CloneItem_List->Items->Strings[itemIndex]);

    char buffer[LEN_LAYER_NAME];
    my_strncpy(buffer, LEN_LAYER_NAME, tempName.c_str(), LEN_LAYER_NAME - 1);
    buffer[LEN_LAYER_NAME - 1] = '\0';

    if (*buffer != '\0')
    {
      if (At3d_Set_Visible_Clone_Item(buffer, 0) > 0)
      {
        //
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::ReadClonesFileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (clonesFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(clonesFilename, MAX_PATH, "%s%sClones-%s.txt", drive0, dir0, name0);
  }

  if (clonesFilename[0] != '\0')
  {
    my_splitpath(clonesFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Clones";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TOpenDialog* dlg;
  try
  {
    dlg = new TOpenDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofFileMustExist << ofHideReadOnly << ofPathMustExist;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  Int32 result = 0;
  Int32 oldMeshCount = At3d_Get_Count_Clone_Mesh();
  Int32 oldItemCount = At3d_Get_Count_Clone_Item();

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(clonesFilename, MAX_PATH, fileName.c_str());


    result = At3d_Read_Clones_File(clonesFilename);
  }

  delete dlg;

  if (result)
  {
    Int32 newMeshCount = At3d_Get_Count_Clone_Mesh();
    Int32 newItemCount = At3d_Get_Count_Clone_Item();

    char name[LEN_LAYER_NAME];

    if (newMeshCount > oldMeshCount)
    {
      for (Int32 i=oldMeshCount; i<newMeshCount; i++)
      {
        if (At3d_Get_Name_Clone_Mesh(i, name) > 0)
        {
          CloneMesh_List->Items->Add(name);
        }
      }
    }

    if (newItemCount > oldItemCount)
    {
      for (Int32 i=oldItemCount; i<newItemCount; i++)
      {
        if (At3d_Get_Name_Clone_Item(i, name) > 0)
        {
          CloneItem_List->Items->Add(name);
        }
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::SaveClonesFileClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if ( (!dllOpen) ||
       (!fileLoaded) )
  {
    return;
  }

  // --

  char path[_MAX_PATH];
  char drive0[_MAX_DRIVE];
  char dir0[_MAX_DIR];
  char name0[_MAX_FNAME];
  char ext0[_MAX_EXT];

  if ( (clonesFilename[0] == '\0') &&
       (inName[0] != '\0') )
  {
    my_splitpath(inName, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_sprintf(clonesFilename, MAX_PATH, "%s%sClones-%s.txt", drive0, dir0, name0);
  }

  if (clonesFilename[0] != '\0')
  {
    my_splitpath(clonesFilename, drive0, _MAX_DRIVE, dir0, _MAX_DIR, name0, _MAX_FNAME,
                 ext0, _MAX_EXT);

    my_strcpy(path, _MAX_PATH, drive0);
    my_strcat(path, _MAX_PATH, dir0);

    my_strcat(name0, _MAX_FNAME, ".txt");
  }
  else
  {
    path[0]  = '\0';
    name0[0] = '\0';
  }

  const char* inputNamePtr = "Clones";
  const char* inputExtensionPtr = ".txt";

  AnsiString filter = inputNamePtr;
  filter += "|*";
  filter += inputExtensionPtr;
  filter += "|All|*.*|";

  TSaveDialog* dlg;
  try
  {
    dlg = new TSaveDialog(MY_NULL_PTR);
  }
  catch(...)
  {
    return;
  }

  dlg->Options.Clear();
  dlg->Options = dlg->Options << ofNoReadOnlyReturn << ofPathMustExist << ofOverwritePrompt;

  dlg->InitialDir  = path;
  dlg->FileName    = name0;
  dlg->Filter      = filter;
  dlg->FilterIndex = 0;

  if (dlg->Execute() == IDOK)
  {
    AnsiString fileName;

    my_copystring_to_ansi_from_vcl(&fileName, dlg->FileName);
    my_strcpy(clonesFilename, MAX_PATH, fileName.c_str());


    At3d_Save_Clones_File(clonesFilename);
  }

  delete dlg;
}
//---------------------------------------------------------------------------

void __fastcall TTest_At3dx_LocationsForm::DisplayClonesClick(TObject *Sender)
{
  sender_W8057_Flag = (Sender) ? 1 : 0;

  // --

  if (!dllOpen) return;

  At3d_WriteFlag(LIBFLAG_DISPLAY_CLONES, (DisplayClones->Checked) ? 1 : 0);
}
//---------------------------------------------------------------------------

